; $Id$
;
; Copyright (c) 2000-2004, Research Systems, Inc.  All rights reserved.
;   Unauthorized reproduction prohibited.
;----------------------------------------------------------------------------
;+
; CLASS_NAME:
;   IDLitToolbase
;
; PURPOSE:
;   This file implements the BT7 PSD Tool object
;
; CATEGORY:
;   
;
; SUPERCLASSES:
;   IDLitTool
;
; SUBCLASSES:
;
; CREATION:
;   See IDLitToolbase::Init
;
; METHODS:
;   This class has the following methods:
;
;   IDLitToolbase::Init
;
; INTERFACES:
; IIDLProperty
;-



;===============================================================================
; Bt7psdTool::GetSystem
; 
; PURPOSE:
;   Return the IDLitSystem object
;
; PARAMETERS:
;
; KEYWORDS:
;
function Bt7psdTool::GetSystem
compile_opt idl2

return, self->_GetSystem()

end


;===============================================================================
; Bt7psdTool::GetUI
; 
; PURPOSE:
;   Return the IDLitUI object for this tool
;
; PARAMETERS:
;
; KEYWORDS:
;
function Bt7psdTool::GetUI
compile_opt idl2

return, self._oUIConnection

end


;-------------------------------------------------------------------------------
pro Bt7psdTool::Cleanup
compile_opt idl2

; Basic error Handler
if (n_elements(!debug) && (!debug eq 0)) then begin
    catch, catchError
    if (catchError ne 0) then begin
        ;;print, 'Error handled!'
        eTitle = 'Bt7psdTool::Cleanup: Error encountered'
        eMsg = 'An error or unusual condition was encountered!'
        eMsg = [eMsg,'Please, report the following to the DAVE team:']
        eMsg = [eMsg,!error_state.msg]
        void = dialog_message(/error,eMsg,title=eTitle)

        catch, /cancel
        ptr_free, Self.prefsPtr
        ptr_free, Self.historyPtr
        Self->IDLitTool::Cleanup
        Self->EnableUpdates
        return
    endif
endif

Self->DisableUpdates

; save session preferences
status = Self->SavePreferences()

ptr_free, Self.prefsPtr
ptr_free, Self.historyPtr

; Clean up samp container
oItems = Self.sampContRef->Get(/all, count=n)
if (n gt 0) then begin
   for i=0,n-1 do begin
      void = oItems[i]->GetMetaData('DATAPTRREF',dataPtr)
      if (ptr_valid(dataPtr)) then heap_free, dataPtr
   endfor
endif

; Clean up samp container
oItems = Self.bkgdContRef->Get(/all, count=n)
if (n gt 0) then begin
   for i=0,n-1 do begin
      void = oItems[i]->GetMetaData('DATAPTRREF',dataPtr)
      if (ptr_valid(dataPtr)) then heap_free, dataPtr
   endfor
endif


obj_destroy, [Self.sampContRef,Self.bkgdContRef]

; Call base class cleanup
self->IDLitTool::Cleanup

Self->EnableUpdates

end
;-------------------------------------------------------------------------------


;===============================================================================
;+
; Bt7psdTool::savePreferences
;
; PURPOSE:
;   Update and save current user preferences
;
; PARAMETERS:
;
; RETURN VALUE:
;-
function Bt7psdTool::SavePreferences, skip_filesave=skip_filesave
compile_opt idl2

oUI = Self->GetUI()
oUI->GetProperty, group_leader = wTLB
; Basic error Handler
if (n_elements(!debug) && (!debug eq 0)) then begin
    catch, catchError
    if (catchError ne 0) then begin
        ;;print, 'Error handled!'
        eTitle = 'Bt7psdTool::savePreferences: Error encountered'
        eMsg = 'An error or unusual condition was encountered!'
        eMsg = [eMsg,'Please, report the following to the DAVE team:']
        eMsg = [eMsg,!error_state.msg]
        void = dialog_message(/error,eMsg,title=eTitle, dialog_parent=wTLB)
        catch, /cancel
        return, 0
    endif
endif

; Save current user graphic customizations and preferences
if (Self->HasVisualizations()) then begin
   oWin = Self->GetCurrentWindow()
   oView = oWin->GetCurrentView()
   oLayer = oView->GetCurrentLayer()
   oWorld = oLayer->GetWorld()
   oDataSpace = oWorld->GetDataSpaces()
   oVis = oDataSpace->GetVisualizations(count = nVis)
   
   for j=nVis-1,0,-1 do begin
      oVis[j]->Select
      Self->saveVisProps, oVis[j]
   endfor
endif

Self->GetProperty, preferences=preferences

if keyword_set(skip_filesave) then return, 1

if (n_elements(preferences) gt 0) then $
   status = BT7PSDReducPreferences(preferences, /save)

return, 1

end
;-------------------------------------------------------------------------------


;===============================================================================
; Bt7psdTool::Exit
;+
; PURPOSE:
;   Exit handler
;
; PARAMETERS:
;
; RETURN VALUE:
;-
function Bt7psdTool::Exit
compile_opt idl2

oUI = Self->GetUI()
oUI->GetProperty, group_leader = wTLB
; Basic error Handler
if (n_elements(!debug) && (!debug eq 0)) then begin
    catch, catchError
    if (catchError ne 0) then begin
        ;;print, 'Error handled!'
        eTitle = 'Bt7psdTool::Exit: Error encountered'
        eMsg = 'An error or unusual condition was encountered!'
        eMsg = [eMsg,'Please, report the following to the DAVE team:']
        eMsg = [eMsg,!error_state.msg]
        void = dialog_message(/error,eMsg,title=eTitle, dialog_parent=wTLB)
        catch, /cancel
        return, 0
    endif
endif

oUI = Self->GetUI()
oUI->GetProperty, group_leader = wTLB
wd_Bt7psdTool_event, { WIDGET_KILL_REQUEST, ID:wTLB, TOP:wTLB, HANDLER:wTLB }

return, 1
end


;-------------------------------------------------------------------------------
pro Bt7psdTool::GetProperty, nameTag=nameTag, prompt=prompt, prmptTitle=prmptTitle, prmptDesc=prmptDesc $
 ,nActiveDetectors=nActiveDetectors,energyUnitFlag=energyUnitFlag $
 ,maskedDets=maskedDets,fastFunc=fastFunc,monScaleValue=monScaleValue,multipleMsliceWinFlag=multipleMsliceWinFlag $
 ,monScaleFlag=monScaleFlag,maskFlag=maskFlag,bkgdFlag=bkgdFlag,calibParamsisSet=calibParamsisSet $
 ,fastFlag=fastFlag,nSamp=nSamp,nBkgd=nBkgd,nFast=nFast,effNormFlag=effNormFlag,logFlag=logFlag $
 ,sumSampFlag=sumSampFlag,sumBkgdFlag=sumBkgdFlag, chDelA4isSet=chDelA4isSet, curMaxDataLen=curMaxDataLen $
 ,sampContRef=sampContRef,bkgdContRef=bkgdContRef,fastContRef=fastContRef,phiOffset=phiOffset $
 ,data_directory=dataDir,mev2wnos=mev2wnos,fastparams=fastparams, daveTool=daveTool $
 ,sampSelected=sampSelected, bkgdSelected=bkgdSelected, fastSelected=fastSelected $
 ,bkgdSubVarFlag=bkgdSubVarFlag,bkgdSubVarTol=bkgdSubVarTol $
 ,constBinFlag=constBinFlag,useTempAverageFlag=useTempAverageFlag $
 ,preferences=preferences,history=history,oplotFastFitFlag=oplotFastFitFlag $
 ,minChan=minChan,maxChan=maxChan,minDPnt=minDPnt,maxDPnt=maxDPnt,dpntIndex=dpntIndex $
 ,sharedSettingsFlag=sharedSettingsFlag,settingIndex=settingIndex $
 ,oplotBkgdFlag=oplotBkgdFlag, modifiedStatus=modifiedStatus, ftpObject=ftpObject $
 ,defaultIntensityFlag=defaultIntensityFlag, maxIntensity=maxIntensity, minIntensity=minIntensity $
 ,latParamFlag=latParamFlag, latticeParameters=latticeParameters, qUnits=qUnits $
 ,xAxisVar=xAxisVar,yAxisVar=yAxisVar,minBinWidth=minBinWidth,binwidths=binWidths $
 ,exportIndex1=exportIndex1,exportIndex2=exportIndex2,exportIndex3=exportIndex3 $
 ,indepVar1List=indepVar1List,indepVar2List=indepVar2List,indepVar3List=indepVar3List $
 ,detailedBalanceFlag=detailedBalanceFlag,fastValue=fastValue, resCorFlag=resCorFlag $
 ,monCorFlag=monCorFlag, normalizeTo=normalizeTo ,mergeChannelsFlag=mergeChannelsFlag $
 ,sampTypeFlag=sampTypeFlag, sampTyps=sampTyps $
 ,selectedIDs=selectedIDs,nSelectedIDs=nSelectedIDs,selectedNames=selectedNames  $
 ,s_errorbar_capsize=s_errorbar_capsize $
 ,s_transparency=s_transparency $
 ,s_antialias=s_antialias $
 ,s_color=s_color $
 ,s_errorbar_color=s_errorbar_color $
 ,s_use_default_color=s_use_default_color $
 ,s_sym_increment=s_sym_increment $
 ,s_sym_index=s_sym_index $
 ,s_sym_size=s_sym_size $
 ,s_sym_color=s_sym_color $
 ,s_sym_thick=s_sym_thick $
 ,s_y_errorbars=s_y_errorbars $
 ,s_linestyle=s_linestyle $
 ,s_thick=s_thick $
 ,s_sym_filled=s_sym_filled $
 ,s_sym_fill_color=s_sym_fill_color $
 ,s_font_name=s_font_name  $
 ,s_font_size=s_font_size $
 ,s_text_color=s_text_color  $
 ,b_errorbar_capsize=b_errorbar_capsize $
 ,b_transparency=b_transparency $
 ,b_antialias=b_antialias $
 ,b_color=b_color $
 ,b_errorbar_color=b_errorbar_color $
 ,b_use_default_color=b_use_default_color $
 ,b_sym_increment=b_sym_increment $
 ,b_sym_index=b_sym_index $
 ,b_sym_size=b_sym_size $
 ,b_sym_color=b_sym_color $
 ,b_sym_thick=b_sym_thick $
 ,b_y_errorbars=b_y_errorbars $
 ,b_linestyle=b_linestyle $
 ,b_thick=b_thick $
 ,b_sym_filled=b_sym_filled $
 ,b_sym_fill_color=b_sym_fill_color $
 ,b_font_name=b_font_name  $
 ,b_font_size=b_font_size $
 ,b_text_color=b_text_color  $
 ,f_errorbar_capsize=f_errorbar_capsize $
 ,f_transparency=f_transparency $
 ,f_antialias=f_antialias $
 ,f_color=f_color $
 ,f_errorbar_color=f_errorbar_color $
 ,f_use_default_color=f_use_default_color $
 ,f_sym_increment=f_sym_increment $
 ,f_sym_index=f_sym_index $
 ,f_sym_size=f_sym_size $
 ,f_sym_color=f_sym_color $
 ,f_sym_thick=f_sym_thick $
 ,f_y_errorbars=f_y_errorbars $
 ,f_linestyle=f_linestyle $
 ,f_thick=f_thick $
 ,f_sym_filled=f_sym_filled $
 ,f_sym_fill_color=f_sym_fill_color $
 ,f_font_name=f_font_name  $
 ,f_font_size=f_font_size $
 ,f_text_color=f_text_color  $
 ,ff_transparency=ff_transparency $
 ,ff_antialias=ff_antialias $
 ,ff_color=ff_color $
 ,ff_use_default_color=ff_use_default_color $
 ,ff_sym_index=ff_sym_index $
 ,ff_linestyle=ff_linestyle $
 ,ff_thick=ff_thick $
 ,sb_transparency=sb_transparency $
 ,sb_antialias=sb_antialias $
 ,sb_color=sb_color $
 ,sb_use_default_color=sb_use_default_color $
 ,sb_sym_index=sb_sym_index $
 ,sb_linestyle=sb_linestyle $
 ,sb_thick=sb_thick $
 ,_REF_EXTRA=etc

compile_opt idl2

; daveTool
if (arg_present(daveTool)) then $
  daveTool = self.daveTool

if (arg_present(s_errorbar_capsize)) then s_errorbar_capsize=Self.s_errorbar_capsize
if (arg_present(s_transparency)) then s_transparency=Self.s_transparency
if (arg_present(s_antialias)) then s_antialias=Self.s_antialias
if (arg_present(s_color)) then s_color=Self.s_color
if (arg_present(s_errorbar_color)) then s_errorbar_color=Self.s_errorbar_color
if (arg_present(s_use_default_color)) then s_use_default_color=Self.s_use_default_color 
if (arg_present(s_sym_increment)) then s_sym_increment=Self.s_sym_increment 
if (arg_present(s_sym_index)) then s_sym_index=Self.s_sym_index 
if (arg_present(s_sym_size)) then s_sym_size=Self.s_sym_size 
if (arg_present(s_sym_color)) then s_sym_color=Self.s_sym_color 
if (arg_present(s_sym_thick)) then s_sym_thick=Self.s_sym_thick 
if (arg_present(s_y_errorbars)) then s_y_errorbars=Self.s_y_errorbars
if (arg_present(s_linestyle)) then s_linestyle=Self.s_linestyle
if (arg_present(s_thick)) then s_thick=Self.s_thick
if (arg_present(s_sym_filled)) then s_sym_filled=Self.s_sym_filled
if (arg_present(s_sym_fill_color)) then s_sym_fill_color=Self.s_sym_fill_color
if (arg_present(s_font_name)) then s_font_name=Self.s_font_name
if (arg_present(s_font_size)) then s_font_size=Self.s_font_size
if (arg_present(s_text_color)) then s_text_color=Self.s_text_color
if (arg_present(b_errorbar_capsize)) then b_errorbar_capsize=Self.b_errorbar_capsize
if (arg_present(b_transparency)) then b_transparency=Self.b_transparency
if (arg_present(b_antialias)) then b_antialias=Self.b_antialias
if (arg_present(b_color)) then b_color=Self.b_color
if (arg_present(b_errorbar_color)) then b_errorbar_color=Self.b_errorbar_color
if (arg_present(b_use_default_color)) then b_use_default_color=Self.b_use_default_color
if (arg_present(b_sym_increment)) then b_sym_increment=Self.b_sym_increment
if (arg_present(b_sym_index)) then b_sym_index=Self.b_sym_index
if (arg_present(b_sym_size)) then b_sym_size=Self.b_sym_size
if (arg_present(b_sym_color)) then b_sym_color=Self.b_sym_color
if (arg_present(b_sym_thick)) then b_sym_thick=Self.b_sym_thick
if (arg_present(b_y_errorbars)) then b_y_errorbars=Self.b_y_errorbars
if (arg_present(b_linestyle)) then b_linestyle=Self.b_linestyle
if (arg_present(b_thick)) then b_thick=Self.b_thick
if (arg_present(b_sym_filled)) then b_sym_filled=Self.b_sym_filled
if (arg_present(b_sym_fill_color)) then b_sym_fill_color=Self.b_sym_fill_color
if (arg_present(b_sym_fill_color)) then b_errorbar_capsize=Self.b_errorbar_capsize
if (arg_present(b_font_name)) then b_font_name=Self.b_font_name
if (arg_present(b_font_size)) then b_font_size=Self.b_font_size
if (arg_present(b_text_color)) then b_text_color=Self.b_text_color
if (arg_present(f_transparency)) then f_transparency=Self.f_transparency
if (arg_present(f_antialias)) then f_antialias=Self.f_antialias
if (arg_present(f_color)) then f_color=Self.f_color
if (arg_present(f_errorbar_color)) then f_errorbar_color=Self.f_errorbar_color
if (arg_present(f_use_default_color)) then f_use_default_color=Self.f_use_default_color
if (arg_present(f_sym_increment)) then f_sym_increment=Self.f_sym_increment
if (arg_present(f_sym_index)) then f_sym_index=Self.f_sym_index
if (arg_present(f_sym_size)) then f_sym_size=Self.f_sym_size
if (arg_present(f_sym_color)) then f_sym_color=Self.f_sym_color
if (arg_present(f_sym_thick)) then f_sym_thick=Self.f_sym_thick
if (arg_present(f_y_errorbars)) then f_y_errorbars=Self.f_y_errorbars
if (arg_present(f_linestyle)) then f_linestyle=Self.f_linestyle
if (arg_present(f_thick)) then f_thick=Self.f_thick
if (arg_present(f_sym_filled)) then f_sym_filled=Self.f_sym_filled
if (arg_present(f_sym_fill_color)) then f_sym_fill_color=Self.f_sym_fill_color
if (arg_present(f_font_name)) then f_font_name=Self.f_font_name
if (arg_present(f_font_size)) then f_font_size=Self.f_font_size
if (arg_present(f_text_color)) then f_text_color=Self.f_text_color
if (arg_present(ff_transparency)) then ff_transparency=Self.ff_transparency
if (arg_present(ff_antialias)) then ff_antialias=Self.ff_antialias
if (arg_present(ff_color)) then ff_color=Self.ff_color
if (arg_present(ff_use_default_color)) then ff_use_default_color=Self.ff_use_default_color
if (arg_present(ff_sym_index)) then ff_sym_index=Self.ff_sym_index
if (arg_present(ff_linestyle)) then ff_linestyle=Self.ff_linestyle
if (arg_present(ff_thick)) then ff_thick=Self.ff_thick
if (arg_present(sb_transparency)) then sb_transparency=Self.sb_transparency
if (arg_present(sb_antialias)) then sb_antialias=Self.sb_antialias
if (arg_present(sb_color)) then sb_color=Self.sb_color
if (arg_present(sb_use_default_color)) then sb_use_default_color=Self.sb_use_default_color
if (arg_present(sb_sym_index)) then sb_sym_index=Self.sb_sym_index
if (arg_present(sb_linestyle)) then sb_linestyle=Self.sb_linestyle
if (arg_present(sb_thick)) then sb_thick=Self.sb_thick


; Preferences structure
if (arg_present(preferences)) then begin
   if (ptr_valid(Self.prefsPtr)) then begin
      preferences = (*self.prefsPtr)

      ; update the preferences structure
      ntags = n_tags(preferences)
      tags = tag_names(preferences)
      for i=0,ntags-1 do begin
         status = Self->GetPropertyByIdentifier(tags[i], value)
         if (status) then preferences.(i) = strjoin(strtrim(string(value),2),',')
;         if (~status) then continue
;         if (strmatch(tags[i],'*color*',/fold_case)) then begin
;            ; for colors, first convert byte vector to a string eg [0,128,0] to '0,128,0'
;            preference.(i) = strjoin(value,',')
;         endif else $
;            preferences.(i) = value
      endfor
   endif
endif

; modifiedStatus
if (arg_present(modifiedStatus)) then modifiedStatus = self.modifiedFlag
; nameTag
if (arg_present(nameTag)) then nameTag = self.nameTag
; nameTag
if (arg_present(history)) then history = (*Self.historyPtr)
; prompt
if (arg_present(prompt)) then prompt = self.prompt
; prompt Title
if (arg_present(prmptTitle)) then prmptTitle = self.promptTitle
; prompt Desc
if (arg_present(prmptDesc)) then prmptDesc = self.promptDesc
;
if (arg_present(constBinFlag)) then constBinFlag = Self.constBinFlag
;
if (arg_present(useTempAverageFlag)) then useTempAverageFlag = Self.useTempAverageFlag
;
if (arg_present(nSamp)) then nSamp = Self.nSamp
;
if (arg_present(nBkgd)) then nBkgd = Self.nBkgd
;
if (arg_present(nFast)) then nFast = Self.nFast
;
if (arg_present(sampSelected)) then sampSelected = Self.sampSelected
;
if (arg_present(bkgdSelected)) then bkgdSelected = Self.bkgdSelected
;
if (arg_present(fastSelected)) then fastSelected = Self.fastSelected
;
if (arg_present(sampContRef)) then sampContRef = Self.sampContRef
;
if (arg_present(bkgdContRef)) then bkgdContRef = Self.bkgdContRef
;
if (arg_present(fastContRef)) then fastContRef = Self.fastContRef
;
if (arg_present(nActiveDetectors)) then nActiveDetectors = Self.nActiveDetectors
;
if (arg_present(energyUnitFlag)) then energyUnitFlag = Self.energyUnitFlag
;
if (arg_present(maskedDets)) then maskedDets = Self.maskedDets
;
if (arg_present(fastparams)) then fastparams = Self.fastparams
;
if (arg_present(fastfunc)) then fastfunc = Self.fastfunc
;
if (arg_present(funcs)) then funcs = Self.fastfuncs
;
if (arg_present(monScaleValue)) then monScaleValue = Self.monScaleValue
;
if (arg_present(monScaleFlag)) then monScaleFlag = Self.monScaleFlag
;
if (arg_present(sumSampFlag)) then sumSampFlag = Self.sumSampFlag
;
if (arg_present(sumBkgdFlag)) then sumBkgdFlag = Self.sumBkgdFlag
;
if (arg_present(maskFlag)) then maskFlag = Self.maskFlag
;
if (arg_present(bkgdFlag)) then bkgdFlag = Self.bkgdFlag
;
if (arg_present(bkgdSubVarFlag)) then bkgdSubVarFlag = Self.bkgdSubVarFlag
;
if (arg_present(bkgdSubVarTol)) then bkgdSubVarTol = Self.bkgdSubVarTol
;
if (arg_present(fastFlag)) then fastFlag = Self.fastFlag
;
if (arg_present(dataDir)) then dataDir = Self.dataDir
;
if (arg_present(mev2wnos)) then mev2wnos = Self.mev2wnos
;
if (arg_present(oplotFastFitFlag)) then oplotFastFitFlag = Self.oplotFastFitFlag
;
if (arg_present(oplotBkgdFlag)) then oplotBkgdFlag = Self.oplotBkgdFlag
;
if (arg_present(ftpObject)) then ftpObject = Self.ftpObject
;
if (arg_present(latParamFlag)) then latParamFlag = Self.latParamFlag
;
if (arg_present(latticeParameters)) then latticeParameters = Self.latticeParameters
;
if (arg_present(detailedBalanceFlag)) then detailedBalanceFlag = Self.detailedBalanceFlag
;
if (arg_present(fastValue)) then fastValue = Self.fastValue
;
if (arg_present(resCorFlag)) then resCorFlag = Self.resCorFlag
;
if (arg_present(monCorFlag)) then monCorFlag = Self.monCorFlag
;
if (arg_present(normalizeTo)) then normalizeTo = Self.normalizeTo
;
if (arg_present(sampTypeFlag)) then sampTypeFlag = Self.sampTypeFlag
;
if (arg_present(sampTyps)) then sampTyps = (*Self.sampTyps)
;
if (arg_present(qUnits)) then qUnits = Self.qUnits
;
if (arg_present(xAxisVar)) then xAxisVar = Self.xAxisVar
;
if (arg_present(yAxisVar)) then yAxisVar = Self.yAxisVar
;
if (arg_present(indepVar1List)) then indepVar1List = (*Self.indepVars1Ptr)
;
if (arg_present(indepVar12List)) then indepVar2List = (*Self.indepVars2Ptr)
;
if (arg_present(indepVar2List)) then indepVar2List = (*Self.indepVars2Ptr)
;
if (arg_present(indepVar3List)) then indepVar3List = (*Self.indepVars3Ptr)
;
if (arg_present(exportIndex1)) then exportIndex1 = (*Self.exportIndex1Ptr)
;
if (arg_present(exportIndex2)) then exportIndex2 = (*Self.exportIndex2Ptr)
;
if (arg_present(exportIndex3)) then exportIndex3 = (*Self.exportIndex3Ptr)
;
if (arg_present(calibParamsisSet)) then calibParamsisSet = Self.calibParamsisSet
;
if (ARG_PRESENT(multipleMsliceWinFlag)) then multipleMsliceWinFlag = Self.multipleMsliceWinFlag
;
if (arg_present(effNormFlag)) then effNormFlag = Self.effNormFlag
;
if (ARG_PRESENT(mergeChannelsFlag)) then mergeChannelsFlag = Self.mergeChannelsFlag
;
if (ARG_PRESENT(minBinWidth)) then minBinWidth = Self.minBinWidth
;
if (ARG_PRESENT(binWidths)) then binWidths = Self.binWidths
;
if (ARG_PRESENT(defaultIntensityFlag)) then defaultIntensityFlag = Self.defaultIntensityFlag
;
if (ARG_PRESENT(maxIntensity)) then maxIntensity = Self.maxIntensity
;
if (ARG_PRESENT(minIntensity)) then minIntensity = Self.minIntensity
;
if (ARG_PRESENT(phiOffset)) then phiOffset = Self.phiOffset
;
if (ARG_PRESENT(logFlag)) then logFlag = Self.logFlag
;
if (ARG_PRESENT(minChan)) then minChan = Self.minChan
;
if (ARG_PRESENT(maxChan)) then maxChan = Self.maxChan
;
if (ARG_PRESENT(minDPnt)) then minDPnt = Self.minDPnt
;
if (ARG_PRESENT(maxDPnt)) then maxDPnt = Self.maxDPnt
;
if (ARG_PRESENT(sharedSettingsFlag)) then sharedSettingsFlag = Self.sharedSettingsFlag
;
if (ARG_PRESENT(settingIndex)) then settingIndex = Self.settingIndex
;
if (ARG_PRESENT(dpntIndex)) then dpntIndex = Self.dpntIndex
;
if (arg_present(selectedIDs) || arg_present(nSelectedIDs) || arg_present(selectedNames)) then begin
   oUI = Self->GetUI()
   oUI->GetProperty, group_leader = wTLB
   wChild = WIDGET_INFO(wTLB, /CHILD)
   WIDGET_CONTROL, wChild, GET_UVALUE=sPtr
   ; retrieve list of all _selected_ datasets from the data manager
   wDM = (*sPtr).wDM ; the data manager widget ID
   widget_control, wDM, get_uvalue=sPtrDM ; the data manager state ptr
   
   oSampCont = Self.sampContRef
   if (oSampCont->Count() eq 0) then begin
      ; no data currently loaded
      (*(*sPtrDM).pdataids) = ''
      selectedIDs = ''
      selectedNames = ''
      nSelectedIDs = 0
   endif else begin
      selectedIDs = (*(*sPtrDM).pdataids)  ; currently selected identifiers
      nSelectedIDs = n_elements(selectedIDs)
      if (selectedIDs[0] eq '') then nSelectedIDs = 0
      selectedNames = (nSelectedIDs eq 0)? '' :  strarr(nSelectedIDs)
      for i=0,nSelectedIDs-1 do begin
         (Self->GetByIdentifier(selectedIDs[i]))->GetProperty, name=name
         selectedNames[i] = name
      endfor
   endelse
endif

if (arg_present(curMaxDataLen)) then begin
   oSampCont = Self.sampContRef
   curMaxDataLen = 1
   if (obj_valid(oSampCont) && (oSampCont->Count() gt 0)) then begin
      nSamp = oSampCont->Count()
      for i = 0, nSamp-1 do begin
         oI = oSampCont->Get(position=i)
         status = oI->GetMetaData('NPTS',npts)
         if (status) then curMaxDataLen = curMaxDataLen > npts
      endfor
   endif 

endif

; Call base class method
if (n_elements(etc) gt 0) then $
  self->IDLitTool::GetProperty, _EXTRA=etc

end


;-------------------------------------------------------------------------------
pro Bt7psdTool::SetProperty, nameTag=nameTag, prompt=prompt,preferences=preferences, _EXTRA=etc $
 ,nActiveDetectors=nActiveDetectors,energyUnitFlag=energyUnitFlag $
 ,maskedDets=maskedDets,fastFunc=fastFunc,monScaleValue=monScaleValue $
 ,monScaleFlag=monScaleFlag,maskFlag=maskFlag,bkgdFlag=bkgdFlag $
 ,sumSampFlag=sumSampFlag,sumBkgdFlag=sumBkgdFlag,effNormFlag=effNormFlag $
 ,fastFlag=fastFlag, data_directory=dataDir, daveTool=daveTool $
 ,bkgdSubVarFlag=bkgdSubVarFlag,bkgdSubVarTol=bkgdSubVarTol $
 ,constBinFlag=constBinFlag,useTempAverageFlag=useTempAverageFlag $
 ,minChan=minChan,maxChan=maxChan,minDPnt=minDPnt,maxDPnt=maxDPnt,dpntIndex=dpntIndex $
 ,sharedSettingsFlag=sharedSettingsFlag,settingIndex=settingIndex $
 ,nSamp=nSamp,nBkgd=nBkgd,nFast=nFast,fastparams=fastparams,phiOffset=phiOffset,logFlag=logFlag $
 ,defaultIntensityFlag=defaultIntensityFlag, maxIntensity=maxIntensity, minIntensity=minIntensity $
 ,sampSelected=sampSelected, bkgdSelected=bkgdSelected, fastSelected=fastSelected $
 ,oplotFastFitFlag=oplotFastFitFlag,oplotBkgdFlag=oplotBkgdFlag, typeCheck=typeCheck $
 ,modifiedStatus=modifiedStatus,minBinWidth=minBinWidth,binWidths=binWidths $
 ,exportIndex1=exportIndex1,exportIndex2=exportIndex2,exportIndex3=exportIndex3 $
 ,latParamFlag=latParamFlag, latticeParameters=latticeParameters, qUnits=qUnits $
 ,xAxisVar=xAxisVar,yAxisVar=yAxisVar,multipleMsliceWinFlag=multipleMsliceWinFlag $
 ,detailedBalanceFlag=detailedBalanceFlag,fastValue=fastValue, resCorFlag=resCorFlag $
 ,monCorFlag=monCorFlag, normalizeTo=normalizeTo, sampTypeFlag=sampTypeFlag,mergeChannelsFlag=mergeChannelsFlag $
 ,s_errorbar_capsize=s_errorbar_capsize $
 ,s_transparency=s_transparency $
 ,s_antialias=s_antialias $
 ,s_color=s_color $
 ,s_errorbar_color=s_errorbar_color $
 ,s_use_default_color=s_use_default_color $
 ,s_sym_increment=s_sym_increment $
 ,s_sym_index=s_sym_index $
 ,s_sym_size=s_sym_size $
 ,s_sym_color=s_sym_color $
 ,s_sym_thick=s_sym_thick $
 ,s_y_errorbars=s_y_errorbars $
 ,s_linestyle=s_linestyle $
 ,s_thick=s_thick $
 ,s_sym_filled=s_sym_filled $
 ,s_sym_fill_color=s_sym_fill_color $
 ,s_font_name=s_font_name  $
 ,s_font_size=s_font_size $
 ,s_text_color=s_text_color  $
 ,b_errorbar_capsize=b_errorbar_capsize $
 ,b_transparency=b_transparency $
 ,b_antialias=b_antialias $
 ,b_color=b_color $
 ,b_errorbar_color=b_errorbar_color $
 ,b_use_default_color=b_use_default_color $
 ,b_sym_increment=b_sym_increment $
 ,b_sym_index=b_sym_index $
 ,b_sym_size=b_sym_size $
 ,b_sym_color=b_sym_color $
 ,b_sym_thick=b_sym_thick $
 ,b_y_errorbars=b_y_errorbars $
 ,b_linestyle=b_linestyle $
 ,b_thick=b_thick $
 ,b_sym_filled=b_sym_filled $
 ,b_sym_fill_color=b_sym_fill_color $
 ,b_font_name=b_font_name  $
 ,b_font_size=b_font_size $
 ,b_text_color=b_text_color  $
 ,f_errorbar_capsize=f_errorbar_capsize $
 ,f_transparency=f_transparency $
 ,f_antialias=f_antialias $
 ,f_color=f_color $
 ,f_errorbar_color=f_errorbar_color $
 ,f_use_default_color=f_use_default_color $
 ,f_sym_increment=f_sym_increment $
 ,f_sym_index=f_sym_index $
 ,f_sym_size=f_sym_size $
 ,f_sym_color=f_sym_color $
 ,f_sym_thick=f_sym_thick $
 ,f_y_errorbars=f_y_errorbars $
 ,f_linestyle=f_linestyle $
 ,f_thick=f_thick $
 ,f_sym_filled=f_sym_filled $
 ,f_sym_fill_color=f_sym_fill_color $
 ,f_font_name=f_font_name  $
 ,f_font_size=f_font_size $
 ,f_text_color=f_text_color  $
 ,ff_transparency=ff_transparency $
 ,ff_antialias=ff_antialias $
 ,ff_color=ff_color $
 ,ff_use_default_color=ff_use_default_color $
 ,ff_sym_index=ff_sym_index $
 ,ff_linestyle=ff_linestyle $
 ,ff_thick=ff_thick $
 ,sb_transparency=sb_transparency $
 ,sb_antialias=sb_antialias $
 ,sb_color=sb_color $
 ,sb_use_default_color=sb_use_default_color $
 ,sb_sym_index=sb_sym_index $
 ,sb_linestyle=sb_linestyle $
 ,sb_thick=sb_thick

compile_opt idl2

if n_elements(s_errorbar_capsize) then Self.s_errorbar_capsize=s_errorbar_capsize
if n_elements(s_transparency) then Self.s_transparency=s_transparency
if n_elements(s_antialias) then Self.s_antialias=s_antialias
if n_elements(s_color) then Self.s_color=s_color
if n_elements(s_errorbar_color) then Self.s_errorbar_color=s_errorbar_color
if n_elements(s_use_default_color) then Self.s_use_default_color=s_use_default_color 
if n_elements(s_sym_increment) then Self.s_sym_increment=s_sym_increment 
if n_elements(s_sym_index) then Self.s_sym_index=s_sym_index 
if n_elements(s_sym_size) then Self.s_sym_size=s_sym_size 
if n_elements(s_sym_color) then Self.s_sym_color=s_sym_color 
if n_elements(s_sym_thick) then Self.s_sym_thick=s_sym_thick 
if n_elements(s_y_errorbars) then Self.s_y_errorbars=s_y_errorbars
if n_elements(s_linestyle) then Self.s_linestyle=s_linestyle
if n_elements(s_thick) then Self.s_thick=s_thick
if n_elements(s_sym_filled) then Self.s_sym_filled=s_sym_filled
if n_elements(s_sym_fill_color) then Self.s_sym_fill_color=s_sym_fill_color
if n_elements(s_font_name) then Self.s_font_name=s_font_name
if n_elements(s_font_size) then Self.s_font_size=s_font_size
if n_elements(s_text_color) then Self.s_text_color=s_text_color
if n_elements(b_errorbar_capsize) then Self.b_errorbar_capsize=b_errorbar_capsize
if n_elements(b_transparency) then Self.b_transparency=b_transparency
if n_elements(b_antialias) then Self.b_antialias=b_antialias
if n_elements(b_color) then Self.b_color=b_color
if n_elements(b_use_default_color) then Self.b_use_default_color=b_use_default_color
if n_elements(b_errorbar_color) then Self.b_errorbar_color=b_errorbar_color
if n_elements(b_sym_increment) then Self.b_sym_increment=b_sym_increment
if n_elements(b_sym_index) then Self.b_sym_index=b_sym_index
if n_elements(b_sym_size) then Self.b_sym_size=b_sym_size
if n_elements(b_sym_color) then Self.b_sym_color=b_sym_color
if n_elements(b_sym_thick) then Self.b_sym_thick=b_sym_thick
if n_elements(b_y_errorbars) then Self.b_y_errorbars=b_y_errorbars
if n_elements(b_linestyle) then Self.b_linestyle=b_linestyle
if n_elements(b_thick) then Self.b_thick=b_thick
if n_elements(b_sym_filled) then Self.b_sym_filled=b_sym_filled
if n_elements(b_sym_fill_color) then Self.b_sym_fill_color=b_sym_fill_color
if n_elements(b_font_name) then Self.b_font_name=b_font_name
if n_elements(b_font_size) then Self.b_font_size=b_font_size
if n_elements(b_text_color) then Self.b_text_color=b_text_color
if n_elements(f_errorbar_capsize) then Self.f_errorbar_capsize=f_errorbar_capsize
if n_elements(f_transparency) then Self.f_transparency=f_transparency
if n_elements(f_antialias) then Self.f_antialias=f_antialias
if n_elements(f_color) then Self.f_color=f_color
if n_elements(f_errorbar_color) then Self.f_errorbar_color=f_errorbar_color
if n_elements(f_use_default_color) then Self.f_use_default_color=f_use_default_color
if n_elements(f_sym_increment) then Self.f_sym_increment=f_sym_increment
if n_elements(f_sym_index) then Self.f_sym_index=f_sym_index
if n_elements(f_sym_size) then Self.f_sym_size=f_sym_size
if n_elements(f_sym_color) then Self.f_sym_color=f_sym_color
if n_elements(f_sym_thick) then Self.f_sym_thick=f_sym_thick
if n_elements(f_y_errorbars) then Self.f_y_errorbars=f_y_errorbars
if n_elements(f_linestyle) then Self.f_linestyle=f_linestyle
if n_elements(f_thick) then Self.f_thick=f_thick
if n_elements(f_sym_filled) then Self.f_sym_filled=f_sym_filled
if n_elements(f_sym_fill_color) then Self.f_sym_fill_color=f_sym_fill_color
if n_elements(f_font_name) then Self.f_font_name=f_font_name
if n_elements(f_font_size) then Self.f_font_size=f_font_size
if n_elements(f_text_color) then Self.f_text_color=f_text_color
if n_elements(ff_transparency) then Self.ff_transparency=ff_transparency
if n_elements(ff_antialias) then Self.ff_antialias=ff_antialias
if n_elements(ff_color) then Self.ff_color=ff_color
if n_elements(ff_use_default_color) then Self.ff_use_default_color=ff_use_default_color
if n_elements(ff_sym_index) then Self.ff_sym_index=ff_sym_index
if n_elements(ff_linestyle) then Self.ff_linestyle=ff_linestyle
if n_elements(ff_thick) then Self.ff_thick=ff_thick
if n_elements(sb_transparency) then Self.sb_transparency=sb_transparency
if n_elements(sb_antialias) then Self.sb_antialias=sb_antialias
if n_elements(sb_color) then Self.sb_color=sb_color
if n_elements(sb_use_default_color) then Self.sb_use_default_color=sb_use_default_color
if n_elements(sb_sym_index) then Self.sb_sym_index=sb_sym_index
if n_elements(sb_linestyle) then Self.sb_linestyle=sb_linestyle
if n_elements(sb_thick) then Self.sb_thick=sb_thick

; DAVETool
if (n_elements(daveTool) && obj_valid(daveTool) && obj_isa(daveTool,'DAVETool')) then $
  Self.daveTool = daveTool

; Preferences
if (n_elements(preferences)) then begin
   if (strcmp(size(preferences,/tname),'STRUCT')) then begin
      ntags = n_tags(preferences)
      tags = tag_names(preferences)
      index = indgen(ntags)
      
      ; handle color tags - convert string to byte array, eg '0,128,255' to [0,128,255]
      spIndex = where(strmatch(tags,'*color*',/fold) eq 1, spCnt, complement=index, ncomplement=theRest)
      if (spCnt gt 0) then begin
         for i=0,spCnt-1 do begin
            strColor = preferences.(spIndex[i])
            bytColor = fix(strsplit(strColor,',',/extract))
            Self->SetPropertyByIdentifier, tags[spIndex[i]], bytColor
         endfor
      endif

      ; all other tags
      for i=0,theRest-1 do begin
         if (strcmp(tags[index[i]],'binWidths',/fold)) then begin
            ; another special case (binwidths) - convert string to 7 element float array
            strBW = preferences.(index[i])
            fltBW = float(strsplit(strBW,',',/extract))
            Self->SetPropertyByIdentifier, tags[index[i]], fltBW
         endif else $
            Self->SetPropertyByIdentifier, tags[index[i]], preferences.(index[i])
      endfor
      
      ; Need further to assign some userdef attribute for user-defined properties
      ; in order for property sheet to reflect updated value
      uProps = ['MASKEDDETS']
      n = n_elements(uProps)
      for i = 0,n-1 do begin
         index = where(strcmp(tags,uProps[i]), found)
         if (found) then $
            Self->SetPropertyAttribute, uProps[i],userdef=preferences.(index)
      endfor
      
      ; Finally, store the preferences structure
      if (ptr_valid(Self.prefsPtr)) then (*Self.prefsPtr) = preferences $
      else Self.prefsPtr = ptr_new(preferences)
   endif
endif

; modifiedStatus
if (n_elements(modifiedStatus)) then Self.modifiedFlag = (fix(modifiedStatus) gt 0)? 1 : 0
; nameTag
if (n_elements(nameTag)) then Self.nameTag = strtrim(nameTag,2)
; prompt
if (n_elements(prompt)) then begin
   Self.prompt = prompt
   if (prompt eq 0) then Self._bDirty = 0
endif

;
if (n_elements(constBinFlag)) then Self.constBinFlag = constBinFlag
;
if (n_elements(useTempAverageFlag)) then Self.useTempAverageFlag = useTempAverageFlag
;
if (n_elements(nActiveDetectors)) then Self.nActiveDetectors = nActiveDetectors
;
if (n_elements(energyUnitFlag)) then Self.energyUnitFlag = energyUnitFlag
;
if (n_elements(maskedDets)) then Self.maskedDets = maskedDets
;
if (n_elements(fastfunc)) then Self.fastfunc = fastfunc
;
if (n_elements(fastparams)) then Self.fastparams = fastparams
;
if (n_elements(monScaleValue)) then Self.monScaleValue = (monScaleValue eq 0.0)? 1.0 : monScaleValue
;
if (n_elements(monScaleFlag)) then Self.monScaleFlag = monScaleFlag
;
if (n_elements(sumSampFlag)) then Self.sumSampFlag = sumSampFlag
;
if (n_elements(sumBkgdFlag)) then Self.sumBkgdFlag = sumBkgdFlag
;
if (n_elements(maskFlag)) then Self.maskFlag = maskFlag
;
if (n_elements(bkgdFlag)) then Self.bkgdFlag = bkgdFlag
;
if (n_elements(bkgdSubVarFlag)) then Self.bkgdSubVarFlag = bkgdSubVarFlag
;
if (n_elements(bkgdSubVarTol)) then Self.bkgdSubVarTol = bkgdSubVarTol
;
if (n_elements(fastFlag)) then Self.fastFlag = fastFlag
;
if (n_elements(dataDir)) then Self.dataDir = dataDir
;
if (n_elements(nSamp)) then Self.nSamp = nSamp
;
if (n_elements(nBkgd)) then Self.nBkgd = nBkgd
;
if (n_elements(nFast)) then Self.nFast = nFast
;
if (n_elements(sampSelected)) then Self.sampSelected = sampSelected
;
if (n_elements(bkgdSelected)) then Self.bkgdSelected = bkgdSelected
;
if (n_elements(fastSelected)) then Self.fastSelected = fastSelected
;
if (n_elements(oplotFastFitFlag)) then Self.oplotFastFitFlag = oplotFastFitFlag
;
if (n_elements(oplotBkgdFlag)) then Self.oplotBkgdFlag = oplotBkgdFlag
;
if (n_elements(latParamFlag)) then  Self.latParamFlag = latParamFlag
;
if (n_elements(latticeParameters)) then  Self.latticeParameters = latticeParameters
;
if (n_elements(detailedBalanceFlag)) then Self.detailedBalanceFlag = detailedBalanceFlag
;
if (n_elements(fastValue)) then Self.fastValue = fastValue
;
if (n_elements(resCorFlag)) then Self.resCorFlag = resCorFlag
;
if (n_elements(monCorFlag)) then Self.monCorFlag = monCorFlag
;
if (n_elements(normalizeTo)) then Self.normalizeTo = normalizeTo
;
if (n_elements(minbinWidth)) then Self.minbinWidth = minbinWidth
;
if (n_elements(binWidths)) then Self.binWidths = binWidths
;
if (n_elements(minChan)) then Self.minChan = minChan > 1
;
if (n_elements(maxChan)) then Self.maxChan = maxChan
;
if (n_elements(minDPnt)) then Self.minDPnt = minDPnt > 1
;
if (n_elements(maxDPnt)) then Self.maxDPnt = maxDPnt
;
if (n_elements(dpntIndex)) then Self.dpntIndex = dpntIndex > 1
;
if (n_elements(sharedSettingsFlag)) then Self.sharedSettingsFlag = sharedSettingsFlag
;
if (n_elements(settingIndex)) then Self.settingIndex = settingIndex
;
if (n_elements(sampTypeFlag)) then begin
   Self.sampTypeFlag = sampTypeFlag
   if (n_elements(typeCheck) eq 0) then typeCheck = 1 ; ==> default typeCheck is set to 1
   Self->UpdatePropertySheet, typeCheck = typeCheck
endif
;
if (n_elements(qUnits)) then Self.qUnits = qUnits
;
if (n_elements(xAxisVar)) then Self.xAxisVar = xAxisVar
;
if (n_elements(yAxisVar)) then Self.yAxisVar = yAxisVar
;
if (n_elements(exportIndex1) gt 0) then (*Self.exportIndex1Ptr) = exportIndex1
;
if (n_elements(exportIndex2) gt 0) then (*Self.exportIndex2Ptr) = exportIndex2
;
if (n_elements(exportIndex3) gt 0) then (*Self.exportIndex3Ptr) = exportIndex3
;
if (n_elements(effNormFlag) gt 0) then Self.effNormFlag = effNormFlag 
;
if (N_ELEMENTS(multipleMsliceWinFlag) gt 0) then Self.multipleMsliceWinFlag = multipleMsliceWinFlag
;
if (N_ELEMENTS(mergeChannelsFlag) gt 0) then Self.mergeChannelsFlag = mergeChannelsFlag
;
if (N_ELEMENTS(defaultIntensityFlag) gt 0) then Self.defaultIntensityFlag = defaultIntensityFlag
;
if (N_ELEMENTS(maxIntensity) gt 0) then begin
   if (maxIntensity gt Self.minIntensity) then Self.maxIntensity = maxIntensity
endif
;
if (N_ELEMENTS(minIntensity) gt 0) then begin
   if (minIntensity lt Self.maxIntensity) then Self.minIntensity = minIntensity
endif
;
if (N_ELEMENTS(phiOffset) gt 0) then Self.phiOffset = phiOffset
;
if (N_ELEMENTS(logFlag) gt 0) then Self.logFlag = logFlag





; Call base class method
if (n_elements(etc) gt 0) then $
  self->IDLitTool::SetProperty, _EXTRA=etc

end


;===============================================================================
; Bt7psdTool::deleteAllDatasets
;
; PURPOSE:
;   Delete all datasets currently loaded
;
; PARAMETERS:
;
; RETURN VALUE:
;
function Bt7psdTool::deleteAllDatasets, _EXTRA=etc
compile_opt idl2

oUI = Self->GetUI()
oUI->GetProperty, group_leader = wTLB
wChild = widget_info(wTLB, /child)
widget_control, wChild, get_uvalue=sPtr

wDM = (*sPtr).wDM
widget_control, wDM, get_uvalue=sPtrDM

cw_bT7PSDDataManager_DeleteData, sPtrDM, wDM, /delall, _EXTRA=etc

return, 1
end
;-------------------------------------------------------------------------------


;===============================================================================
; Bt7psdTool::ReloadAllDatasets
;
; PURPOSE:
;   Reload all datasets currently loaded
;
; PARAMETERS:
;
; RETURN VALUE:
;
function Bt7psdTool::ReloadAllDatasets, _EXTRA=etc
compile_opt idl2

oBItems = Self.bkgdContRef->Get(/all, count=nBkgd)
oSItems = Self.sampContRef->Get(/all, count=nSamp)

; reload bkgd data, if any
; NB: reload bkgd first before samp!
if (nBkgd gt 0) then begin
  for i=0,nBkgd-1 do status = Self->LoadBkgd(dataObject=oBItems[i], /reload)
  ; if no samp present then refresh display with bkgd data
  if (nSamp le 0) then wd_BT7PSDTool_sendPlotEvent, Self->GetUI(), oBItems[0]
endif

; reload samp data, if any
if (nSamp gt 0) then begin
  for i=0,nSamp-1 do status = Self->LoadSamp(dataObject=oSItems[i], /reload)
  wd_BT7PSDTool_sendPlotEvent, Self->GetUI(), oSItems[0]
endif

return, 1
end
;-------------------------------------------------------------------------------


;===============================================================================
; Bt7psdTool::loadSamp
;
; PURPOSE:
;   Specify and load sample data file(s)
;
; PARAMETERS:
;
; KEYWORDS:
;
function Bt7psdTool::GetFileList, title, count, fromFTP=fromFTP, filter=filter $
                                ,multiple_files=multiple_files, path=path, newPath=newPath

oUI = Self->GetUI()
oUI->GetProperty, group_leader = wTLB

if (keyword_set(fromFTP)) then begin
  Self->GetProperty, ftpObject=oFTP
  if (n_elements(filter) eq 0) then filter = ['*.bt7','*.*']
  files = Dialog_NCNRpublicData(oFTP,title=title,filter=filter,group_leader=wTLB,count=count)
endif else begin
  if (n_elements(filter) eq 0) then filter = '*.bt7'
  if (n_elements(multiple_files) eq 0) then multiple_files=1
  if (n_elements(path) eq 0) then Self->GetProperty, data_directory=path
  if (n_elements(title) eq 0) then title='Specify files to load'
  
  files = dialog_pickfile(title=title,multiple_files=multiple_files,/must_exist, dialog_parent=wTLB $
                         ,filter=filter,path=path, get_path=newPath)
  
  if (files[0] eq '') then begin
     count = 0
     return, ''
  endif
  
  count = n_elements(files)
  ;Self->SetProperty, data_directory=newPath
endelse

return, files

end
;-------------------------------------------------------------------------------



;===============================================================================
; Bt7psdTool::refreshPropertySheet
;
; PURPOSE:
;   Refresh the propertysheet UI
;
; PARAMETERS:
;
; KEYWORDS:
;
pro Bt7psdTool::RefreshPropertySheet

oUI = Self->GetUI()
oUI->GetProperty, group_leader = wTLB
wChild = widget_info(wTLB, /child)
widget_control, wChild, get_uvalue=sPtr

widget_control, (*sPtr).wPS, /refresh_property

end
;-------------------------------------------------------------------------------


;===============================================================================
; Bt7psdTool::loadFastFtp
;
; PURPOSE:
;   Specify and load fast bkgd data file(s) from NCNR ftp server
;
; PARAMETERS:
;
; KEYWORDS:
;
function Bt7psdTool::loadFastFtp

return, Self->LoadFast(/fromFtp)

end
;-------------------------------------------------------------------------------


;===============================================================================
; Bt7psdTool::loadBkgdFtp
;
; PURPOSE:
;   Specify and load bkgd data file(s) from NCNR ftp server
;
; PARAMETERS:
;
; KEYWORDS:
;
function Bt7psdTool::loadBkgdFtp

return, Self->LoadBkgd(/fromFtp)

end
;-------------------------------------------------------------------------------


;===============================================================================
;+
; Bt7psdTool::loadBkgd
;
; PURPOSE:
;   Select and load Bkgd data file(s)
;
; PARAMETERS:
;
; KEYWORDS:
; 
;   dataObject - specifies the data object whose data is to be reloaded
;   
;   reload  - set if the data specified by the dataObject keyword is to be reloaded
;   
;   fromFTP - set if the source of the datafile to be loaded is the FTP server
;-
function Bt7psdTool::loadBkgd, reload=reload, dataObject=oItem, fromFTP=fromFTP

oUI = Self->GetUI()
oUI->GetProperty, group_leader = wTLB

; Basic error Handler
if (n_elements(!debug) && (!debug eq 0)) then begin
  catch, catchError
  if (catchError ne 0) then begin
    ;;print, 'Error handled!'
    eTitle = 'Bt7psdTool::loadBkgd: Error encountered'
    eMsg = 'An error or unusual condition was encountered!'
    eMsg = [eMsg,'Please, report the following to the DAVE team:']
    eMsg = [eMsg,!error_state.msg]
    void = dialog_message(/error,eMsg,title=eTitle, dialog_parent=wTLB)
    catch, /cancel
    Self->EnableUpdates
    return, 0
  endif
endif
  
reloading = keyword_set(reload)
if (reloading) then begin
   if (~obj_valid(oItem)) then return, 0
   oItem->GetProperty, description=files
   count = 1
   void = oItem->GetMetaData('FROMFTP',fromFtp)
endif else begin
   ; Let user specify files to load
   title = 'Select Background Files to Load'
   files = Self->GetFileList(title,count,fromFTP=fromFTP)
   if (count eq 0) then return, 0
endelse

; switch on the busy symbol for cursor
widget_control, /HOURGLASS
  
Self->DisableUpdates
; Read the contents of the data files and make entries in the 'Background' tree of the data manager
Self->GetProperty, sumBkgdFlag=sumFlag, bkgdContRef=oCont, nBkgd=nData
for i=0, count-1 do begin

  filename = files[i]
  if (~Self->readData(filename, dataPtr, fromFTP=fromFTP)) then continue
  
  basename = file_basename(filename,'.bt7',/fold_case)
  
  if (~reloading) then begin
    ;; create new object and attach dataPtr
    oItem = obj_new('IDLitParameterSet',name=basename,description=filename,type='bkgd')
    oItem->AddMetaData,'DATAPTRREF',dataPtr
    oItem->AddMetaData,'FROMFTP',keyword_set(fromFTP)
    
    ; iTools does not handle '_[0-9]' characters in identifiers well eg _0, _1, etc
    ; replace all '_' with '-'
    oItem->GetProperty, identifier=idd
    hasChanged = 0
    while ((pos = stregex(idd,'_[0-9]')) ne -1) do begin
      strput, idd,'-',pos
      hasChanged = 1
    endwhile
    if (strmatch(idd,'[0-9]*')) then begin   ; if first char is a number
      idd = 'id'+idd
      hasChanged = 1
    endif
    if (hasChanged) then oItem->SetProperty,identifier=idd
    
    ; Add to the data manager
    Self->AddByIdentifier, 'Bkgd Data Manager', oItem, position=0
  endif else begin
    ;; simply replace previous dataPtr with updated one
    void = oItem->GetMetaData('DATAPTRREF',tempPtr)
    if (ptr_valid(tempPtr)) then heap_free, tempPtr
    oItem->AddMetaData,'DATAPTRREF',dataPtr
  endelse
  
  ; Update calculations (q,e,intensity) for data
  Self->UpdateCalculation, oItem
  
  if (~reloading) then begin
    oCont->Add, oItem
    Self.nBkgd++
  endif
  
  dataPtr = 0   ; make dataPtr invalid before it is used again when looping
endfor

Self.modifiedFlag = 1

; Since bkgd data has changed, trigger an update so this change can be reflected.
Self->UpdateCounts

Self->EnableUpdates

;Self.nBkgd = oCont->Count()

;Self->SetPropertyAttribute, 'SUMBKGDFLAG', sensitive=(Self.nBkgd gt 0)
Self->SetPropertyAttribute, 'BKGDFLAG', sensitive=(Self.nBkgd gt 0)
Self->SetPropertyAttribute, 'bkgdSubVarFlag', sensitive=(Self.bkgdFlag eq 1)
Self->SetPropertyAttribute, 'bkgdSubVarTol', sensitive=(Self.bkgdFlag eq 1)

Self->UpdatePropertySheet
Self->RefreshPropertySheet

return, 1

end
;-------------------------------------------------------------------------------



;===============================================================================
; SPINSpsdTool::ViewCalibParams
;
; PURPOSE:
;   View raw contents of loaded PSD detector calibration parameter file
;   (Pixel efficiencies and delta A4, Ef).
;
; PARAMETERS:
;
; KEYWORDS:
;
function Bt7psdTool::ViewCalibParams

  oUI = Self->GetUI()
  oUI->GetProperty, group_leader = wTLB

  status = file_test(Self.calibParamsFilename,/read)
  if (~status) then begin
    msg = ['The following calibration file could not be located',Self.calibParamsFilename]
    Self->ErrorMessage, msg,title='Calibration not found',severity=0
    Self->StatusMessage, Self.calibParamsFilename+' could not be located'
    return, 0
  endif

  font = (!version.os_family eq 'Windows')? 'Courier New*24*Bold' : $
    '-adobe-courier-bold-o-normal--24-240-75-75-m-150-iso8859-1'
  font1 = (!version.os_family eq 'Windows')? 'Courier New*18*Bold' : $
    '-adobe-courier-bold-o-normal--18-180-75-75-m-110-iso8859-1'
  font2 = (!version.os_family eq 'Windows')? 'Courier New*16' : $
    '-adobe-courier-medium-o-normal--14-140-75-75-m-90-iso8859-1'

  doneTxt = "Close "+Self.calibParamsBaseFilename

  xdisplayfile, Self.calibParamsFilename, done_button=doneTxt, /grow_to_screen, group=wTLB, font=font2

  return, 1

end
;-------------------------------------------------------------------------------


;===============================================================================
; Bt7psdTool::loadCalibParams
;
; PURPOSE:
;   Specify calibration file and load PSD detector calibration parameters 
;   (channel efficiencies and delta A4, Ef) values from it
;
; PARAMETERS:
;
; KEYWORDS:
;
function Bt7psdTool::loadCalibParams

; Basic error Handler
if (n_elements(!debug) && (!debug eq 0)) then begin
    catch, catchError
    if (catchError ne 0) then begin
        ;;print, 'Error handled!'
        eTitle = 'Bt7psdTool::loadCalibParams: Error encountered'
        eMsg = 'An error or unusual condition was encountered!'
        eMsg = [eMsg,'Please, report the following to the DAVE team:']
        eMsg = [eMsg,!error_state.msg]
        void = dialog_message(/error,eMsg,title=eTitle, dialog_parent=wTLB)
        catch, /cancel
        if (n_elements(lun) gt 0) then free_lun, lun, /force
        return, 0
    endif
endif

; Let user specify file to load
title = 'Select PSD Calibration Parameter file to load'
etc = {filter:'*',multiple_files:0}
calidFileNameExists = (strlen(Self.calibParamsFilename) gt 0) && $
                       file_test(Self.calibParamsFilename)
if (calidFileNameExists) then $
  etc = create_struct(etc,'path',file_dirname(Self.calibParamsFilename))
filename = Self->GetFileList(title,count,newPath=newPath,_EXTRA=etc)
if (count eq 0) then return, 0
if (n_elements(newPath) gt 0) then Self->SetProperty, data_directory=newPath
lines = ''
openr, lun, filename, /get_lun
readf, lun, lines
isCalib = stregex(lines[0],'BT7 PSD Calibration',/fold,/bool)
if (~isCalib) then begin
  free_lun, lun, /force
  msg = [Filename, 'is not a proper BT7 calibration parameter file']
  Self->ErrorMessage, msg,title='Invalid file',severity=1
  return, 0
endif
nlines = 1
while (~EOF(lun)) do begin
  readf, lun, lines
  if (~strcmp(lines,'#',1)) then break
  nlines++
endwhile
point_lun, lun, 0 ; rewind to beginning of file

lines = strarr(nlines)
buf = fltarr(3,48)
;if (isCalib) then begin
readf, lun, lines
readf, lun, buf
free_lun, lun, /force

Self.chEff = reform(buf[0,*])
Self.chDelA4 = reform(buf[1,*])
Self.chDelEf = reform(buf[2,*])
Self.calibParamsIsSet = 1
Self.calibParamsFilename = filename
Self.calibParamsBaseFilename = file_basename(filename)
 
; parse efficiencies for channels to be masked by default - will have negative efficiencies
maskIndex = where(stregex(lines,'Channels to be masked',/fold,/bool) eq 1, maskFound)
index = where(Self.chEff lt 0.0, nMask)
if (nMask gt 0) then begin
   status = intVector2CompactString(index+1,sIndex)   ; index+1 converts indices to channel number
   Self->SetProperty, MASKEDDETS=sIndex
   Self->SetPropertyAttribute, 'MASKEDDETS', userdef=sIndex, sensitive=1
   Self->SetProperty, MASKFLAG=1
endif else if (maskFound) then begin
  ; check if there are channels to be masked in header section
  ; this is the new way of specifying channels for masking
  toks = strsplit(lines[maskIndex[0]],':',/extract,count=ntoks)
  if (ntoks eq 2 && strtrim(toks[1]) ne '') then begin
    Self->SetProperty, MASKEDDETS=strtrim(toks[1])
    Self->SetPropertyAttribute, 'MASKEDDETS', userdef=strtrim(toks[1]), sensitive=1
    Self->SetProperty, MASKFLAG=1
  endif
endif
 
Self->SetPropertyAttribute, 'calibParamsBaseFilename', userdef=Self.calibParamsBaseFilename
Self->SetPropertyAttribute, 'effNormFlag', sensitive=1
 
Self->refreshPropertySheet
;endif else begin
;   free_lun, lun, /force
;   msg = [Filename, 'is not a proper BT7 calibration parameter file']
;   Self->ErrorMessage, msg,title='Invalid file',severity=1
;   return, 0
;endelse

; re-evaluate calculated quantities for already loaded datasets
Self->GetProperty, sampContRef=oCont
oData = oCont->Get(/all,count=nData)
for i = 0, nData-1 do Self->UpdateCalculation, oData[i]

; initiate a plot event to update any plotted data
wd_Bt7psdTool_sendPlotEvent, Self->GetUI()

return, 1

end
;-------------------------------------------------------------------------------


;===============================================================================
; Bt7psdTool::loadSampFtp
;
; PURPOSE:
;   Specify and load sample data file(s) from NCNR ftp server
;
; PARAMETERS:
;
; KEYWORDS:
;
function Bt7psdTool::loadSampFtp

return, Self->LoadSamp(/fromFtp)

end
;-------------------------------------------------------------------------------


;===============================================================================
;+
; Bt7psdTool::loadSamp
;
; PURPOSE:
;   Specify and load sample data file(s)
;
; PARAMETERS:
;
; KEYWORDS:
; 
;   dataObject - specifies the data object whose data is to be reloaded
;   
;   reload  - set if the data specified by the dataObject keyword is to be reloaded
;   
;   fromFTP - set if the source of the datafile to be loaded is the FTP server
;-
function Bt7psdTool::loadSamp, reload=reload, dataObject=oItem, fromFTP=fromFTP

oUI = Self->GetUI()
oUI->GetProperty, group_leader = wTLB

; Basic error Handler
if (n_elements(!debug) && (!debug eq 0)) then begin
    catch, catchError
    if (catchError ne 0) then begin
        ;;print, 'Error handled!'
        eTitle = 'Bt7psdTool::loadSamp: Error encountered'
        eMsg = 'An error or unusual condition was encountered!'
        eMsg = [eMsg,'Please, report the following to the DAVE team:']
        eMsg = [eMsg,!error_state.msg]
        void = dialog_message(/error,eMsg,title=eTitle, dialog_parent=wTLB)
        catch, /cancel
        Self->EnableUpdates
        return, 0
    endif
endif

reloading = keyword_set(reload)
if (reloading) then begin
   if (~obj_valid(oItem)) then return, 0
   oItem->GetProperty, description=files
   count = 1
   void = oItem->GetMetaData('FROMFTP',fromFtp)
   ;if (~Self->ReadData(filename, dataPtr)) then return, 0
endif else begin
   ; Let user specify files to load
   title = 'Select Sample Files to Load'
   
   files = Self->GetFileList(title,count,fromFTP=fromFTP,newPath=newPath)
   if (count eq 0) then return, 0
   if (n_elements(newPath) gt 0) then Self->SetProperty, data_directory=newPath
endelse

; switch on the busy symbol for cursor
widget_control, /HOURGLASS

Self->DisableUpdates
; Read the contents of the data files and make entries in the 'Sample' tree of the data manager
Self->GetProperty, sumSampFlag=sumFlag, sampContRef=oCont, nSamp=nData
for i=0, count-1 do begin

   filename = files[i]
   if (~Self->readData(filename, dataPtr, fromFTP=fromFTP)) then continue

   basename = file_basename(filename,'.bt7',/fold_case)

   if (~reloading) then begin
      ;; create new object and attach dataPtr
      oItem = obj_new('IDLitParameterSet',name=basename,description=filename,type='samp')   
      oItem->AddMetaData,'DATAPTRREF',dataPtr
      oItem->AddMetaData,'FROMFTP',keyword_set(fromFTP)
      
      ; iTools does not handle '_[0-9]' characters in identifiers well eg _0, _1, etc
      ; replace all '_' with '-'
      oItem->GetProperty, identifier=idd
      hasChanged = 0
      while ((pos = stregex(idd,'_[0-9]')) ne -1) do begin
         strput, idd,'-',pos
         hasChanged = 1
      endwhile
      if (strmatch(idd,'[0-9]*')) then begin   ; if first char is a number
         idd = 'id'+idd
         hasChanged = 1
      endif
      if (hasChanged) then oItem->SetProperty,identifier=idd 

      ; Add to the data manager
      Self->AddByIdentifier, 'Sample Data Manager', oItem, position=0
   endif else begin
      ;; simply replace previous dataPtr with updated one
      void = oItem->GetMetaData('DATAPTRREF',tempPtr)
      if (ptr_valid(tempPtr)) then heap_free, tempPtr
      oItem->AddMetaData,'DATAPTRREF',dataPtr
   endelse
   
   ; Update calculations (q,e,intensity) for data
   Self->UpdateCalculation, oItem
   
   if (~reloading) then begin
      oCont->Add, oItem
      Self.nSamp++
   endif

   dataPtr = 0   ; make dataPtr invalid before it is used again when looping
endfor
; TODO
;if ((Self.nSamp gt 1) && sumFlag) then Self->MergeData, 'samp'
;
;Self->SetPropertyAttribute, 'SUMSAMPFLAG', sensitive=(Self.nSamp gt 0)
;Self->refreshPropertySheet   ; force prop sheet to update

Self.modifiedFlag = 1
Self->EnableUpdates

nSamp = oCont->Count()
wChild = WIDGET_INFO(wTLB, /CHILD)
WIDGET_CONTROL, wChild, GET_UVALUE=sPtr
widget_control, (*sPtr).wMslice, sensitive=((nSamp gt 0) && (Self.sampTypeFlag lt 2) )
;widget_control, (*sPtr).wDaveDM, sensitive=(nSamp gt 0)
;widget_control, (*sPtr).wPAN, sensitive=(nSamp gt 0)

Self->UpdatePropertySheet
Self->RefreshPropertySheet

return, 1
end
;-------------------------------------------------------------------------------


;===============================================================================
; Bt7psdTool::UpdateCalculation
;
; PURPOSE:
;   Update the PSD wavevector and energy transfer for the dataset
;
; PARAMETERS:
;   oPSet [in|out] - object whose data is to be updated
;
; KEYWORDS:
;
pro Bt7psdTool::UpdateCalculation, oPSet
compile_opt idl2

void = oPSet->GetMetaData('DATAPTRREF',dataPtr)
if (~ptr_valid(dataPtr)) then return
dataStr = *dataPtr
Self->GetProperty, QUNITS=rlu, sampTypeFlag=sampTypeFlag ;  rlu = 0 => 1/A, 1 => r.l.u.; sampTypeFlag=0 (powder), 1 (single xtal)
ekFac = Self.ekFac  ; E = ekFac * k^2

; calibration stuff
chDelA4 = Self.chDelA4
chEff = abs(Self.chEff) ; some values may be negative, signfying channels intended to be masked
chDelEf = Self.chDelEf
nCh = n_elements(chDelA4)

; norminal values for ei, ef, a3, a4
eio = (*dataStr.ei)
efo = (*dataStr.ef)
a3o = (*dataStr.a3)
a4o = (*dataStr.a4)
nPts = n_elements(a4o)

qType = partype(a3o)

oPSet->AddMetaData,'sampTypeFlag',sampTypeFlag

case sampTypeFlag of

   2: begin    ;; raw view
      counts = []

      ;Loop through data points in dataset:
      for i = 0, nPts-1 do begin
        counts = [[counts],[(*dataStr.PSDDETS)[*,i]]]
      endfor
      
      ; A1 -> A6
      oA1 = IDLitData((*dataStr.a1),name='A1',type=qtype)
      oA1->AddMetaData,'Long_name','A1'
      oA1->AddMetaData,'Units',''
      oA2 = IDLitData((*dataStr.a2),name='A2',type=qtype)
      oA2->AddMetaData,'Long_name','A2'
      oA2->AddMetaData,'Units',''
      oA3 = IDLitData((*dataStr.a3),name='A3',type=qtype)
      oA3->AddMetaData,'Long_name','A3'
      oA3->AddMetaData,'Units',''
      oA4 = IDLitData((*dataStr.a4),name='A4',type=qtype)
      oA4->AddMetaData,'Long_name','A4'
      oA4->AddMetaData,'Units',''
      oA5 = IDLitData((*dataStr.a5),name='A5',type=qtype)
      oA5->AddMetaData,'Long_name','A5'
      oA5->AddMetaData,'Units',''
      oA6 = IDLitData((*dataStr.a6),name='A6',type=qtype)
      oA6->AddMetaData,'Long_name','A6'
      oA6->AddMetaData,'Units',''
      ; H,K,L
      oH = IDLitData((*dataStr.h),name='H',type=qtype)
      oH->AddMetaData,'Long_name','H'
      oH->AddMetaData,'Units','rlu'
      oK = IDLitData((*dataStr.k),name='K',type=qtype)
      oK->AddMetaData,'Long_name','K'
      oK->AddMetaData,'Units','rlu'
      oL = IDLitData((*dataStr.l),name='K',type=qtype)
      oL->AddMetaData,'Long_name','K'
      oL->AddMetaData,'Units','rlu'
      ; E
      oE = IDLitData((*dataStr.e),name='E',/no_copy,type=qtype)
      oE->AddMetaData,'Long_name','E'
      oE->AddMetaData,'Units','meV'
      
      
      ; TEMP
      temp = (*dataStr.temp)
      oTemp = IDLitData(temp,name='TEMP',/no_copy,type=qtype)
      oTemp->AddMetaData,'Long_name','Temperature'
      oTemp->AddMetaData,'Units','K'

      ; magfield
      magfield = (*dataStr.magfield)
      ;magfield = unitVec # (*dataStr.magfield)
      oMagfield = IDLitData(magfield,name='MAGFIELD',/no_copy,type=qtype)
      oMagfield->AddMetaData,'Long_name','Magnetic-field'
      oMagfield->AddMetaData,'Units','T'
      
      ; Data point index
      ;unitVec = intarr(nPts) + 1
      dPt = indgen(nPts) + 1
      odPt = IDLitData(name='Data_Index',dpt,/no_copy,type=partype(dpt))
      odPt->AddMetaData,'Long_name','Data Index'
      odPt->AddMetaData,'Units',''

      ;PSD Channel Nos
      ch = indgen(nCh) + 1
      oChNos = IDLitData(name='PSDChannel',ch,/no_copy,type=partype(ch))
      oChNos->AddMetaData,'Long_name','PSD channel Nos'
      oChNos->AddMetaData,'Units',''
      
      ;PSD Channel A4
      oChA4 = IDLitData(name='PSDChannelA4',chDelA4,type=partype(chDelA4))
      oChA4->AddMetaData,'Long_name','PSD channel $2\theta$'
      oChA4->AddMetaData,'Units','$\deg$'
      

      ; Error
      zType = partype(counts)
      oErr = IDLitData(name='Error',type=qtype) ; NULL data to start
      oErr->AddMetaData,'Long_name','Error'
      oErr->AddMetaData,'Units',''
      ; Counts
      oZ = IDLitData(name='Counts',type=qtype)  ; NULL data to start
      oZ->AddMetaData,'Long_name','Counts'
      oZ->AddMetaData,'Units',''
      oZ->AddMetaData,'Signal', 1


      ; Channel Effieciencies
      unitVec = intarr(nPts) + 1
      chEff = chEff # unitVec       ; creates a array of dims [nCh,nPts]

      unitVec = fltarr(nCh) + 1.0
;      ; Monitor
      mon = unitVec # (*dataStr.mon)  ; replicate mon at each datapoint to all 48 channels of PSD
;      ; TIME
      time = unitVec # (*dataStr.time)

      oPSet->AddMetaData,'OriginalCounts',counts
;      oZ->AddMetaData,'2Theta',A4
;      oZ->AddMetaData,'QTOT',qtot
;      oZ->AddMetaData,'OMEGA',omega
      oPSet->AddMetaData,'CHEFF',ChEff
      oPSet->AddMetaData,'CHDELA4',ChDelA4
      oPSet->AddMetaData,'INDEP_VAR_TAG',dataStr.indep_var_tag
      oPSet->AddMetaData,'INDEP_VAR_VAL',(*dataStr.indep_var_val)
      oPSet->AddMetaData,'TIME',time
      oPSet->AddMetaData,'MON',mon
      oPSet->AddMetaData,'NPTS',nPts
      oPSet->AddMetaData,'MINDPNT',1
      oPSet->AddMetaData,'MAXDPNT',nPts
      oPSet->AddMetaData,'DPNTINDEX',1
      objects = [odPt,oChNos,oA1,oA2,oA3,oA4,oA5,oA6,oE,oH,oK,oL,oTemp,oMagfield,oZ,oErr,oChA4]
      pnames = ['Data_Index','PSDChannel','A1','A2','A3','A4','A5','A6','E','H','K','L','TEMP','MAGFIELD','Counts','Error','PSDChannelA4']
      oPSet->Add, objects, parameter_name=pnames
 
   end
   
   
   0: begin    ;; powder
      ; initialize a few variables
      A4 = []
      qtot = []
      omega = []
      counts = []

      ;Loop through data points in dataset:
      ; - calculate q, e and dspace
      nPts = n_elements(a4o)
      for i = 0, nPts-1 do begin
        cha4 = a4o[i] - chDelA4
        chEi = eio[i]
        chEf = efo[i] - chDelEf  ; 
        chki = sqrt(chEi/ekFac)
        chkf = sqrt(chEf/ekFac)

        chq = Sqrt(chki^2 + chkf^2 - 2*chki*chkf*Cos(cha4*!dtor))
        a4 = [[a4],[cha4]]
        qTot = [[qTot],[chq]]
        omega = [[omega],[chEi - chEf]]

        counts = [[counts],[(*dataStr.PSDDETS)[*,i]]]
      endfor

      qType = partype(qTot)
      ; A4
      oA4 = IDLitData(name='2Theta',type=qtype)
      oA4->AddMetaData,'Long_name','$2\theta$'
      oA4->AddMetaData,'Units','$\deg$'
      ; d-space
      odsp = IDLitData(name='dspace',type=qtype)
      odsp->AddMetaData,'Long_name','d'
      odsp->AddMetaData,'Units','$\AA$'
      ; Q
      oQTot = IDLitData(name='|Q|',type=qtype)
      oQTot->AddMetaData,'Long_name','|Q|'
      oQTot->AddMetaData,'Units','$\AA^{-1}$'
      ; E
      oE = IDLitData(name='E',type=qtype)
      oE->AddMetaData,'Long_name','E'
      oE->AddMetaData,'Units','meV'
      
      unitVec = fltarr(nCh) + 1.0
;      ; Monitor
      mon = unitVec # (*dataStr.mon)  ; replicate mon at each datapoint to all 48 channels of PSD
;      ; TIME
      time = unitVec # (*dataStr.time)
      ; TEMP
      temp = (*dataStr.temp)
      tempAverage = (fltarr(nPts)+1) * mean(temp)
      oTemp = IDLitData(name='TEMP',type=qtype)
      oTemp->AddMetaData,'Long_name','Temperature'
      oTemp->AddMetaData,'Units','K'
      ; magfield
      magfield = (*dataStr.magfield)
      ;magfield = unitVec # (*dataStr.magfield)
      oMagfield = IDLitData(name='MAGFIELD',type=qtype)
      oMagfield->AddMetaData,'Long_name','Magnetic-field'
      oMagfield->AddMetaData,'Units','T'

      ; Channel Effieciencies
      unitVec = intarr(nPts) + 1
      chEff = chEff # unitVec       ; creates a array of dims [nCh,nPts]
      ;oChEff = IDLitData(name='PSDChannelEff',chEff,/no_copy,type=qType)

      ; Error
      zType = partype(counts)
      oErr = IDLitData(name='Error',type=ztype) ; NULL data to start
      oErr->AddMetaData,'Long_name','Error'
      oErr->AddMetaData,'Units',''
      ; Counts
      oZ = IDLitData(name='Counts',type=ztype)  ; NULL data to start
      oZ->AddMetaData,'Long_name','Counts'
      oZ->AddMetaData,'Units',''
      oZ->AddMetaData,'Signal', 1
      oZ->AddMetaData,'OriginalCounts',counts
      oZ->AddMetaData,'2Theta',A4
      oZ->AddMetaData,'QTOT',qtot
      oZ->AddMetaData,'D-SPACE',2*!pi/qtot
      oZ->AddMetaData,'OMEGA',omega
      oZ->AddMetaData,'TEMP',(fltarr(nCh) + 1.0) # temp
      oZ->AddMetaData,'TEMPAVERAGE',(fltarr(nCh) + 1.0) # tempAverage
      oZ->AddMetaData,'MAGFIELD',(fltarr(nCh) + 1.0) # magfield
      oZ->AddMetaData,'CHEFF',ChEff
      oZ->AddMetaData,'CHDELA4',ChDelA4
      oZ->AddMetaData,'TIME',time
      oZ->AddMetaData,'MON',mon
      oZ->AddMetaData,'NPTS',nPts
      oZ->AddMetaData,'INDEP_VAR_TAG',dataStr.indep_var_tag
      oZ->AddMetaData,'INDEP_VAR_VAL',(*dataStr.indep_var_val)

      
      ; Data point index
      ;unitVec = intarr(nPts) + 1
      dPt = indgen(nPts) + 1
      odPt = IDLitData(name='Data_Index',dpt,/no_copy,type=partype(dpt))
      odPt->AddMetaData,'Long_name','Data Index'
      odPt->AddMetaData,'Units',''
      
      
      oPSet->Add, [oA4,oDsp,oE,oQTot,oTemp,oMagfield,oZ,oErr,odPt], $
                  parameter_name=['2Theta','d-space','E','|Q|','TEMP','MAGFIELD','Counts','Error','Data_Index']
 
      Self->UpdatePowderVariables, oPSet
   
   end 
   
   1: begin    ;; single crystal

      ; initialize a few variables
      qTot = []
      qx = []
      qy = []
      counts = []
      omega = []
      Temp = []
      magfield = []
      mon = []
      
      
      orient = *dataStr.ORIENTATION_PTR ; retrieve crystal orientation vectors u,v in rec. space coordinates
      orient_u = orient[0:2]            ; the crystal orientation vectors specified in multiples of a*, b* and c*
      orient_v = orient[3:5]
      
      ; determine appropriate labels to use for qx and qy
      xlabrlu = (orient_u[0] eq 0)? '0' : 'H'
      xlabrlu += ','+(orient_u[1] eq 0)? '0' : 'K'
      xlabrlu += ','+(orient_u[2] eq 0)? '0' : 'L'
      ylabrlu = (orient_v[0] eq 0)? '0' : 'H'
      ylabrlu += ','+(orient_v[1] eq 0)? '0' : 'K'
      ylabrlu += ','+(orient_v[2] eq 0)? '0' : 'L'
      xunitrlu = 'r.l.u.'
      yunitrlu = 'r.l.u.'
      
      xlabang = (float(!version.RELEASE) lt 8.3)? 'qx $||$ '+xlabrlu : 'qx $\parallel$ '+xlabrlu
      ylabang = (float(!version.RELEASE) lt 8.3)? 'qy $\bot$ '+xlabrlu : 'qy $\perp$ '+xlabrlu 
      xunitang = '$\AA^{-1}$'
      yunitang = '$\AA^{-1}$'      
      if (rlu eq 1) then begin
        xlab = xlabrlu
        ylab = ylabrlu
        xunit = xunitrlu
        yunit = xunitrlu
      endif else begin
        xlab = xlabang
        ylab = ylabang
        xunit = xunitang
        yunit = xunitang        
      endelse
      
      ; calibration stuff
      chDelA4 = Self.chDelA4
      chEff = abs(Self.chEff) ; some values may be negative, signfying channels intended to be masked
      chDelEf = Self.chDelEf
      nCh = n_elements(chDelA4)
      
      
      ;Loop through data points in dataset:
      ; - calculate (q,qx,qy), given ki, kf, a3 and a4 for each channel
      nPts = n_elements(a4o)
      for i = 0, nPts-1 do begin
        cha3 = a3o[i]
        cha4 = a4o[i] - chDelA4
        chEi = eio[i]
        chEf = efo[i] - chDelEf  ; 
        chki = sqrt(chEi/ekFac)
        chkf = sqrt(chEf/ekFac)
      ;  monNorm = baseMon/(*dataStr.mon)[i]
      
        angle2q, chki, chkf, cha3, cha4, chq, chqx, chqy, phiOffset=Self.phiOffset
        
        omega = [[omega],[chEi - chEf]]
        qTot = [[qTot],[chq]]
      
        qx = [[qx],[chqx]]  
        qy = [[qy],[chqy]]  
        counts = [[counts],[(*dataStr.PSDDETS)[*,i]]]
        ;counts = [[counts],[(*dataStr.PSDDETS)[*,i]/chEff]] ;Normalize to detector efficiency
      endfor
      
      ; Q
      qType = partype(qTot)
      oQTot = IDLitData(qTot,name='|Q|',/no_copy,type=qtype)
      oQTot->AddMetaData,'Long_name','|Q|'
      oQTot->AddMetaData,'Units','$\AA^{-1}$'
      ; E
      oE = IDLitData(omega,name='E',/no_copy,type=qtype)
      oE->AddMetaData,'Long_name','E'
      oE->AddMetaData,'Units','meV'
      ; Qx
      oQx = IDLitData(qx,name='QX',/no_copy,type=qtype)
      oQx->AddMetaData,'Long_name',xlab
      oQx->AddMetaData,'Units',xunit
      oQx->AddMetaData,'RLU',rlu
      oQx->AddMetaData,'AA2RLU',1            ; will be updated later in UpdateQxQy method
      oQx->AddMetaData,'LABRLU', xlabrlu
      oQx->AddMetaData,'LABANG', xlabang
      ; Qy
      oQy = IDLitData(qy,name='QY',/no_copy,type=qtype)
      oQy->AddMetaData,'Long_name',ylab
      oQy->AddMetaData,'Units',yunit
      oQy->AddMetaData,'RLU',rlu
      oQy->AddMetaData,'AA2RLU',1            ; will be updated later in UpdateQxQy method
      oQy->AddMetaData,'LABRLU', ylabrlu
      oQy->AddMetaData,'LABANG', ylabang
      
      unitVec = fltarr(nCh) + 1.0 
      
      ; Monitor
      mon = unitVec # (*dataStr.mon)  ; replicate mon at each datapoint to all 48 channels of PSD
      ;oMon = IDLitData(mon,name='MON',/no_copy,type=qtype)
      ;oMon->AddMetaData,'Long_name','Monitor'
      ;oMon->AddMetaData,'Units',''
      ; TIME
      time = unitVec # (*dataStr.time)
      ;oTime = IDLitData(time,name='TIME',/no_copy,type=qtype)
      ;oTime->AddMetaData,'Long_name','Time'
      ;oTime->AddMetaData,'Units','seconds'
      ; TEMP
      temp = unitVec # (*dataStr.temp)
      oTemp = IDLitData(temp,name='TEMP',/no_copy,type=qtype)
      oTemp->AddMetaData,'Long_name','Temperature'
      oTemp->AddMetaData,'Units','K'
      ; magfield
      magfield = unitVec # (*dataStr.magfield)
      oMagfield = IDLitData(magfield,name='MAGFIELD',/no_copy,type=qtype)
      oMagfield->AddMetaData,'Long_name','Magnetic-field'
      oMagfield->AddMetaData,'Units','T'

      ;PSD Channel Nos
      unitVec = intarr(nPts) + 1
      ch = indgen(nCh) + 1
      ch = ch # unitVec ; creates a array of dims [nCh,nPts], same as counts variable
      oChNos = IDLitData(name='PSDChannel',ch,/no_copy,type=partype(ch))
      oChNos->AddMetaData,'Long_name','PSD channel Nos'
      oChNos->AddMetaData,'Units',''
      
      ; Channel Effieciencies
      chEff = chEff # unitVec
      ;oChEff = IDLitData(name='PSDChannelEff',chEff,/no_copy,type=qType)


      ; Error
      zType = partype(counts)
      oErr = IDLitData(name='Error',type=ztype) ; NULL data to start
      oErr->AddMetaData,'Long_name','Error'
      oErr->AddMetaData,'Units',''
      ; Counts
      oZ = IDLitData(name='Counts',type=ztype)  ; NULL data to start
      oZ->AddMetaData,'Long_name','Counts'
      oZ->AddMetaData,'Units',''
      oZ->AddMetaData,'Signal', 1
      oZ->AddMetaData,'OriginalCounts',counts
      oZ->AddMetaData,'CHEFF',ChEff
      oZ->AddMetaData,'CHDELA4',ChDelA4
      oZ->AddMetaData,'TIME',time
      oZ->AddMetaData,'MON',mon      
      oZ->AddMetaData,'INDEP_VAR_TAG',dataStr.indep_var_tag
      oZ->AddMetaData,'INDEP_VAR_VAL',(*dataStr.indep_var_val)
      
      oPSet->Add, [oQx,oQy,oE,oQTot,oTemp,oMagfield,oZ,oErr,oChNos], $
                  parameter_name=['QX','QY','E','|Q|','TEMP','MAGFIELD','Counts','Error','PSDChannel']

      ;
      Self->UpdateIndependentVariables, oPSet
      Self->UpdateCounts, oPSet
   
   end
endcase
   


end
;-------------------------------------------------------------------------------


;===============================================================================
; Bt7psdTool::UpdatePowderVariables
;+
; PURPOSE:
;   Update all variables (dependent and independent) to reflect current user settings
;   Only applicable to powder samples
;
; PARAMETERS:
;   oPSet [in|out] - object whose data is to be updated
;
; KEYWORDS:
;-
pro Bt7psdTool::UpdatePowderVariables, oPSet
compile_opt idl2

if ((n_elements(oPSet) eq 0) || ~obj_valid(oPSet[0])) then begin
   ; retrieve all loaded datasets
   Self->GetProperty, sampContRef=oCont, bkgdContref=oContB
   oPSet = oCont->Get(/all,count=nData)
   oPSetB = oContB->Get(/all,count=nDataB)
   if (nData gt 0 && nDataB gt 0) then begin
     oPSet = [oPSet,oPSetB]
     nData = nData+nDataB
   endif else if (nDataB gt 0) then begin
     oPSet = oPSetB
     nData = nDataB     
   endif
endif else begin
   nData = n_elements(oPSet)
endelse

if (nData eq 0) then return

Self->GetProperty, MONSCALEVALUE=monScaleVal, NORMALIZETO=normTo, MONCORFLAG=monCorFlag $
                 ,RESCORFLAG=resCorFlag, FASTFLAG=fastFlag, FASTVALUE=fastVal $
                 ,DETAILEDBALANCEFLAG=detBalFlag,effNormFlag=effFlag, sampTypeFlag=sampTypeFlag $
                 ,bkgdFlag=bkgdFlag, nBkgd=nBkgd, bkgdContRef=bkgdContRef $
                 ,bkgdSubVarTol=bkgdSubVarTol, bkgdSubVarFlag=bkgdSubVarFlag

if (sampTypeFlag ne 0) then return  ; only proceed if user selection is set to powder

for i=0,nData-1 do begin
   oData = oPSet[i]
   oZ = oData->GetByName('Counts')
   if (~obj_valid(oZ)) then return
   status = oData->GetMetaData('sampTypeFlag',sTypeFlag)
   if (sTypeFlag ne 0) then continue  ; only proceed if dataset was originally read in as powder
   
   oErr = oData->GetByName('Error')
   if (~obj_valid(oErr)) then return
   if (~oZ->GetMetaData('OriginalCounts',counts)) then return
   error = counts
   index = where(error le 0.0,cnt)
   if (cnt gt 0) then error[index] = 1.0
   error = sqrt(error)

   status = oZ->GetMetaData('MON',mon)
   status = oZ->GetMetaData('TIME',time)   
   
   ;; Perform bkgd subtraction if appropriate
   status = oData->GetMetaData('DATAPTRREF',dataPtr)
   dataStr = *dataPtr
   bkgdSubVarTag = strupcase(Self.bkgdSubVars[bkgdSubVarFlag])
   tagNames = tag_names(dataStr)
   if (bkgdSubVarTag eq 'HKL') then begin
     ; replace 'HKL' with 'H','K' and 'L'
     varHIndex = where(tagNames eq 'H', HtagFound4Samp)
     varKIndex = where(tagNames eq 'K', KtagFound4Samp)
     varLIndex = where(tagNames eq 'L', LtagFound4Samp)
     tagFound4Samp = HtagFound4Samp && KtagFound4Samp && LtagFound4Samp
     if (tagFound4Samp) then begin
       bkgdSubVarValH = mean((*dataStr.(varHIndex)))
       bkgdSubVarValK = mean((*dataStr.(varKIndex)))
       bkgdSubVarValL = mean((*dataStr.(varLIndex)))
     end
   endif else begin
     varIndex = where(tagNames eq bkgdSubVarTag, tagFound4Samp)
     if (tagFound4Samp) then bkgdSubVarVal = mean((*dataStr.(varIndex)))
   endelse

   oData->GetProperty, type=type
   if (strcmp(type,'samp') && bkgdFlag && (nBkgd gt 0)) then begin
      status = oZ->GetMetaData('INDEP_VAR_TAG',sampIndepVarTag)
      status = oZ->GetMetaData('INDEP_VAR_VAL',sampIndepVarVal)
      tolB = 0.005
      bkgdCntBuf = counts*0.0
      bkgdMonBuf = bkgdCntBuf
      bkgdSub = 0
      for j = 0, nBkgd-1 do begin
        oPsetB = bkgdContRef->Get(position=j)
        if (~obj_valid(oPsetB)) then continue
;        status = oPsetB->GetMetaData('sampTypeFlag',sTypeFlagB)
;        if (sTypeFlagB ne 0) then continue  ; only proceed if dataset was originally read in as powder

        ; retrieve the bkgdSubVarVal for bkgd dataset and compare with
        ; same quantity for signal dataset
        if (tagFound4Samp) then begin
          status = oPsetB->GetMetaData('DATAPTRREF',dataPtr)
          dataStr = *dataPtr
          if (bkgdSubVarTag eq 'HKL') then begin
            bkgdSubVarValBH = mean((*dataStr.(varHIndex)))                
            bkgdSubVarValBK = mean((*dataStr.(varKIndex)))
            bkgdSubVarValBL = mean((*dataStr.(varLIndex)))
            HNotSame = abs(bkgdSubVarValH-bkgdSubVarValBH) gt bkgdSubVarTol
            KNotSame = abs(bkgdSubVarValK-bkgdSubVarValBK) gt bkgdSubVarTol
            LNotSame = abs(bkgdSubVarValL-bkgdSubVarValBL) gt bkgdSubVarTol
            ; Is the keyword variable same for singnal and bkgd datasets?
            ; If not then skip to next bkgd dataset
            if (HNotSame || KNotSame || LNotSame) then continue  ; to next bkgd dataset
          endif else begin
            bkgdSubVarValB = mean((*dataStr.(varIndex)))
            ; Is the keyword variable same for singnal and bkgd datasets?
            ; If not then skip to next bkgd dataset
            if (abs(bkgdSubVarVal-bkgdSubVarValB) gt bkgdSubVarTol) then continue ; to next bkgd dataset
          endelse
        endif

        oZB = oPsetB->GetByName('Counts')
        if (~obj_valid(oZB)) then continue

        status = oZB->GetMetaData('INDEP_VAR_TAG',bkgdIndepVarTag)
        status = oZB->GetMetaData('INDEP_VAR_VAL',bkgdIndepVarVal)        
        if (sampIndepVarTag ne bkgdIndepVarTag) then continue
        
        status = oZB->GetMetaData('MON',monB)
        if (~oZB->GetMetaData('OriginalCounts',countsB)) then continue
        
        ; now perform subtraction for correspong datapoints in signal and bkgd
        ; datasets that have the same value of the independent variable (scan variable)
        sIndex = []
        bIndex = []
        for k=0, n_elements(bkgdIndepVarVal)-1 do begin
          index = where(abs(sampIndepVarVal - bkgdIndepVarVal[k]) lt tolB, found)
          if (found eq 1) then begin
            sIndex = [sIndex,index]
            bIndex = [bIndex,k]
          endif
        endfor
        if (n_elements(sIndex) gt 0) then begin
          ;; accumulate the bkgd counts and corresponding monitor values so that
          ;; we end up with a sum of counts and sum of monitor over all bkgd before doing
          ;; the subtration outside the bkgd loop
          bkgdSub = 1
          bkgdCntBuf[*,sIndex] = bkgdCntBuf[*,sIndex] + countsB[*,bIndex]
          bkgdMonBuf[*,sIndex] = bkgdMonBuf[*,sIndex] + monB[*,bIndex]
        endif
      endfor
      if (bkgdSub) then begin
        index = where(bkgdMonBuf eq 0.0, zeroMonPresent)
        if (zeroMonPresent gt 0) then bkgdMonBuf[index] = 1.0
        scaleFactB = mon / bkgdMonBuf
        counts = temporary(counts) - scaleFactB * bkgdCntBuf
        error = sqrt(temporary(error)^2 + scaleFactB * bkgdCntBuf)        
      endif
   endif
   
   if (effFlag) then begin
      if (~oZ->GetMetaData('CHEFF',chEff)) then return
      counts = temporary(counts)/chEff
      error = temporary(error)/chEff   ;temporary(error)/sqrt(chEff)
      ;oChEff = oData->GetByName('ChEff')
      ;if (obj_valid(ochEff)) then begin
      ;   status = oChEff->GetData(chEff)
      ;   counts = temporary(counts)/chEff
      ;   error = temporary(error)/sqrt(chEff)
      ;endif
   endif

   if (fastFlag) then begin   ; perform corrections that depend on first applying fast bkgd correction
      ;; Apply fast Correction
      if (fastVal ne 0.0) then begin
         counts = temporary(counts) - (time/60.0)*fastVal ; time is measured in secs and fastVal is fast bkgd in cnts/min
      endif
      
      ;; Apply monitor correction
      if (monCorFlag) then begin
         ;; Constraints are:
         ;; 1 - PG monochromator is in use (d spacing = 3.354)
         ;; 2 - 5 <= Ei <= 34; For Ei < 5, flag an error, for Ei > 34, correction = 1
         ;; CF(Ei) = M0 + M1*Ei + M2*Ei^2 + M3*Ei^3 + M4*Ei^4
         status = oData->GetMetaData('DATAPTRREF',dataPtr)
         if (~ptr_valid(dataPtr)) then return
         dspacem = (*dataPtr).dspacem  ; d spacing of monochromator
         diff = abs(dspacem - 3.354)
         PGMON = diff lt 0.001
         Ei = (*(*dataPtr).Ei)
         index = where(Ei lt 5.0,Eilt5)  ; correction not aloowed if Ei < 5.0 meV
         if (Eilt5) then begin
            msg = ['Ei is lower than 5.0 meV.','Monitor correction was not applied']
            title = 'Monitor Correction'
            Self->ErrorMessage,msg,title=title,severity=0
         endif else if (PGMON) then begin
            M0 =  6.1230
            M1 = -5.9630e-1
            M2 =  2.6244e-2
            M3 = -4.8430e-4
            M4 =  2.8810e-6
            index = where(Ei gt 34.0,Eigt34)
            CFEi = M0 + M1*Ei + M2*Ei^2 + M3*Ei^3 + M4*Ei^4
            if (Eigt34) then CFEi[index] = 1.0  ; for Ei > 34, CF = 1
            counts = TEMPORARY(counts)*CFEi
            error  = TEMPORARY(error)*CFEi 
         endif
      endif 
      
;      if (resCorFlag) then begin
;         ; apply resolution correction
;      endif
;      
;      if (detBalFlag) rhen begin
;         ; apply detailed balance correction
;      endif
   endif
   
   ;; Apply monitor normalization
   case normTo of
   
      0: begin
         monScale = monScaleVal/mon
      end
      
      1: begin
         monScale = monScaleVal/time
      end
      
      else: monScale = monScaleVal
   endcase
   counts = TEMPORARY(counts)*monScale
   error = TEMPORARY(error)*monscale
   
   ;; Mask bad channels
   Self->GetProperty, maskFlag=maskFlag
   sBadChans = Self.maskedDets
   maskedChansExist = ~strcmp(sBadChans,'')
   if (maskFlag && maskedChansExist) then begin
      status = compactstring2intvector(sBadChans, badChans)
      badChanIndex = badChans - 1   ; convert channel nos to 0-based channel index
      ; Apply mask by simple replacing the counts in the masked channels by NaNs
      ; Leave error variable untouched
      counts[badChanIndex,*] = !values.F_NAN
      error[badChanIndex,*] = 0.0
   endif
      
   ;; Store processed 2D counts and erros in metadata in oZ and oErr objects
   ;; the processed counts and errors are used only when exporting powder data to Mslice!
   oZ->AddMetaData, 'ProcessedCounts',counts
   oZ->AddMetaData, 'ProcessedErrors',error
   
   ;; Merge data wrt 2theta
   status = oZ->GetMetaData('2Theta',var)
   mgCount = counts
   mgError = error
   Self->MergePowderChannels, var, mgCount, mgError, tol=0.05   
   oVar = oData->GetByName('2Theta',count=cnt)
   status = oVar->SetData(var,/no_copy,/no_notify)
   oVar->AddMetaData,'Counts',mgCount
   oVar->AddMetaData,'Error',mgError
   
   ;; Merge data wrt E
   status = oZ->GetMetaData('Omega',var)
   mgCount = counts
   mgError = error
   Self->MergePowderChannels, var, mgCount, mgError, tol=0.001   
   oVar = oData->GetByName('E',count=cnt)
   status = oVar->SetData(var,/no_copy,/no_notify)
   oVar->AddMetaData,'Counts',mgCount 
   oVar->AddMetaData,'Error',mgError 
   
   ;; Merge data wrt Q and d
   status = oZ->GetMetaData('Qtot',var)
   mgCount = counts
   mgError = error
   Self->MergePowderChannels, var, mgCount, mgError, tol=0.001   
   oVar = oData->GetByName('|Q|',count=cnt)
   status = oVar->SetData(var,/no_notify) ; don't use no_copy keyword as var is used again
   oVar->AddMetaData,'Counts',mgCount
   oVar->AddMetaData,'Error',mgError
   
   status = oZ->GetMetaData('Qtot',var)
   var = 2*!pi/var
   mgCount = counts
   mgError = error
   Self->MergePowderChannels, var, mgCount, mgError, tol=0.001
   oVar = oData->GetByName('d-space',count=cnt)
   status = oVar->SetData(var,/no_copy,/no_notify)
   oVar->AddMetaData,'Counts',mgCount
   oVar->AddMetaData,'Error',mgError

   ;; Merge data wrt Temp
   Self->GetProperty, useTempAverageFlag=useTempAverageFlag
   if (useTempAverageFlag) then $
     status = oZ->GetMetaData('TEMPAVERAGE',var) else $
     status = oZ->GetMetaData('TEMP',var)  
   mgCount = counts
   mgError = error
   Self->MergePowderChannels, var, mgCount, mgError, tol=0.05
   oVar = oData->GetByName('TEMP',count=cnt)
   status = oVar->SetData(var,/no_copy,/no_notify)
   oVar->AddMetaData,'Counts',mgCount
   oVar->AddMetaData,'Error',mgError
   ;; Merge data wrt Magfield
   status = oZ->GetMetaData('MAGFIELD',var)
   mgCount = counts
   mgError = error
   Self->MergePowderChannels, var, mgCount, mgError, tol=0.05
   oVar = oData->GetByName('MAGFIELD',count=cnt)
   status = oVar->SetData(var,/no_copy,/no_notify)
   oVar->AddMetaData,'Counts',mgCount
   oVar->AddMetaData,'Error',mgError
   ;; Merge data wrt data_index
   mgCount = counts
   mgError = error
   Self->MergePowderChannels, var, mgCount, mgError, /SUMCHANNELS
   oVar = oData->GetByName('Data_Index',count=cnt)
   oVar->AddMetaData,'Counts',mgcounts
   oVar->AddMetaData,'Error',mgError
   ;status = oZ->SetData(counts,/no_copy);,/no_notify)
   ;status = oErr->SetData(error,/no_copy,/no_notify)
endfor

end
;-------------------------------------------------------------------------------


;===============================================================================
; Bt7psdTool::MergePowderChannels
;+
; PURPOSE:
;   Merge the PSD channels together for powder samples so that overlapping data 
;   determined with respect to the specified xvalues (x) are combined such 
;   that the corresponding errors are added in quadrature. NaNs are appropriately handled 
;
; PARAMETERS:
;   x [in|out] - the xvalues controlling how the data are to be merged
;   
;   counts [in|out] - the data to be merged
;   
;   error [in|out] - the error are to be added in quadrature where there is overlap
;
; KEYWORDS:
;      
;   tolerance - any two values seperated by less than or equal to tolerance are treated 
;               as the same and hence added in quadrature. 
;   
;   sumChannels - Set this keyword to simply sum counts and error wrt to the channel dimension 
;                 to have counts at each data point
;-
pro Bt7psdTool::MergePowderChannels, x, counts, error, tolerance=tol, sumChannels=sumChannels
;pro MergePowderChannels, x, counts, error, tolerance=tol, sumChannels=sumChannels
compile_opt idl2

; Note: if xvals, counts and error are 2D then they are of the form 
; counts[n_ch, n_pts] where 
;     n_ch are the nos of PSD channels and 
;     n_pts are the number of data points

dimC = size(counts,/N_DIMENSIONS)
if (keyword_set(sumChannels) && (dimC eq 2)) then begin
   ;; simply sum counts and error wrt to the channel dimension to have counts at each data point
   counts = total(temporary(counts),1,/NAN)
   error = sqrt(total(temporary(error)^2,1,/NAN))
   return
endif

;if (n_elements(tol) eq 0) then tol = 0.001
;Self->GetProperty, minbinWidth=tol

; if x is not varying then simply sum everthing and return scalars
xmax = max(x,min=xmin)
if (abs(xmax-xmin) le tol) then begin ;if (xmax eq xmin) then begin
   x = x[0]
   counts = total(counts,/NAN)
   error = total(error,/NAN)
   return
endif

dimX = size(x,/N_DIMENSIONS)
if (dimX eq 2) then x = reform(x,n_elements(x),/OVERWRITE)
if (dimC eq 2) then begin
   counts = reform(counts,n_elements(counts),/OVERWRITE)
   error = reform(error,n_elements(error),/OVERWRITE)
endif

; If constant bin width is required, initially regroup the data using
; very fine bins (specified through the tol keyword) and then use 
; Drebin to rebin the regrouped data into constant bin
Self->GetProperty, constBinFlag=constBinFlag
if (constBinFlag eq 0) then Self->GetProperty, minbinWidth=tol

; sort data in ascending x order
sindex = sort(x)
x = x[sindex]
counts = counts[sindex]
error = error[sindex]

; Get rid of data points that are NaNs
nanIndex = where(finite(counts,/NAN),nanPresent, complement=validIndex)
if (nanPresent gt 0) then begin
   x = x[validIndex]
   counts = counts[validIndex]
   error = error[validIndex]
endif

;;----------------------------------------------------
;; append a dummy point at the end which is necessary for the 
;; algorithm to work properly but otherwise does not affect the results
;counts = [counts,!values.F_NAN]
;x = [x,2*tol]
;error = [error,0.0]
;
;nElmts = n_elements(counts)
;sf = fltarr(nElmts) + 1.0   ; each pt has a default weight of 1
;
;; if there are NaN's in counts, note the locations by setting their sf to 0.0
;nanIndex = where(finite(counts,/NAN),nanPresent)
;if (nanPresent gt 0) then sf[nanIndex] = 0.0
;
;; locate all duplicate x values within the specified tolerance
;diff = abs(x[0:nElmts-2] - x[1:nElmts-1])
;dupIndex = where(diff lt tol, complement=uniqIndex, nDup)
;
;; sum all points sharing the same x value (the duplicates)
;i = 0
;while (i lt ndup) do begin
;   index = [dupIndex[i], dupIndex[i]+1]   ; each entry in dupIndex ==> a dup pair in x
;   for j=i, nDup-1 do begin
;      gap = 2  ; the default gap
;      if (j lt nDup-1) then gap = abs(dupIndex[j] - dupIndex[j+1])
;      if (gap eq 1) then $ ;==> more than two consecutive x values are the same 
;         index = [index,dupIndex[j]+2] $
;      else $
;         break
;   endfor
;   i = j + 1
;   
;   ; sum the duplicates found so far
;   counts[dupIndex[j]+1] = total(counts[index],/NAN)
;   error[dupIndex[j]+1] = sqrt(total(error[index]^2,/NAN))
;   sf[dupIndex[j]+1] = total(sf[index],/NAN)
;   
;   ; a sf of 0.0 => the corresponding count should be undefined. This explicit 
;   ; assignment is necessary b/c total(counts[index],/NAN)will return 0.0 instead of
;   ; a NaN if counts[index] contains all NaNs
;   if (sf[dupIndex[j]+1] eq 0.0) then counts[dupIndex[j]+1] = !values.F_NAN
;endwhile
;
;if (nDup gt 0) then begin
;   counts = temporary(counts[uniqIndex])/sf[uniqIndex]
;   error = temporary(error[uniqIndex])/sf[uniqIndex]
;   x = x[uniqIndex]
;endif else begin
;   ;; remove that last dummy point that was added
;   x = x[0:nElmts-2]
;   counts = counts[0:nElmts-2]
;   error = error[0:nElmts-2]
;endelse
;;-------------------------------------------------------

;; Locate duplicate x values
;nElmts = n_elements(counts)
;i = 0
;uniqIndex = []
;dupIndex = []
;while (i lt nElmts-1) do begin
;   uniqIndex = [uniqIndex,i]
;   for j = i, nElmts-2 do begin
;      diff = abs(x[i] - x[j+1])
;      if (diff gt tol) then break
;      dupIndex = [dupIndex,i]
;   endfor
;   i = j + 1
;endwhile 
;nDup = n_elements(dupIndex)
;
;xout = x[uniqIndex]
;zin =counts
;ein = error
;drebin,x,counts,error,xout,zout,zeout,/points,/to_points,err=err,emsg=emsg
;if (err ne 0) then return
;x = xout
;counts = zout
;error = zeout


; Locate duplicate x values
nElmts = n_elements(counts)
i = 0
newx = []
newc = []
newe = []
while (i lt nElmts-1) do begin
   index = i
   for j = i, nElmts-2 do begin
      diff = abs(x[i] - x[j+1])
      if (diff le tol) then begin
         index = [index,j+1]
      endif else begin
         break
      endelse
   endfor
   i = j + 1
   xmean = mean(x[index]) ;(moment(x[index],maxmoment=1))[0]
   newx = [newx,xmean]
   c1 = mean(counts[index],/NAN) ;total(counts[index],/NAN)/float(n_elements(index))
   e1 = sqrt(total(error[index]^2,/NAN))/float(n_elements(index))
   newc = [newc,c1]
   newe = [newe,e1]
endwhile
lastIndex = index[n_elements(index)-1]
if (lastIndex eq nElmts-2) then begin
   newx = [newx,x[nElmts-1]]
   newc = [newc,counts[nElmts-1]]
   newe = [newe,error[nElmts-1]]
endif 

if (constBinFlag eq 0) then begin
  x = newx
  counts = newc
  error = newe
endif else begin
  Self->GetProperty, minbinWidth=tol
  x = xmin + tol*findgen((xmax - xmin)/tol + 1)
  drebin,newx,newc,newe,x,counts,error,/points,/to_points,err=err,emsg=emsg
  if (err ne 0) then begin
    x = newx
    counts = newc
    error = newe
  endif
endelse

end
;-------------------------------------------------------------------------------

;===============================================================================
; Bt7psdTool::MergeDupGroups
;+
; PURPOSE:
;   For the specified 2D dataset, merge groups that have overlapping values. Use the
;   specified tolerance to determine if group values are the same or not. Groups are combined such 
;   that the corresponding errors are added in quadrature. NaNs are appropriately handled 
;
; PARAMETERS:
;   y [in|out] - the y or group values that determine how the data are to be merged
;   
;   counts [in|out] - the data to be merged
;   
;   error [in|out] - the error are to be added in quadrature where there is overlap
;
; KEYWORDS:
;      
;   tolerance - any two values seperated by less than or equal to tolerance are treated 
;               as the same and hence added in quadrature. 
;   
;-
pro Bt7psdTool::MergeDupGroups, y, z, dz, tolerance=tol
compile_opt idl2

; y must be a vector
; z and dz are 2D of dimension 
; z[n_x, n_y] where 
;     n_x are nos of x values 
;     n_y are the number of y values or groups
if (size(z,/n_dimensions) ne 2) then return
if (size(dz,/n_dimensions) ne 2) then return
if (size(y,/n_dimensions) ne 1) then return

dims = size(z,/dimensions)
n_x = dims[0]
n_y = dims[1]
if (size(y,/dimensions) ne n_y) then return

; Locate duplicate y values
i = 0
while (i lt n_y-1) do begin
   index = i
   for j = i, n_y-2 do begin
      diff = abs(y[i] - y[j+1])
      if (diff le tol) then begin
         index = [index,j+1]
      endif else begin
         break
      endelse
   endfor
   i = j + 1
   ymean = mean(y[index])  ;(moment(y[index],maxmoment=1))[0]
   nIndex = n_elements(index)
   if (nIndex gt 1) then begin
     c1 = mean(z[*,index],dimension=2,/NAN) ;total(z[*,index],2,/NAN)/float(nIndex)
     e1 = sqrt(total(dz[*,index]^2,2,/NAN))/float(nIndex)
   endif else begin
     c1 = z[*,index]
     e1 = dz[*,index]
   endelse
   if (size(c1,/n_dimensions) eq 1) then begin
     c1 = reform(c1,n_x,1)
     e1 = reform(e1,n_x,1)
   endif
   if (n_elements(newy) eq 0) then begin
     newy = ymean
     newc = c1
     newe = e1
   endif else begin
     newy = [newy,ymean]
     newc = transpose([transpose(newc),transpose(c1)])
     newe = transpose([transpose(newe),transpose(e1)])
   endelse
;   if (i gt 6189) then begin
;      print,i
;   endif
endwhile
lastIndex = index[n_elements(index)-1]
if (lastIndex eq n_y-2) then begin
  ; the last group was not processed so include it.
   newy = [newy,y[n_y-1]]
   c1 = reform(z[*,n_y-1],n_x,1)
   e1 = reform(dz[*,n_y-1],n_x,1)
   newc = transpose([transpose(newc),transpose(c1)])
   newe = transpose([transpose(newe),transpose(e1)])
endif 
y = newy
z = newc
dz = newe



;
;
;
;; Look for duplicate y values and merge the corresponding groups in z and dz
;diff = abs(y[0:n_y-2] - y[1:n_y-1])
;dupIndex = where(diff lt tol, complement=uniqIndex, nDup)
;if (nDup le 0) then return
;
;i = 0
;mon = fltarr(n_x,n_y) + 1.0
;nanIndex = where(finite(z,/NAN),nanPresent);, complement=validIndex)
;if (nanPresent gt 0) then mon[nanIndex] = 0.0 ; set mon eq 0 for undefined counts
;
;while (i lt nDup) do begin
;   index = [dupIndex[i], dupIndex[i] + 1]   ; each entry in dupIndex implies a dup pair in y
;   for j = i, nDup-1 do begin
;      gap = 2
;      if (j lt nDup-1) then gap = abs(dupIndex[j] - dupIndex[j+1])
;      if (gap eq 1) then $ => more than two consecutive values are the same in y
;         index = [index, dupIndex[j] + 2] $
;      else $
;         break
;   endfor
;   ;print,index
;   i = j + 1
;   
;   ; add up duplicate entries
;   z[*,dupIndex[j]+1] = total(z[*,index], 2, /NAN) 
;   dz[*,dupIndex[j]+1] = sqrt(total(dz[*,index]^2, 2, /NAN)) 
;   mon[*,dupIndex[j]+1] = total(mon[*,index], 2, /NAN)
;   zeroMonIndex = where(mon[*,dupIndex[j]+1] eq 0.0,zeroMonCnt)
;   if (zeroMonCnt gt 0) then begin
;      ;a mon value of 0.0 signifies that the corresponding count should be undefined
;      ;so make sure those values are replaced with NaNs. This step is necessary b/c 
;      ;total(z[],/NAN) will return a 0.0 instead of a NaN if z[] contains all NaNs
;      z[zeroMonIndex,dupIndex[j]+1] = !values.D_NAN
;      dz[zeroMonIndex,dupIndex[j]+1] = !values.D_NAN
;   endif
;endwhile
;
;
;if (nDup gt 0) then begin
;   z = temporary(z[*,uniqIndex])
;   dz = temporary(dz[*,uniqIndex])
;   mon = temporary(mon[*,uniqIndex])
;   y = y[uniqIndex] 
;endif
;
;; normalize so duplicate groups are smoothed out
;z = z/mon
;dz = dz/mon

end
;-------------------------------------------------------------------------------


;===============================================================================
; Bt7psdTool::UpdateCounts
;+
; PURPOSE:
;   Update Counts/Error for the specified dataset according to current settings
;
; PARAMETERS:
;   oPSet [in|out] - object whose data is to be updated. If not specified, then 
;
; KEYWORDS:
;-
pro Bt7psdTool::UpdateCounts, oPSet
compile_opt idl2

if ((n_elements(oPSet) eq 0) || ~obj_valid(oPSet[0])) then begin
   ; retrieve all loaded datasets
   Self->GetProperty, sampContRef=oCont, bkgdContref=oContB
   oPSet = oCont->Get(/all,count=nData)
   oPSetB = oContB->Get(/all,count=nDataB)
   if (nData gt 0 && nDataB gt 0) then begin
     oPSet = [oPSet,oPSetB]
     nData = nData+nDataB
   endif else if (nDataB gt 0) then begin
     oPSet = oPSetB
     nData = nDataB     
   endif
endif else begin
   nData = n_elements(oPSet)
endelse

if (nData eq 0) then return

Self->GetProperty, MONSCALEVALUE=monScaleVal, NORMALIZETO=normTo, MONCORFLAG=monCorFlag $
                 ,RESCORFLAG=resCorFlag, FASTFLAG=fastFlag, FASTVALUE=fastVal $
                 ,DETAILEDBALANCEFLAG=detBalFlag,effNormFlag=effFlag, sampTypeFlag=sampTypeFlag $
                 ,logFlag=logFlag, bkgdFlag=bkgdFlag, nBkgd=nBkgd, bkgdContRef=bkgdContRef $
                 ,bkgdSubVarTol=bkgdSubVarTol, bkgdSubVarFlag=bkgdSubVarFlag

case sampTypeFlag of
   0: Self->UpdatePowderVariables, oPSet
   
   1: begin
      ;; single xtal
      
      maxVals = []
      minVals = []
      for i=0,nData-1 do begin
         oData = oPSet[i]

         status = oData->GetMetaData('sampTypeFlag',sTypeFlag)
         if (sTypeFlag ne 1) then continue  ; only proceed if dataset was originally read in as Single Crystal

         oZ = oData->GetByName('Counts')
         if (~obj_valid(oZ)) then return
         oErr = oData->GetByName('Error')
         if (~obj_valid(oErr)) then return
         if (~oZ->GetMetaData('OriginalCounts',counts)) then return
         error = counts
         index = where(error le 0.0,cnt)
         if (cnt gt 0) then error[index] = 1.0
         error = sqrt(error)
         
         status = oZ->GetMetaData('MON',mon)
         status = oZ->GetMetaData('TIME',time)

         ;; Perform bkgd subtraction if appropriate
         status = oData->GetMetaData('DATAPTRREF',dataPtr)
         dataStr = *dataPtr
         bkgdSubVarTag = strupcase(Self.bkgdSubVars[bkgdSubVarFlag])
         tagNames = tag_names(dataStr)
         if (bkgdSubVarTag eq 'HKL') then begin
           ; replace 'HKL' with 'H','K' and 'L'
           varHIndex = where(tagNames eq 'H', HtagFound4Samp)
           varKIndex = where(tagNames eq 'K', KtagFound4Samp)
           varLIndex = where(tagNames eq 'L', LtagFound4Samp)
           tagFound4Samp = HtagFound4Samp && KtagFound4Samp && LtagFound4Samp
           if (tagFound4Samp) then begin
             bkgdSubVarValH = mean((*dataStr.(varHIndex)))
             bkgdSubVarValK = mean((*dataStr.(varKIndex)))
             bkgdSubVarValL = mean((*dataStr.(varLIndex)))
           end
         endif else begin
           varIndex = where(tagNames eq bkgdSubVarTag, tagFound4Samp)
           if (tagFound4Samp) then bkgdSubVarVal = mean((*dataStr.(varIndex)))
         endelse

         oData->GetProperty, type=type
         if (strcmp(type,'samp') && bkgdFlag && (nBkgd gt 0)) then begin
           status = oZ->GetMetaData('INDEP_VAR_TAG',sampIndepVarTag)
           status = oZ->GetMetaData('INDEP_VAR_VAL',sampIndepVarVal)
           tolB = 0.005
           bkgdCntBuf = counts*0.0
           bkgdMonBuf = bkgdCntBuf
           bkgdSub = 0
           for j = 0, nBkgd-1 do begin
             oPsetB = bkgdContRef->Get(position=j)
             if (~obj_valid(oPsetB)) then continue
;             status = oPsetB->GetMetaData('sampTypeFlag',sTypeFlagB)
;             if (sTypeFlagB ne 1) then continue  ; only proceed if dataset was originally read in as single crystal
             
             ; retrieve the bkgdSubVarVal for bkgd dataset and compare with
             ; same quantity for signal dataset
             if (tagFound4Samp) then begin
               status = oPsetB->GetMetaData('DATAPTRREF',dataPtr)
               dataStr = *dataPtr
               if (bkgdSubVarTag eq 'HKL') then begin
                 bkgdSubVarValBH = mean((*dataStr.(varHIndex)))                
                 bkgdSubVarValBK = mean((*dataStr.(varKIndex)))
                 bkgdSubVarValBL = mean((*dataStr.(varLIndex)))
                 HNotSame = abs(bkgdSubVarValH-bkgdSubVarValBH) gt bkgdSubVarTol
                 KNotSame = abs(bkgdSubVarValK-bkgdSubVarValBK) gt bkgdSubVarTol
                 LNotSame = abs(bkgdSubVarValL-bkgdSubVarValBL) gt bkgdSubVarTol
                 ; Is the keyword variable same for singnal and bkgd datasets?
                 ; If not then skip to next bkgd dataset
                 if (HNotSame || KNotSame || LNotSame) then continue  ; to next bkgd dataset
               endif else begin
                 bkgdSubVarValB = mean((*dataStr.(varIndex)))
                 ; Is the keyword variable same for singnal and bkgd datasets?
                 ; If not then skip to next bkgd dataset
                 if (abs(bkgdSubVarVal-bkgdSubVarValB) gt bkgdSubVarTol) then continue ; to next bkgd dataset
               endelse
             endif
             
             oZB = oPsetB->GetByName('Counts')
             if (~obj_valid(oZB)) then continue
             
             status = oZB->GetMetaData('INDEP_VAR_TAG',bkgdIndepVarTag)
             status = oZB->GetMetaData('INDEP_VAR_VAL',bkgdIndepVarVal)
             if (sampIndepVarTag ne bkgdIndepVarTag) then continue
             
             status = oZB->GetMetaData('MON',monB)
             if (~oZB->GetMetaData('OriginalCounts',countsB)) then continue
             
             ; now perform subtraction for correspong datapoints in signal and bkgd
             ; datasets that have the same value of the independent variable (scan variable)
             sIndex = []
             bIndex = []
             for k=0, n_elements(bkgdIndepVarVal)-1 do begin
               index = where(abs(sampIndepVarVal - bkgdIndepVarVal[k]) lt tolB, found)
               if (found eq 1) then begin
                 sIndex = [sIndex,index]
                 bIndex = [bIndex,k]
               endif
             endfor
             if (n_elements(sIndex) gt 0) then begin
               ;; accumulate the bkgd counts and corresponding monitor values so that
               ;; we end up with a sum of counts and sum of monitor over all bkgd before doing
               ;; the subtration outside the bkgd loop
               bkgdSub = 1
               bkgdCntBuf[*,sIndex] = bkgdCntBuf[*,sIndex] + countsB[*,bIndex]
               bkgdMonBuf[*,sIndex] = bkgdMonBuf[*,sIndex] + monB[*,bIndex]
             endif
           endfor
           if (bkgdSub) then begin
             index = where(bkgdMonBuf eq 0.0, zeroMonPresent)
             if (zeroMonPresent gt 0) then bkgdMonBuf[index] = 1.0
             scaleFactB = mon / bkgdMonBuf
             counts = temporary(counts) - scaleFactB * bkgdCntBuf
             error = sqrt(temporary(error)^2 + scaleFactB * bkgdCntBuf)
           endif
         endif

         if (effFlag) then begin
            status = oZ->GetMetaData('CHEFF',chEff)
            counts = temporary(counts)/chEff
            error = temporary(error)/sqrt(chEff)
            ;oChEff = oData->GetByName('ChEff')
            ;if (obj_valid(ochEff)) then begin
            ;   status = oChEff->GetData(chEff)
            ;   counts = temporary(counts)/chEff
            ;   error = temporary(error)/sqrt(chEff)
            ;endif
         endif
         
         if (fastFlag) then begin   ; perform corrections that depend on first applying fast bkgd correction
            ;; Apply fast Correction
            if (fastVal ne 0.0) then begin
               counts = temporary(counts) - (time/60.0)*fastVal ; time is measured in secs and fastVal is fast bkgd in cnts/min
            endif
            
            ;; Apply monitor correction
            if (monCorFlag) then begin
               ;; Constraints are:
               ;; 1 - PG monochromator is in use (d spacing = 3.354)
               ;; 2 - 5 <= Ei <= 34; For Ei < 5, flag an error, for Ei > 34, correction = 1
               ;; CF(Ei) = M0 + M1*Ei + M2*Ei^2 + M3*Ei^3 + M4*Ei^4
               status = oData->GetMetaData('DATAPTRREF',dataPtr)
               if (~ptr_valid(dataPtr)) then return
               dspacem = (*dataPtr).dspacem  ; d spacing of monochromator
               diff = abs(dspacem - 3.354)
               PGMON = diff lt 0.001
               Ei = (*(*dataPtr).Ei)
               index = where(Ei lt 5.0,Eilt5)  ; correction not aloowed if Ei < 5.0 meV
               if (Eilt5) then begin
                  msg = ['Ei is lower than 5.0 meV.','Monitor correction was not applied']
                  title = 'Monitor Correction'
                  Self->ErrorMessage,msg,title=title,severity=0
               endif else if (PGMON) then begin
                  M0 =  6.1230
                  M1 = -5.9630e-1
                  M2 =  2.6244e-2
                  M3 = -4.8430e-4
                  M4 =  2.8810e-6
                  index = where(Ei gt 34.0,Eigt34)
                  CFEi = M0 + M1*Ei + M2*Ei^2 + M3*Ei^3 + M4*Ei^4
                  if (Eigt34) then CFEi[index] = 1.0  ; for Ei > 34, CF = 1
                  counts = TEMPORARY(counts)*CFEi
                  error  = TEMPORARY(error)*CFEi 
               endif
            endif 
            
      ;      if (resCorFlag) then begin
      ;         ; apply resolution correction
      ;      endif
      ;      
      ;      if (detBalFlag) rhen begin
      ;         ; apply detailed balance correction
      ;      endif
         endif
         
         ;; Apply monitor normalization
         case normTo of
         
            0: begin
               monScale = monScaleVal/mon 
            end
            
            1: begin
               monScale = monScaleVal/time
            end
            
            else: monScale = monScaleVal
         endcase
         counts = TEMPORARY(counts)*monScale
         error = TEMPORARY(error)*monscale
         
         ;; Mask bad channels
         Self->GetProperty, maskFlag=maskFlag
         sBadChans = Self.maskedDets
         maskedChansExist = ~strcmp(sBadChans,'')
         if (maskFlag && maskedChansExist) then begin
            status = compactstring2intvector(sBadChans, badChans)
            badChanIndex = badChans - 1   ; convert channel nos to 0-based channel index
            ; Apply mask by simple replacing the counts in the masked channels by NaNs
            ; Leave error variable untouched
            counts[badChanIndex,*] = !values.F_NAN
         endif   
         
         maxVal = max(counts, min=minVal,/NAN)
         maxVals = [maxVals,maxVal]
         minVals = [minVals,minVal]
         
         ; Save copy of counts in linearCount metadata 
         oZ->AddMetaData, 'linearCounts',counts
         
         ; take a log of the data if necessary
         if (logFlag) then begin
            index = where(counts le 0.0, nBad)
            if (nBad gt 0) then counts[index] = !values.F_NAN
            counts = alog10(temporary(counts))
         endif
         
         status = oZ->SetData(counts,/no_copy);,/no_notify)
         status = oErr->SetData(error,/no_copy,/no_notify)
      endfor
      
      ; If max and min intensity properties are undefined,
      ; use the current max and min values to define them
      if ((Self.minIntensity eq 0.0) && (Self.maxIntensity eq 0.0)) then begin
         ; NB: This should run at most once during a session!

         minVal = min(minVals)   ; these are linear values!
         maxVal = max(maxVals)
         
         ; take a log of the intensity limits if necessary
         case logFlag of
           0:
           
           1: begin ; log scale is active
              minVal = (minVal gt 0.0)? alog10(minVal) : 0.0
              maxVal = (maxVal gt 0.0)? alog10(maxVal) : 0.0
           end
         endcase

         Self.maxIntensity = maxVal
         Self.minIntensity = minVal
      endif

   end
   
   else:
endcase


end
;-------------------------------------------------------------------------------


;===============================================================================
; Bt7psdTool::UpdateIntensityScale
;+
; PURPOSE:
;   Update the intensity scale - switch between linear/logbase10 scales
;
; PARAMETERS:
;   
;
; KEYWORDS:
;-
pro Bt7psdTool::UpdateIntensityScale
compile_opt idl2

Self->GetProperty, logFlag=logFlag, sampContRef=oCont, minIntensity=minval, maxIntensity=maxval

; must be single crystal sample
if (Self.sampTypeFlag ne 1) then return

nData = oCont->Count()
if (nData eq 0) then return
 
oData = oCont->Get(/all)

case logFlag of
   0: begin ; switch to linear scale
      minVal = 10.0^(minVal)  ; update max and min intensity to the same scale as the intensity
      maxVal = 10.0^(maxVal)
      for i = 0,nData-1 do begin
         oZ = oData[i]->GetByName('Counts')
         if (~obj_valid(oZ)) then continue
         if (~oZ->GetMetaData('LinearCounts',counts)) then continue
         status = oZ->SetData(counts,/no_copy);,/no_notify)      
      endfor
   end
   
   1: begin ; switch to log scale
       minVal = (minVal gt 0.0)? alog10(minVal) : 0.0 ; update max and min intensity to the same scale as the intensity
       maxVal = (maxVal gt 0.0)? alog10(maxVal) : 0.0 
       for i = 0,nData-1 do begin
         oZ = oData[i]->GetByName('Counts')
         if (~obj_valid(oZ)) then continue
         if (~oZ->GetMetaData('LinearCounts',counts)) then begin
            ; probably data from an older session so create the metadata
            status = oZ->Getdata(counts)
            oZ->AddMetaData, 'linearCounts',counts
         endif

         index = where(counts le 0.0, nBad)
         if (nBad gt 0) then counts[index] = !values.F_NAN
         counts = alog10(temporary(counts))
         status = oZ->SetData(counts,/no_copy);,/no_notify)      
      endfor
   
   end
   
   else:
endcase
Self->SetProperty, minIntensity=minVal, maxIntensity=maxVal

; Update the intensity range
Self->UpdatePlotIntensityRange

end
;-------------------------------------------------------------------------------


;===============================================================================
; Bt7psdTool::UpdatePlotIntensityRange
;+
; PURPOSE:
;   Update the max and min intensity of the existing plot
;
; PARAMETERS:
;   
;
; KEYWORDS:
;-
pro Bt7psdTool::UpdatePlotIntensityRange, oVis
compile_opt idl2

Self->GetProperty, defaultIntensityFlag=defIntFlag, sampTypeFlag=sampTypeFlag, maxIntensity=maxVal, minIntensity=minVal

; must be single crystal sample
if (Self.sampTypeFlag ne 1) then return

if (Self.defaultIntensityFlag eq 1) then begin  ;using default intensity range
   Self->GetProperty, selectedIDs=selIDs,nSelectedIDs=nSel
   if (nSel eq 0) then return   ; exit if no dataset is currently selected
   ; Retrieve intensity of selected data
   intensity = []
   for i=0,nSel - 1 do begin
      oTest = Self->GetByIdentifier(selIDs[i])
      if (~isa(oTest,'IDLitParameterSet')) then continue
      status = oTest->GetMetaData('sampTypeFlag',sTypeFlag) ; only consider single crystal data to Mslice
      if (sTypeFlag ne 1) then continue
      oZ = oTest->GetByName('Counts')
      status = oZ->GetData(counts)
      intensity = [intensity,reform(counts,n_elements(counts))]
   endfor
   if (~isa(intensity)) then return
   maxVal = max(intensity,min=minVal,/NAN)
endif else begin
   maxVal = Self.maxIntensity
   minVal = Self.minIntensity
endelse

if (maxVal eq minVal) then return

if (maxVal lt minVal) then begin
   tempVal = maxVal
   maxVal = minVal
   minVal = tempVal
endif

if (obj_valid(oVis) && isa(oVis,'IDLitVisualization')) then begin
   oVis->SetProperty, min_value=minVal, max_value=maxVal
endif else begin
   if (Self->HasVisualizations()) then begin
      oWin = Self->GetCurrentWindow()
      oView = oWin->GetCurrentView()
      oLayer = oView->GetCurrentLayer()
      oWorld = oLayer->GetWorld()
      oDataSpace = oWorld->GetDataSpaces()
      oVis = oDataSpace->GetVisualizations(count = nVis)
   
      for i=0, nVis-1 do oVis[i]->SetProperty, min_value=minVal, max_value=maxVal
   endif
endelse
end
;-------------------------------------------------------------------------------


;===============================================================================
; Bt7psdTool::UpdateIndependentVariables
;+
; PURPOSE:
;   Update the Qx and Qy variables due to changes in the lattice parameter
;
; PARAMETERS:
;   oPSet [in|out] - object whose data is to be updated
;
; KEYWORDS:
;-
pro Bt7psdTool::UpdateIndependentVariables, oPSet, updateObservers=updateObservers
compile_opt idl2

void = oPSet->GetMetaData('DATAPTRREF',dataPtr)
if (~ptr_valid(dataPtr)) then return
dataStr = *dataPtr
Self->GetProperty, LATPARAMFLAG=lpFlag, LATTICEPARAMETERS=latParam_str, indepVar1List=varList, sampTypeFlag=sampTypeFlag 

case (sampTypeFlag) of
   0: begin ; Powder sample
   
   end
   
   1: begin ; Single Crystal sample
      lpLen = strlen(strtrim(latParam_str))
      if (lpFlag) then begin             ; 0 = from data, 1 = user specified
         ; if using user specified lat params
         toks = strsplit(latParam_str,/extract,count=nToks)
         if (nToks eq 6) then latParam = float(toks)
      endif
      if ((lpFlag eq 0) || (n_elements(latParam) eq 0)) then begin
         ; if lat params is undefined or not using user specified lat params
         ; use the dataset's lattice parameters
         latParam = (*dataStr.LATTICE_PTR)
         if (lpLen eq 0) then begin
            ; if the Tool's lattice Parameter property is blank, initialize it with the current dataset's
            latParam_str = strjoin(strtrim(string(float(latParam)),2),' ',/single)
            Self->SetProperty, latticeParameters = latParam_str
            Self->RefreshPropertysheet
         endif
      endif
      
      basis = dm_basis_r(latParam)        ; convert lattice paramters to basis vectors in rec. space for crystal
      
      orient = (*dataStr.ORIENTATION_PTR) ; retrieve crystal orientation vectors u,v in rec. space coordinates
                                          ; These are defined in terms of the rec. space basis vectors
      
      a_star = basis[*,0]                 ; the three basis vectors a*, b*, c* in reciprocal space
      b_star = basis[*,1]
      c_star = basis[*,2]
      
      orient_u = orient[0:2]              ; the crystal orientation vectors specified in multiples of a*, b* and c*
      orient_v = orient[3:5]
      
      ; Q(hkl) = h.a_star + k.b_star + l.c_star
      ; Q vector on TAS is measured in a plane such that two of it's components are determine
      ; along u and v. Using (ki,kf,a3 and a4), the manitude of these components can be determined
      ; in units of inverse angstroms. To convert from inverse angstroms to rel. lattice units (hkl)
      ; we need to divide by the magnitude of the basis vectors along u and v.
      uVec = orient_u[0]*a_star + orient_u[1]*b_star + orient_u[2]*c_star ; the orientation vectors in rec space
      vVec = orient_v[0]*a_star + orient_v[1]*b_star + orient_v[2]*c_star
      magn_uVec = sqrt(dotp(uVec,uVec))   ; the magnitude of the orientation vectors == HKLnorm(orient_u,latparm=latParam)
      magn_vVec = sqrt(dotp(vVec,vVec))
      
      xAA2RLU = magn_uVec                 ; e.g. ~ 2*pi/a  if u = [1,0,0]
      yAA2RLU = magn_vVec                 ; e.g. ~ 2*pi/c  if v = [0,0,1]

      ; determine appropriate labels to use for qx and qy
      xlabrlu = (orient_u[0] eq 0)? '0' : 'H'
      xlabrlu += (orient_u[1] eq 0)? ',0' : ',K'
      xlabrlu += (orient_u[2] eq 0)? ',0' : ',L'
      ylabrlu = (orient_v[0] eq 0)? '0' : 'H'
      ylabrlu += (orient_v[1] eq 0)? ',0' : ',K'
      ylabrlu += (orient_v[2] eq 0)? ',0' : ',L'

      auv = !RADEG*acos(dotp(uVec,vVec)/(magn_uVec*magn_vVec)) ; == getangle2hkl(orient_u,orient_v,latparm=latParam,/degree)
      if (abs(auv - 90.0) gt 0.01) then begin
        ; In case v is not perpendicular to u, w is obtained so that u & w are orthogonal
        orient_w = orient_v - orient_u/magn_uVec*magn_vVec*sin((90.-auv)*!dtor)
        qw = HKLnorm(orient_w,latparm=latParam)
        ;orient_w = orient_w/qw*qv
  
        ; Angle between the two vectors
        ;print,'Orientation vectors and angle'
        ;print,'u: ',uVec
        ;print,'v: ',vVec
        ;print,'angle: ',!RADEG*acos(dotp(uVec,vVec)/(magn_uVec*magn_vVec))
        ;auv = getangle2hkl(orient_u,orient_w,latparm=latParam,/degree)
        
        xAA2RLU = magn_uVec                 ; e.g. ~ 2*pi/a  if u = [1,0,0]
        yAA2RLU = qw                        ; e.g. ~ 2*pi/c  if v = [0,0,1]
        if (orient_w[0] eq 0 || orient_w[0] eq 1) then begin
          ylabrlu = (orient_w[0] eq 0)? '0' : 'H'          
        endif else begin
          ylabrlu = strtrim(string(orient_w[0],format='(F5.2)'),2)+'H'
        endelse
        if (orient_w[1] eq 0 || orient_w[1] eq 1) then begin
          ylabrlu += (orient_w[1] eq 0)? ',0' : ',K'
        endif else begin
          ylabrlu += ','+strtrim(string(orient_w[1],format='(F5.2)'),2)+'K'
        endelse
        if (orient_w[2] eq 0 || orient_w[2] eq 1) then begin
          ylabrlu += (orient_w[2] eq 0)? ',0' : ',L'
        endif else begin
          ylabrlu += ','+strtrim(string(orient_w[2],format='(F5.2)'),2)+'L'
        endelse
      endif
      
      Self->GetProperty, QUNITS=rlu       ;  rlu = 0 => 1/A, 1 => r.l.u.
            
      oQX = oPSet->GetByName(varList[0],count=count)
      if (count ne 1) then return
      oQY = oPSet->GetByName(varList[1],count=count)
      if (count ne 1) then return
      status = oQx->GetMetaData('RLU',unitFlag)    ; unitFlag eq 0 => 1/A, 1 => rlu 
      status = oQx->GetMetaData('AA2RLU',oldxAA2RLU)
      status = oQy->GetMetaData('AA2RLU',oldyAA2RLU)
      
      ; Only need to rescale data if the active units for qx,qy is relative lattice units
      if (rlu eq 1) then begin                     ; rlu is equivalent to unitFlag, just stored at different locations
         ; rescale using updated conversion factor
         status = oQX->GetData(data)
         data = data*(oldxAA2RLU/xAA2RLU)
         status = oQX->SetData(data,/no_copy,/no_notify)
         status = oQY->GetData(data)
         data = data*(oldyAA2RLU/yAA2RLU)
         status = oQY->SetData(data,/no_copy,/no_notify)
         oQx->AddMetaData,'Long_name',xlabrlu
         oQy->AddMetaData,'Long_name',ylabrlu
      endif
      
      ; update metadata and the stored conversion factors
      oQx->AddMetaData,'AA2RLU', xAA2RLU
      oQx->AddMetaData,'LABRLU', xlabrlu
      oQy->AddMetaData,'AA2RLU', yAA2RLU
      oQy->AddMetaData,'LABRLU', ylabrlu
        
   end
   
   else:
endcase


if (keyword_set(updateObservers)) then begin
   oQX->NotifyDataChange
   oQX->NotifyDataComplete
   oQY->NotifyDataChange
   oQY->NotifyDataComplete
endif

end
;-------------------------------------------------------------------------------


;===============================================================================
; Bt7psdTool::UpdateQxQyUnit
;+
; PURPOSE:
;   Update the units for the Qx and Qy variables for all loaded datasets. User can 
;   swtich between inverse angstroms and relative lattice units
;
; PARAMETERS:
;
; KEYWORDS:
;-
pro Bt7psdTool::UpdateQxQyUnit
compile_opt idl2

; Get the QX and QY parameters for all loaded datasets
; and change the units accordingly
Self->GetProperty, sampContRef=oCont, indepVar1List=varList, qUnits=newUnitFlag
nData = oCont->Count()
if (nData eq 0) then return
 
newUnit = (newUnitFlag eq 0)? '$\AA^{-1}$' : 'r.l.u.'
oData = oCont->Get(/all)
for i = 0,nData-1 do begin
   oQX = oData[i]->GetByName(varList[0],count=count)
   if (count ne 1) then continue
   oQY = oData[i]->GetByName(varList[1],count=count)
   if (count ne 1) then continue
   status = oQx->GetMetaData('RLU',oldUnitFlag)
   status = oQx->GetMetaData('AA2RLU',xAA2RLU)
   status = oQy->GetMetaData('AA2RLU',yAA2RLU)
   changed = 0
   if (oldUnitFlag eq 0 and newUnitFlag eq 1) then begin
     status = oQX->GetData(data)
     data = data/xAA2RLU
     status = oQX->SetData(data,/no_copy,/no_notify)
     status = oQY->GetData(data)
     data = data/yAA2RLU
     status = oQY->SetData(data,/no_copy,/no_notify)
     aspect_ratio = yAA2RLU/xAA2RLU
     changed = 1
     status = oQx->GetMetaData('LABRLU',xlab)
     status = oQy->GetMetaData('LABRLU',ylab)
   endif else if (oldUnitFlag eq 1 and newUnitFlag eq 0) then begin
     status = oQX->GetData(data)
     data = data*xAA2RLU
     status = oQX->SetData(data,/no_copy,/no_notify)
     status = oQY->GetData(data)
     data = data*yAA2RLU
     status = oQY->SetData(data,/no_copy,/no_notify)
     aspect_ratio = 1.0
     changed = 1
     status = oQx->GetMetaData('LABANG',xlab)
     status = oQy->GetMetaData('LABANG',ylab)
   endif
   if (changed) then begin
     oQX->AddMetaData,'UNITS', newUnit
     oQX->AddMetaData,'RLU', newUnitFlag
     oQY->AddMetaData,'UNITS', newUnit
     oQY->AddMetaData,'RLU', newUnitFlag
     oQx->AddMetaData,'Long_name',xlab
     oQy->AddMetaData,'Long_name',ylab
     oQX->NotifyDataChange
     oQX->NotifyDataComplete
     oQY->NotifyDataChange
     oQY->NotifyDataComplete
   endif
endfor

if (changed) then begin
  ; If both x and y are in Q then if the units is in 1/A then the aspect_ratio of the 
  ; plot is 1. However when it is r.l.u. then the x and y scales will be different. 
  ; In this, reset the aspect ration to be the same as the ratio of the y and x 
  ; relative lattice units so that the effective aspect ration comes out to be 1 again. 
  ; The aspect_ration value is determined above.
  if (Self->HasVisualizations()) then begin
    oWin = Self->GetCurrentWindow()
    oView = oWin->GetCurrentView()
    oLayer = oView->GetCurrentLayer()
    oWorld = oLayer->GetWorld()
    oDataSpace = oWorld->GetDataSpaces()
    if (obj_valid(oDataSpace)) then begin
      oDataSpace->SetProperty, aspect_ratio = aspect_ratio
    endif
  endif
endif

end
;-------------------------------------------------------------------------------


;===============================================================================
; Bt7psdTool::Mslice
;+
; PURPOSE:
;   Launch Mslice tool with a copy of the currently selected data for further analysis. 
;   If Mslice is already launched, update its data with the currently selected data.
;
; PARAMETERS:
;
; KEYWORDS:
;-
function Bt7psdTool::Mslice
compile_opt idl2

; Basic error Handler
if (n_elements(!debug) && (!debug eq 0)) then begin
    catch, catchError
    if (catchError ne 0) then begin
        if (widget_info(wB,/valid)) then widget_control, wB, /destroy
        Self->StatusMessage, !ERROR_STATE.msg
        catch, /cancel
        return, 0
    endif
endif

oUI = Self->GetUI()
oUI->GetProperty, group_leader = wTLB

; display dialog to let user select variables to be exported to Mslice 
Self->GetProperty, daveTool=daveTool $
                 , exportIndex1=exportIndex1, exportIndex2=exportIndex2, exportIndex3=exportIndex3  $
                 ,indepVar1List=indepVar1List,indepVar2List=indepVar2List,indepVar3List=indepVar3List $
                 ,selectedIDs=selIDs,nSelectedIDs=nSel, sampTypeFlag=sampTypeFlag
if (nSel eq 0) then return, 0  ; exit if no dataset is currently selected

case sampTypeFlag of
  0: begin 
    exportList = indepVar2List
    exportIndex = exportIndex2 
  end; powder
  1: begin 
    exportList = indepVar1List 
    exportIndex = exportIndex1
  end; single crystal
  2: begin 
    exportList = indepVar3List 
    exportIndex = exportIndex3
  end; raw
endcase

; construct basic dialog
title = 'Select Variables to Export'
wB = widget_base(/col,title=title,/modal,group_leader=wTLB)
wB1 = widget_base(wB,/col,grid=2,/frame,/nonexclusive,scr_xsize=200)
wB2 = widget_base(wB,/row,/frame)

n = n_elements(exportList)
wButtons = lindgen(n)
for i=0,n-1 do begin
   wButtons[i] = widget_button(wB1,value=exportList[i],uname=exportList[i])
   widget_control, wButtons[i], set_button=exportIndex[i]
endfor
wOK = widget_button(wB2,value='OK',uname='APPLY',/align_center)
wCancel = widget_button(wB2,value='Cancel',uname='CANCEL',/align_center)

statePtr = ptr_new({wButtons:wButtons,exportIndex:exportIndex,cancel:0})
widget_control, wB, /realize, set_uvalue=statePtr

xmanager, 'BT7PSDExport2Mslice', wB, event_handler='BT7PSDExport2Mslice_event'

exportIndex = (*statePtr).exportIndex
cancel = (*statePtr).cancel
ptr_free, statePtr

if (cancel) then return, 0

if (sampTypeFlag eq 0) then Self->SetProperty, exportIndex2=exportIndex
if (sampTypeFlag eq 1) then Self->SetProperty, exportIndex1=exportIndex
if (sampTypeFlag eq 2) then Self->SetProperty, exportIndex3=exportIndex

index = where(exportIndex eq 1,nVar)
if (nVar lt 2) then return, 0

exportList = exportList[index]

; Init Counts and Error
z = []
Err = []
label = ['Intensity','Error']
unit = ['','']
unique = [0,0]

oData = []

case sampTypeFlag of
  0: begin
    for i=0,nSel - 1 do begin
      oTest = Self->GetByIdentifier(selIDs[i])
      if (~isa(oTest,'IDLitParameterSet')) then continue
      status = oTest->GetMetaData('sampTypeFlag',sTypeFlag) ; must be powder data
      if (sTypeFlag ne 0) then continue
      oZ = oTest->GetByName('Counts')
      status = oZ->GetMetaData('ProcessedCounts',counts)
      status = oZ->GetMetaData('ProcessedErrors',error)
      Z = [Z,reform(counts,n_elements(counts))]
      Err = [Err,reform(error,n_elements(error))]
      oData = [oData,oTest]
    endfor
    nSel = n_elements(oData)
    npts = n_elements(Z)
    if (npts eq 0) then return, 0
    data = fltarr(npts,2+nVar)
    data[*,0] = Z
    data[*,1] = Err
    binWidths = fltarr(2) + 0.1
    ; Get the independent vars
    for i=0,nVar-1 do begin
      varName = exportList[i]
      varData = []
      for j=0,nSel-1 do begin
        oVar = oData[j]->GetByName(varName)
        if (~obj_valid(oVar)) then continue
        if (j eq 0) then begin
          status = oVar->GetMetaData('Long_name',varLab)
          status = oVar->GetMetaData('Units',varUnit)
          label = [label,varLab]
          unit = [unit,varUnit]
          unique = strcmp(varName,'E',/fold_case)? [unique,1] : [unique,0]
          ;unique = [unique,1]
        endif
        oZ = oData[j]->GetByName('Counts')
        metaName = (strcmp(varName,'|Q|'))? 'QTOT' : varName
        if (strcmp(varName,'TEMP',/fold)) then begin
          ; need to treat it specially
          Self->GetProperty, useTempAverageFlag=useTempAverageFlag
          metaName = (useTempAverageFlag)? 'TEMPAVERAGE' : 'TEMP'
        endif
        status = oZ->GetMetaData(metaName,iVarData) ; Get 2D variable values that have not been rebinned!
        npts = n_elements(iVarData)
        iVarData = reform(TEMPORARY(iVarData),npts)
        varData = [vardata,iVarData]
        if (i eq 0 || i eq 1) then begin
          iVarData = ivardata[uniq(ivardata, sort(ivardata))] ; return a copy of the sorted array with duplicate adjacent elements removed
          npts = n_elements(iVarData)
          binWidths[i] = (npts gt 1)? $
            max(abs(iVarData[1:npts-1] - iVarData[0:npts-2])) : $ ; use the max bin width
            1.0
          if (binWidths[i] lt 0.1) then binWidths[i] = 1.15*binWidths[i]
        endif
      endfor
      data[*,i+2] = varData
    endfor

  end 

  1: begin
    for i=0,nSel - 1 do begin
      oTest = Self->GetByIdentifier(selIDs[i])
      if (~isa(oTest,'IDLitParameterSet')) then continue
      status = oTest->GetMetaData('sampTypeFlag',sTypeFlag) ; only export single crystal data to Mslice
      if (sTypeFlag ne 1) then continue
      oZ = oTest->GetByName('Counts')
      if (~oZ->GetMetaData('LinearCounts',counts)) then status = oZ->GetData(counts)
      Z = [Z,reform(counts,n_elements(counts))]
      oErr = oTest->GetByName('Error')
      status = oErr->GetData(error)
      Err = [Err,reform(error,n_elements(error))]
      oData = [oData,oTest]
    endfor
    nSel = n_elements(oData)
    npts = n_elements(Z)
    if (npts eq 0) then return, 0
    data = fltarr(npts,2+nVar)
    data[*,0] = Z
    data[*,1] = Err
    binWidths = fltarr(2) + 0.1
    ; Get the independent vars
    for i=0,nVar-1 do begin
      varName = exportList[i]
      varData = []
      for j=0,nSel-1 do begin
        oVar = oData[j]->GetByName(varName)
        if (~obj_valid(oVar)) then continue
        if (j eq 0) then begin
          status = oVar->GetMetaData('Long_name',varLab)
          status = oVar->GetMetaData('Units',varUnit)
          label = [label,varLab]
          unit = [unit,varUnit]
          unique = strcmp(varName,'E',/fold_case)? [unique,1] : [unique,0]
          ;unique = [unique,1]
        endif
        status = oVar->GetData(iVarData)
        npts = n_elements(iVarData)
        iVarData = reform(TEMPORARY(iVarData),npts)
        varData = [vardata,iVarData]
        if (i eq 0 || i eq 1) then begin
          iVarData = ivardata[uniq(ivardata, sort(ivardata))] ; return a copy of the sorted array with duplicate adjacent elements removed
          npts = n_elements(iVarData)
          binWidths[i] = (npts gt 1)? $
            max(abs(iVarData[1:npts-1] - iVarData[0:npts-2])) : $ ; use the max bin width
            1.0
          if (binWidths[i] lt 0.1) then binWidths[i] = 1.15*binWidths[i]
        endif
      endfor
      data[*,i+2] = varData
    endfor
  end
  
  2: begin
  end
  
  else:
ENDCASE

;; discriminate for any masked data
;void = where(finite(data),complement=mskIndex,ncomplement=nMsk)
;if (nMsk gt 0) then data[mskIndex] = -1.0e20


; Determine the top-level base widget ID for the main DAVE window
; and use this as the group_leader when Mslice is called. This way the Mslice instance will
; be killed automatically only when DAVE goes down rather than if the BT7 reduction module is killed.
oUIMain = daveTool->GetUI()
oUIMain->GetProperty, group_leader = wTLBMain


if (obj_valid(Self.objMslice) && (Self.multipleMsliceWinFlag eq 0)) then begin
   Self.objMslice->Reset_data, data=data, label=tex2idl(label), instrument='BT7' $
                             ,is_uniq=unique, unit=tex2idl(unit), /no_copy
   ; initialize the stepsize params in the Mslice UI to values commensurate with the data
   names = ['dispxstep','dispystep','cutstep']
   values = [binWidths[0],binWidths[1],binWidths[0]]
   ;values = [!values.F_NAN,!values.F_NAN,!values.F_NAN]
   Self.objMslice->script,'set_parameter',name=names,value=values
endif else begin
   dataPtr = ptr_new({data:temporary(data),label:tex2idl(label),unit:tex2idl(unit),instrument:'BT7',is_uniq:unique},/no_copy)
   Self->GetProperty, DAVETOOL=daveTool, DATA_DIRECTORY=dataDir, working_directory=workDir
   dcs_mslice, dataPtr=dataPtr, obj_mslice=objMslice, group_leader=wTLBMain $
             ,DAVETool=daveTool, workDir=workDir, dataDir=dataDir, /no_parmfile
   Self.objMslice = objMslice
   
   ; initialize the stepsize params in the Mslice UI to values commensurate with the data
   names = ['dispxstep','dispystep','cutstep']
   values = [binWidths[0],binWidths[1],binWidths[0]]
   ;values = [!values.F_NAN,!values.F_NAN,!values.F_NAN]
   objMslice->script,'set_parameter',name=names,value=values
endelse

return, 0
end
;-------------------------------------------------------------------------------


;===============================================================================
pro BT7PSDExport2Mslice_event, event
compile_opt idl2
uname = widget_info(event.id,/uname)
widget_control, event.top, get_uvalue=statePtr

case uname of
   'APPLY': begin
      exportIndex = widget_info((*statePtr).wButtons,/button_set)
      (*statePtr).exportIndex = exportIndex
      widget_control, event.top, /destroy  
   end
   
   'CANCEL': begin
      (*statePtr).cancel = 1
      widget_control, event.top, /destroy  
   end
   
   else:
   
endcase
if (strcmp(uname,'APPLY')) then begin
endif

end
;-------------------------------------------------------------------------------


;===============================================================================
; Bt7psdTool::restoreDataVersion
;
; PURPOSE:
;   Restore the specified object to the latest version
;
; PARAMETERS:
;   oData [in|out] - object to be restored to the latest version
;
; KEYWORDS:
;
pro Bt7psdTool::restoreDataVersion, oData
compile_opt idl2

; restore to latest IDLitData version
oData->Restore

; Pull the data pointer structure and ensure it is up to data
currentVersion = Self.dataVersion

void = oData->GetMetaData('DATAPTRREF',tempPtr)
oldDataVersion = (*tempPtr).dataVersion

if (currentVersion gt oldDataVersion) then begin
   ; Update the data as necessary
endif

end


;===============================================================================
; Bt7psdTool::readData
;
; PURPOSE:
;   Read the contents of a .bt4 file
;
; PARAMETERS:
;   file [in] - filename to be read.
;
;   dataPtr [out] - data structure to read file contents into
;
; KEYWORDS:
;
function Bt7psdTool::ReadData, filename, dataPtr, fromFTP=fromFTP
compile_opt idl2

oUI = Self->GetUI()
oUI->GetProperty, group_leader = wTLB

; Basic error Handler
if (n_elements(!debug) && (!debug eq 0)) then begin
    catch, catchError
    if (catchError ne 0) then begin
        ;;print, 'Error handled!'
        eTitle = 'Bt7psdTool::readData: Error encountered'
        eMsg = 'An error or unusual condition was encountered!'
        eMsg = [eMsg,'Please, report the following to the DAVE team:']
        eMsg = [eMsg,!error_state.msg]
        void = dialog_message(/error,eMsg,title=eTitle,dialog_parent=wTLB)
        catch, /cancel
        if (ptr_valid(dataPtr)) then heap_free, dataPtr
        if (n_elements(lun) gt 0) then free_lun, lun, /force
        return, 0
    endif
endif


; check parameter count
if (n_params() ne 2) then return, 0

; check file parameter is defined
if (n_elements(filename) le 0) then return, 0

fromFtp = (stregex(filename,'/pub/ncnrdata/',/fold,/bool))? 1 : 0
if (~fromFTP && ~file_test(filename)) then return, 0

if (keyword_set(fromFTP)) then Self->GetProperty, ftpObject=oFTP

; filename must be an ICE file
if ~is_ice(filename,errmsg = errmsg,oFTP=oFtp) then return, 0

if (fromFTP) then begin
  ; retrieve file contents from ftp server and store into dstring array
  dstring = oFtp->GetFileContent(filename,/string)
  if (strcmp(dstring[0],'')) then return, 0
  nlines = n_elements(dstring)
endif else begin
  read_ok = dave_read_filecontents(filename,dstring=dstring,errmsg=errmsg)
  if ~read_ok then begin
      void = dialog_message(errmsg)
      return, 0
  endif
endelse
parse_ok = dave_parse_ice(dstring,dstruct,header=header $
                          ,dattaText=dattaText, headText=headText $
                          ,indep_var_pos=indep_var_pos $
                          ,dep_var_pos=dep_var_pos $
                          ,lattice=lattice $
                          ,fixedEType=fixedEType $
                          ,fixedEValue=fixedEValue $
                          ,dSpaceM=dSpaceM $
                          ,dSpaceA=dSpaceA $
                          ,orientation=orientation $
                          ,sdesc=sdesc $
                          ,anaPkgStr=anaPkgStr $
                          ,errmsg=errmsg $
                          ,/PSD_DOI_only $
                         )
if (~parse_ok) then begin
  if (keyword_set(quiet)) then begin
    print, 'Error reading file: ',filename
    print, errmsg
  endif else begin
    void = dialog_message([filename,' ',errmsg],/error)
  endelse
  return, 0
endif

if (ptr_valid(dataPtr)) then heap_free, dataPtr
dataPtr =  { filename:'' $
            ,data_ptr:ptr_new()           $
            ,dataVersion:0                $
            ,indep_var_tag:''             $
            ,indep_var_val:ptr_new()      $
;            ,scan:''                      $
;            ,prf:0L                       $
;            ,tot_mon:0L                   $
;            ,num_pts:0                    $
;            ,type:''                      $
;            ,date:''                      $
;            ,time:''                      $
;            ,comments:''                  $
;            ,header_ptr:ptr_new()        $
;            ,headText_ptr:ptr_new()      $
;            ,dattaText_ptr:ptr_new()      $
;            ,fit_ptr:ptr_new()           $
;            ,descriptr:ptr_new()         $
            ,monave:0L                       $
;            ,signal_type:''               $
;            ,ytitle:''                    $
;            ,magfield:0D                    $
;            ,shift_value:0D               $
;            ,plot_ptr:ptr_new()           $
;            ,mcf_ptr:ptr_new()            $
;            ,collim_ptr:ptr_new()         $
;            ,econfig_ptr:ptr_new()        $
;            ,qconfig_ptr:ptr_new()        $
;            ,mosaic_ptr:ptr_new()         $
;            ,instrument_ptr:ptr_new()     $
;            ,motors_ptr:ptr_new()         $
            ,lattice_ptr:ptr_new()        $
;            ,xtal_orientation_ptr:ptr_new()  $
;            ,second_var_name:''           $
;            ,treatment_ptr:ptr_new()      $
            ,orientation_ptr:ptr_new() $
            ,fixedEType:'' $
            ,fixedEValue:0.0 $
            ,dSpaceM:0.0 $
            ,dSpaceA:0.0 $
;            ,isICE:0B $       ; is data from an ICE formated file? 0=>no, 1=>yes
            ,anaPkgPtr:ptr_new() $ ; info about BT7 analyzer package
;            ,oFTP:obj_new() $
;            ,sdesc:'' $
            ,psddets:ptr_new() $
            ,tddets:ptr_new() $
            ,dddets:ptr_new() $
            ,sddets:ptr_new() $
;            ,angles:ptr_new() $
            ,A1:ptr_new() $
            ,A2:ptr_new() $
            ,A3:ptr_new() $
            ,A4:ptr_new() $
            ,A5:ptr_new() $
            ,A6:ptr_new() $
            ,e:ptr_new() $
            ,ei:ptr_new() $
            ,ef:ptr_new() $
            ,h:ptr_new() $
            ,k:ptr_new() $
            ,l:ptr_new() $
            ,mon:ptr_new() $
            ,time:ptr_new() $
            ,temp:ptr_new() $
            ,magfield:ptr_new() $
         }


;(*self.data_file_contents) = dstring
;(*self.header_ptr) = header       ;dstruct.header
;self.display_name = dtas_get_filename(self.filename)
;self.isICE = 1
;
;(*self.headText_ptr) = headText
;(*self.dattaText_ptr) = dattaText

; Ok now we have the entire data structure with titles and all
; stored in dstruct.  Now we need to convert it to the form used
; throughout the TAS data reduction.  In other words...warp it into
; the ICP format.

;; What is the file extension?
;ppos = strpos(self.filename,'.')
;self.extension = strmid(self.filename,ppos+1)
;self.display_name = dtas_get_filename(self.filename)

dataPtr.filename = filename
dataPtr.dataVersion = Self.dataVersion

; Lattice
dataPtr.lattice_ptr = ptr_new(lattice)
; Orientation
dataPtr.orientation_ptr = ptr_new(orientation)
; Fixed anergy and its value
dataPtr.fixedEType = fixedEType
dataPtr.fixedEValue = fixedEValue
; Monochromator and Analyzer d spacing
dataPtr.dspaceA = dspaceA
dataPtr.dspaceM = dspaceM
; Scan description
;dataPtr.sdesc = sdesc

;; Retrieve plottable data
;x = ptr_new(*dstruct.(indep_var_pos))
;xtitle = titles[indep_var_pos]
;y = ptr_new(*dstruct.(dep_var_pos))
;ytitle = titles[dep_var_pos]
;dy = ptr_new(*dstruct.error)

;; BT7 Analyzer package details
;nd = n_elements(*y)
;TDy = fltarr(nd)*0.0
;anaMode = 0
;show_td = 0
dataPtr.indep_var_tag = dattaText[indep_var_pos]
dataPtr.indep_var_val = ptr_new(*dstruct.(indep_var_pos))


if (n_elements(anaPkgStr) gt 0) then begin
    dataPtr.anaPkgPtr = ptr_new(anaPkgStr)
;
;    ;; Retrieve door detector data
;    anaMode = anaPkgStr.mode
;    if (anaMode eq 2 || anaMode eq 3) then begin
;        detsInSum = strsplit(anaPkgStr.TDDetsInSum,';',/extract,count=nDets)
;        if (nDets gt 0) then begin
;            for i=0,nDets-1 do begin
;                ind = where(strupcase(titles) eq strupcase(detsInSum[i]))
;                TDy = TDy + *dstruct.(ind[0]) 
;            endfor
;        endif
;    endif
;    
;    status = tasreducpreferences(tasprefs)
;    if (status) then begin
;      tags = tag_names(tasprefs)
;      index = where(tags eq 'SHOW_TD', found)
;      if (found) then show_td = fix(tasprefs.(index))
;    endif
endif

; The Data block stored as a structure using column headers as field name tags
dataPtr.data_ptr = ptr_new(dstruct)
tags = strupcase(tag_names(dstruct))

; a1 - a6 angles
atags = ['A1','A2','A3','A4','A5','A6']
ntags = 6
dtags = tag_names(dataPtr)
;angles = fltarr(6,ndat)
for i=0,ntags-1 do begin
  index = where(tags eq atags[i], cnt)
;  if cnt then angles[i,*] = (*dstruct.(index))
  if (cnt eq 1) then begin
    index1 = where(dtags eq atags[i], cnt1)
    if (cnt1 eq 1) then dataPtr.(index1) = dstruct.(index)  ; make dataPtr.A* and dstruct.A* point to same heap var
  endif
endfor

; monitor 
index = where(tags eq 'MONITOR',cnt)
if (cnt) then begin
  dataPtr.monave = (*dstruct.(index))[0]
  dataPtr.mon = ptr_new(*dstruct.(index))
endif

; time (counting time)
index = where(tags eq 'TIME',cnt)
if (cnt) then dataPtr.time = ptr_new(*dstruct.(index))

; magfield
index = where(tags eq 'MAGFIELD',cnt)
if (cnt) then begin
  dataPtr.magfield = ptr_new(*dstruct.(index))
endif else begin
  ; magfield not normally defined in ICE intermediate raw file so inialize to zero
  n = n_elements(*dataPtr.mon)
  dataPtr.magfield = ptr_new(fltarr(n))
endelse

; temperature
index = where(tags eq 'TEMP',cnt)
if (cnt) then begin
   dataPtr.temp = ptr_new(*dstruct.(index))
endif else begin
  ; temperature not defined in ICE intermediate raw file so inialize to zero
  n = n_elements(*dataPtr.mon)
  dataPtr.temp = ptr_new(fltarr(n))
endelse

; hkl components
index = where(tags eq 'H',cnt)
if (cnt) then dataPtr.h = ptr_new(*dstruct.(index))
index = where(tags eq 'K',cnt)
if (cnt) then dataPtr.k = ptr_new(*dstruct.(index))
index = where(tags eq 'L',cnt)
if (cnt) then dataPtr.l = ptr_new(*dstruct.(index))
;dataPtr.hkl = ptr_new(transpose([[h],[k],[l]]))

; E (energy transfer)
index = where(tags eq 'E',eFound)
if (eFound) then begin
  eTrans = *dstruct.(index)
  dataPtr.E = ptr_new(eTrans)
endif

; Ei 
index = where(tags eq 'EI',eiFound)
if (eiFound) then dataPtr.Ei = ptr_new(*dstruct.(index))

; Ef
index = where(tags eq 'EF',efFound)
if (efFound) then dataPtr.Ef = ptr_new(*dstruct.(index))

; Some older files did not include Ei, Ef fields but did include
; the 'fixedE' keyword in the header which can be used to determine Ei or Ef given 
; two out of Ei,Ef,E
if ((~eiFound || ~efFound) && eFound) then begin
  fixedET = dataPtr.fixedEType
  fixedEV = dataPtr.fixedEValue
  nDat = n_elements(eTrans)
  if (strcmp(fixedET,'Ei',/fold)) then begin
    ;; ==> Ei is fixed
    ei = fltarr(nDat) + fixedEV
    ef = ei - eTrans
  endif else if (strcmp(fixedET,'Ef',/fold)) then begin
    ;; ==> Ef is fixed
    ef = fltarr(nDat) + fixedEV
    ei = ef + eTrans
  endif else begin
    ;; calculate Ei from A2;  
    ;; Ei = 2.0722*Ki^2 where ki = 2*pi/lambda_i and lambda_i = 2*(dSpaceM)*sin(A2/2);
    ei = Self.ekFac * (2*!pi/(2*dSpaceM*sin(0.5*(*dataPtr.a2)*!dtor)))^2
    ef = ei - eTrans
  endelse
  dataPtr.Ei = ptr_new(ei)
  dataPtr.Ef = ptr_new(ef)  
endif

; In 2-axis mode, A6 is expected to be zero. however, there sometimes rounding-off errors in ICE
; that results in A6 being asigned a small value hence screwing up energy calculations. Hence 
; if A6 is small (ie zero within tolerance), then assumme we are running in 2-axis mode and therefore
; set Ei=Ef and determine the value from A2.
A6_tolerance = 0.05
small = 0.001
A6 = (*dataPtr.A6)
n = n_elements(A6)
notVarying = abs(A6[n-1] - A6[0]) le small
isZero = abs(a6[0]) le A6_tolerance
if (isZero && notVarying) then begin
  ; data likely collected in 2-Axis mode
  ; so recalculate energy values
  Ei = Self.ekFac * (2*!pi/(2*dSpaceM*sin(0.5*(*dataPtr.a2)*!dtor)))^2
  (*dataPtr.Ei) = Ei
  (*dataPtr.Ef) = Ei  
  (*dataPtr.E) = Ei*0.0  
endif 

; psd detectors from PSD Detector package
dettags = strupcase(anaPkgStr.psddets)
ndets = n_elements(dettags)
ndat = n_elements(etrans)
dets = fltarr(ndets,ndat)
for i=0,ndets-1 do begin
  index = where(tags eq dettags[i], cnt)
  if cnt then dets[i,*] = (*dstruct.(index))
endfor
dataPtr.psddets = ptr_new(dets)

; door detectors from PSD Detector package
dettags = strupcase(anaPkgStr.tddets)
ndets = n_elements(dettags)
dets = fltarr(ndets,ndat)
for i=0,ndets-1 do begin
  index = where(tags eq dettags[i], cnt)
  if cnt then dets[i,*] = (*dstruct.(index))
endfor
dataPtr.tddets = ptr_new(dets)

; diffraction detectors from PSD Detector package
dettags = strupcase(anaPkgStr.dddets)
ndets = n_elements(dettags)
dets = fltarr(ndets,ndat)
for i=0,ndets-1 do begin
  index = where(tags eq dettags[i], cnt)
  if cnt then dets[i,*] = (*dstruct.(index))
endfor
dataPtr.dddets = ptr_new(dets)

; single detectors from PSD Detector package
dettags = strupcase(anaPkgStr.sddets)
ndets = n_elements(dettags)
dets = fltarr(ndets,ndat)
for i=0,ndets-1 do begin
  index = where(tags eq dettags[i], cnt)
  if cnt then dets[i,*] = (*dstruct.(index))
endfor
dataPtr.sddets = ptr_new(dets)


;mon_counts = strtrim(string(self.tot_mon,format = '(e15.3)'),2)
;ytitle = 'COUNTS PER '+mon_counts+' MONITOR'
;self.ytitle = ytitle
;plot_str = {x:x,y:y,dy:dy,TDy:ptr_new(TDy),anaMode:anaMode,show_td:show_td $
;            ,xtitle:xtitle,ytitle:self.ytitle,title:self.display_name}
;*self.plot_ptr = plot_str
;new_treatment = 'Raw data file read in: '+self.display_name
;ret = self->update_treatment(new_treatment)

;; Set the descriptive variable to the average temperature of the file
;; as a starting point (default).
;ret = self->udpate_descriptr(/temperature)
;ret = self->get_magfield()

dataPtr = ptr_new(dataPtr,/no_copy)
return, 1
end
;-------------------------------------------------------------------------------


;===============================================================================
; Bt7psdTool::UpdatePropertySheet
;
; PURPOSE:
;   Update the propertysheet to reflect the current sample type
;
; PARAMETERS:
;
; KEYWORDS:
;    typeCheck - if datasets already loaded use dialog to warn about possibility of samplt type change
;
pro Bt7psdTool::UpdatePropertySheet, typeCheck=typeCheck
compile_opt idl2

Self->GetProperty, SAMPCONTREF=oSamp, sampTyps=sampTyps, sampTypeFlag=sampTypeFlag
nSamp = oSamp->Count()
if (nSamp gt 0 && keyword_set(typeCheck)) then begin
   sampType=sampTyps[sampTypeFlag]
   msg = 'Not advisable to change Sample Type with data already loaded!'
   msg = [msg,'Do you wish to reload all datasets as "'+sampType+'" first?']
   title = 'Sample Type has Changed: Reload All Data?'
   status = Self->PromptUserYesNo(msg,answer,title=title)
   if (status && answer) then $
      void = Self->ReloadAllDatasets() ;Self->DeleteAllDatasets(/noPrompt)
endif

sxProps = ptr_valid(Self.sxtalOnlyPropsPtr)? (*Self.sxtalOnlyPropsPtr) : []
nsxProps = n_elements(sxProps)
pdProps = ptr_valid(Self.powderOnlyPropsPtr)? (*Self.powderOnlyPropsPtr) : []
npdProps = n_elements(pdProps)
sxPdProps = ptr_valid(Self.sxtalPowderPropsPtr)? (*Self.sxtalPowderPropsPtr) : []
nSxPdProps = n_elements(sxPdProps)
rwProps = ptr_valid(Self.rawOnlyPropsPtr)? (*Self.rawOnlyPropsPtr) : []
nrwProps = n_elements(rwProps)

oUI = Self->GetUI()
uiPresent = obj_valid(oUI)
if (uiPresent) then begin
   oUI->GetProperty, group_leader = wTLB
   wChild = widget_info(wTLB, /child)
   widget_control, wChild, get_uvalue=sPtr
endif

Self->SetPropertyAttribute, 'BKGDFLAG', sensitive=(Self.nBkgd gt 0)
Self->SetPropertyAttribute, 'bkgdSubVarFlag', sensitive=(Self.bkgdFlag eq 1)
Self->SetPropertyAttribute, 'bkgdSubVarTol', sensitive=(Self.bkgdFlag eq 1)

case Self.sampTypeFlag of

   0: begin ; powder
      for i = 0, npdProps-1 do Self->SetPropertyAttribute, pdProps[i], hide=0
      for i = 0, nSxPdProps-1 do Self->SetPropertyAttribute, sxPdProps[i], hide=0

      for i = 0, nsxProps-1 do Self->SetPropertyAttribute, sxProps[i], hide=1
      for i = 0, nrwProps-1 do Self->SetPropertyAttribute, rwProps[i], hide=1

      ; fix label for x-axis variable
      Self->SetPropertyAttribute,'xAxisVar', name='Scan Variable', enumlist=(*Self.indepVars2Ptr)

      if (uiPresent) then begin
         ;widget_control, (*sPtr).wMslice, sensitive=0
         ;; unmap the sxtalbase and map the powderbase
         ;widget_control, (*sPtr).wSxtalBase, map=0
         ;widget_control, (*sPtr).wPowderBase, map=1
      endif
   end
   
   1: begin ; single crystal
      for i = 0, nsxProps-1 do Self->SetPropertyAttribute, sxProps[i], hide=0
      for i = 0, nSxPdProps-1 do Self->SetPropertyAttribute, sxPdProps[i], hide=0

      for i = 0, npdProps-1 do Self->SetPropertyAttribute, pdProps[i], hide=1
      for i = 0, nrwProps-1 do Self->SetPropertyAttribute, rwProps[i], hide=1

      ; fix label for x-axis variable
      Self->SetPropertyAttribute,'xAxisVar', name='Horizontal Axis Variable', enumlist=(*Self.indepVars1Ptr)

      ; fix label for y-axis variable
      Self->SetPropertyAttribute,'yAxisVar', name='Vertical Axis Variable', enumlist=(*Self.indepVars1Ptr)
      Self->GetProperty, yAxisVar=yAxisVar
      if (yAxisVar ge n_elements((*Self.indepVars1Ptr)) ) then begin
         Self->SetProperty, yAxisVar = 1
      endif
      
      Self->SetPropertyAttribute, 'maxIntensity', hide=(Self.defaultIntensityFlag eq 1)
      Self->SetPropertyAttribute, 'minIntensity', hide=(Self.defaultIntensityFlag eq 1)
      if (uiPresent) then begin
         ;widget_control, (*sPtr).wMslice, sensitive=1
      endif
   end
   
   2: begin ; Raw View
      for i = 0, nrwProps-1 do Self->SetPropertyAttribute, rwProps[i], hide=0
      
      for i = 0, nsxProps-1 do Self->SetPropertyAttribute, sxProps[i], hide=1    
      for i = 0, npdProps-1 do Self->SetPropertyAttribute, pdProps[i], hide=1
      for i = 0, nSxPdProps-1 do Self->SetPropertyAttribute, sxPdProps[i], hide=1

      ; fix label for x-axis variable
      Self->SetPropertyAttribute,'xAxisVar', name='Plot Type', enumlist=(*Self.rawModesPtr)
      Self.xAxisVar = (Self.xAxisVar > 0) < 2
      xAxisVar = Self.xAxisVar
      ;if ((xAxisVar lt 0) || (xAxisVar gt 2)) then xAxisVar = 0
      case xAxisVar of
         0: begin ; Cnts vs Channel
            Self->SetPropertyAttribute,'yAxisVar', hide=1
            Self->SetPropertyAttribute,'minChan', hide=1
            Self->SetPropertyAttribute,'maxChan', hide=1
            Self->SetPropertyAttribute,'minDPnt', hide=1
            Self->SetPropertyAttribute,'maxDPnt', hide=1
            Self->SetPropertyAttribute,'sharedSettingsFlag', hide=0
            Self->GetProperty, selectedIDs=selIDs, nSelectedIDs=nSel, curMaxDataLen=maxLen $
                             , settingIndex=sIndex, dpntIndex=dpntIndex
            hideSI = Self.sharedSettingsFlag || (nSel lt 2)
            showSI = ~Self.sharedSettingsFlag && (nSel ge 2)
            Self->SetPropertyAttribute,'settingIndex', hide=hideSI, name='Dataset name:' $
                                      , enumlist = file_basename(selIDs)
            if (hideSI) then begin
               Self.dpntIndex = Self.dpntIndex < maxLen
               Self->SetPropertyAttribute,'dpntIndex', hide=0, valid_range=[1,maxLen,1]
            endif else begin
               oItem = Self->GetByIdentifier(selIDs[sIndex])
               if (obj_valid(oItem)) then begin
                  status = oItem->GetMetaData('NPTS',nPts)
                  dpntIndex = (dpntIndex > 1) < nPts
                  Self.dpntIndex = dpntIndex
                  oItem->AddMetaData, 'DPNTINDEX', dpntIndex
                  Self->SetPropertyAttribute, 'dpntIndex', valid_range=[1,nPts,1]
               endif
            endelse
         end
         
         1: begin ; Integrated Cnts vs Channel
            Self->SetPropertyAttribute,'yAxisVar', hide = 1
            Self->SetPropertyAttribute,'minChan', hide=1
            Self->SetPropertyAttribute,'maxChan', hide=1
            Self->SetPropertyAttribute,'dpntIndex', hide=1
            Self->SetPropertyAttribute,'sharedSettingsFlag', hide=0
            Self->GetProperty, selectedIDs=selIDs, nSelectedIDs=nSel, curMaxDataLen=maxLen $
                             , settingIndex=sIndex, minDPnt=minDPnt, maxDPnt=maxDPnt
            hideSI = Self.sharedSettingsFlag || (nSel lt 2)
            Self->SetPropertyAttribute,'settingIndex', hide=hideSI, name='Dataset name:' $
                                      , enumlist = file_basename(selIDs)

            if (hideSI) then begin
               Self.maxChan = Self.maxChan < maxLen
               Self->SetPropertyAttribute,'minDPnt', hide=0, valid_range=[1,maxLen,1]
               Self->SetPropertyAttribute,'maxDPnt', hide=0, valid_range=[1,maxLen,1]
            endif else begin
               oItem = Self->GetByIdentifier(selIDs[sIndex])
               if (obj_valid(oItem)) then begin
                  status = oItem->GetMetaData('NPTS',nPts)
                  minDPnt = minDPnt > 1
                  maxDPnt = maxDPnt < nPts
                  oItem->AddMetaData, 'MINDPNT', minDPnt
                  oItem->AddMetaData, 'MAXDPNT', maxDPnt
                  Self->SetPropertyAttribute,'minDPnt', hide=0, valid_range=[1,nPts,1]
                  Self->SetPropertyAttribute,'maxDPnt', hide=0, valid_range=[1,nPts,1]
               endif
            endelse


         end
         
         2: begin ; Integrated Cnts vs Scan Variable
            Self->GetProperty, indepVar3List=scanList
            Self->SetPropertyAttribute,'yAxisVar', hide = 0, name='Scan Variable', enumlist=scanList
            Self->SetPropertyAttribute,'minDPnt', hide=1
            Self->SetPropertyAttribute,'maxDPnt', hide=1
            Self->SetPropertyAttribute,'dpntIndex', hide=1
            Self->SetPropertyAttribute,'sharedSettingsFlag', hide=1
            Self->SetPropertyAttribute,'settingIndex', hide=1
            Self.maxChan = Self.maxChan < 48
            Self->SetPropertyAttribute,'minChan', hide=0, name = 'Min Channel to sum', valid_range=[1,48,1]
            Self->SetPropertyAttribute,'maxChan', hide=0, name = 'Max Channel to sum', valid_range=[1,48,1]         
         end
         
         else:
      endcase
   end
   
   else:
endcase

;Self->refreshPropertySheet

end
;-------------------------------------------------------------------------------


;===============================================================================
;+
; Bt7psdTool::Plot
;
; PURPOSE:
;   Generates a plot of the specified dataset
;
; PARAMETERS:
;   oItem   - dataset to be plotted
;   sPtr    - heap variable containing useful widget related info
;   overPlotFlag  - is overplotting on (1) or off (0)
;   lastPltFlag   - if overplotting, is this the last plot to be included
;
; KEYWORDS:
;-
pro Bt7psdTool::Plot, oItem, sPtr, overPlotFlag, lastPlotFlag, position=pos
compile_opt idl2

Self->GetProperty, sampTypeFlag = sampTypeFlag

if (sampTypeFlag eq 0) then Self->PlotPowder, oItem, sPtr, overPlotFlag, lastPlotFlag, position=pos
if (sampTypeFlag eq 1) then Self->PlotSinglecrystal, oItem, sPtr, overPlotFlag, lastPlotFlag
if (sampTypeFlag eq 2) then Self->PlotRaw, oItem, sPtr, overPlotFlag, lastPlotFlag, position=pos

end
;-------------------------------------------------------------------------------


;===============================================================================
;+
; Bt7psdTool::PlotRaw
;
; PURPOSE:
;   Generates a line or contour plot of the specified dataset for raw data view
;
; PARAMETERS:
;   oItem   - dataset to be plotted
;   sPtr    - heap variable containing useful widget related info
;   overPlotFlag  - is overplotting on (1) or off (0)
;   lastPltFlag   - if overplotting, is this the last plot to be included
;
; KEYWORDS:
;-
pro Bt7psdTool::PlotRaw, oItem, sPtr, overPlotFlag, lastPlotFlag, position=pos
compile_opt idl2

if (~obj_isa(oItem, 'IDLitParameterSet')) then return
if (oItem->GetMetaDataCount() eq 0) then return

; Ensure dataset was processed for 'Raw Data' view
status = oItem->GetMetaData('sampTypeFlag',sTypeFlag) 
if (sTypeFlag ne 2) then begin
  stypes = ['Powder','Single Crystal','Raw Data']
  msg = 'Cannot plot dataset; It is of Sample Type: "'+stypes[sTypeFlag]+'" which does not match current setting!'
  Self->StatusMessage, msg
  return
endif

    ;; Plot the data
Self->GetProperty, xAxisVar=mode, yAxisVar=xAxisVar, indepVar3List=varList, calibParamsisSet=calibParamsisSet $
                 , minChan=minChan,maxChan=maxChan,minDPnt=minDPnt,maxDPnt=maxDPnt,dpntIndex=dpntIndex $
                 , MONSCALEVALUE=monScaleVal, NORMALIZETO=normTo, effNormFlag=effFlag $
                 , sampTypeFlag=sampTypeFlag, logFlag=logFlag, nSelectedIDs=nSel $
                 ,bkgdFlag=bkgdFlag, nBkgd=nBkgd, bkgdContRef=bkgdContRef $
                 ,bkgdSubVarTol=bkgdSubVarTol, bkgdSubVarFlag=bkgdSubVarFlag

status = oItem->GetMetaData('OriginalCounts',counts)
error = counts
index = where(error le 0.0,cnt)
if (cnt gt 0) then error[index] = 1.0
error = sqrt(error)

status = oItem->GetMetaData('MON',mon)
status = oItem->GetMetaData('TIME',time)

;; Perform bkgd subtraction if appropriate
status = oItem->GetMetaData('DATAPTRREF',dataPtr)
dataStr = *dataPtr
bkgdSubVarTag = strupcase(Self.bkgdSubVars[bkgdSubVarFlag])
tagNames = tag_names(dataStr)
if (bkgdSubVarTag eq 'HKL') then begin
  ; replace 'HKL' with 'H','K' and 'L'
  varHIndex = where(tagNames eq 'H', HtagFound4Samp)
  varKIndex = where(tagNames eq 'K', KtagFound4Samp)
  varLIndex = where(tagNames eq 'L', LtagFound4Samp)
  tagFound4Samp = HtagFound4Samp && KtagFound4Samp && LtagFound4Samp
  if (tagFound4Samp) then begin
    bkgdSubVarValH = mean((*dataStr.(varHIndex)))
    bkgdSubVarValK = mean((*dataStr.(varKIndex)))
    bkgdSubVarValL = mean((*dataStr.(varLIndex)))
  end
endif else begin
  varIndex = where(tagNames eq bkgdSubVarTag, tagFound4Samp)
  if (tagFound4Samp) then bkgdSubVarVal = mean((*dataStr.(varIndex)))
endelse

oItem->GetProperty, type=type
if (strcmp(type,'samp') && bkgdFlag && (nBkgd gt 0)) then begin
  status = oItem->GetMetaData('INDEP_VAR_TAG',sampIndepVarTag)
  status = oItem->GetMetaData('INDEP_VAR_VAL',sampIndepVarVal)
  tolB = 0.005  
  bkgdCntBuf = counts*0.0
  bkgdMonBuf = bkgdCntBuf
  bkgdSub = 0
  for j = 0, nBkgd-1 do begin
    oPsetB = bkgdContRef->Get(position=j)
    if (~obj_valid(oPsetB)) then continue
    ;        status = oPsetB->GetMetaData('sampTypeFlag',sTypeFlagB)
    ;        if (sTypeFlagB ne 2) then continue  ; only proceed if dataset was originally read in as powder
    
    ; retrieve the bkgdSubVarVal for bkgd dataset and compare with
    ; same quantity for signal dataset
    if (tagFound4Samp) then begin
      status = oPsetB->GetMetaData('DATAPTRREF',dataPtr)
      dataStr = *dataPtr
      if (bkgdSubVarTag eq 'HKL') then begin
        bkgdSubVarValBH = mean((*dataStr.(varHIndex)))                
        bkgdSubVarValBK = mean((*dataStr.(varKIndex)))
        bkgdSubVarValBL = mean((*dataStr.(varLIndex)))
        HNotSame = abs(bkgdSubVarValH-bkgdSubVarValBH) gt bkgdSubVarTol
        KNotSame = abs(bkgdSubVarValK-bkgdSubVarValBK) gt bkgdSubVarTol
        LNotSame = abs(bkgdSubVarValL-bkgdSubVarValBL) gt bkgdSubVarTol
        ; Is the keyword variable same for singnal and bkgd datasets?
        ; If not then skip to next bkgd dataset
        if (HNotSame || KNotSame || LNotSame) then continue  ; to next bkgd dataset
      endif else begin
        bkgdSubVarValB = mean((*dataStr.(varIndex)))
        ; Is the keyword variable same for singnal and bkgd datasets?
        ; If not then skip to next bkgd dataset
        if (abs(bkgdSubVarVal-bkgdSubVarValB) gt bkgdSubVarTol) then continue ; to next bkgd dataset
      endelse
    endif
    
    status = oPsetB->GetMetaData('INDEP_VAR_TAG',bkgdIndepVarTag)
    status = oPsetB->GetMetaData('INDEP_VAR_VAL',bkgdIndepVarVal)
    if (sampIndepVarTag ne bkgdIndepVarTag) then continue
    
    status = oPsetB->GetMetaData('MON',monB)
    if (~oPsetB->GetMetaData('OriginalCounts',countsB)) then continue
    
    ; now perform subtraction for correspong datapoints in signal and bkgd
    ; datasets that have the same value of the independent variable (scan variable)
    sIndex = []
    bIndex = []
    for k=0, n_elements(bkgdIndepVarVal)-1 do begin
      index = where(abs(sampIndepVarVal - bkgdIndepVarVal[k]) lt tolB, found)
      if (found eq 1) then begin
        sIndex = [sIndex,index]
        bIndex = [bIndex,k]
      endif
    endfor
    if (n_elements(sIndex) gt 0) then begin
      ;; accumulate the bkgd counts and corresponding monitor values so that
      ;; we end up with a sum of counts and sum of monitor over all bkgd before doing
      ;; the subtration outside the bkgd loop
      bkgdSub = 1
      bkgdCntBuf[*,sIndex] = bkgdCntBuf[*,sIndex] + countsB[*,bIndex]
      bkgdMonBuf[*,sIndex] = bkgdMonBuf[*,sIndex] + monB[*,bIndex]
    endif
  endfor
  if (bkgdSub) then begin
    index = where(bkgdMonBuf eq 0.0, zeroMonPresent)
    if (zeroMonPresent gt 0) then bkgdMonBuf[index] = 1.0
    scaleFactB = mon / bkgdMonBuf
    counts = temporary(counts) - scaleFactB * bkgdCntBuf
    error = sqrt(temporary(error)^2 + scaleFactB * bkgdCntBuf)
  endif
endif

; Efficiency correction
if (effFlag) then begin
   status = oItem->GetMetaData('CHEFF',chEff)
   counts = temporary(counts)/chEff
   error = temporary(error)/sqrt(chEff)
endif

;; Mask bad channels
Self->GetProperty, maskFlag=maskFlag
sBadChans = Self.maskedDets
maskedChansExist = ~strcmp(sBadChans,'')
if (maskFlag && maskedChansExist) then begin
   status = compactstring2intvector(sBadChans, badChans)
   badChanIndex = badChans - 1   ; convert channel nos to 0-based channel index
   ; Apply mask by simple replacing the counts in the masked channels by NaNs
   ; Leave error variable untouched
   counts[badChanIndex,*] = !values.F_NAN
   error[badChanIndex,*] = 0.0
endif


;; Apply monitor normalization
case normTo of
   0: begin
      monScale = monScaleVal/mon
   end
   
   1: begin
      monScale = monScaleVal/time
   end
   
   else: monScale = monScaleVal
endcase
counts = TEMPORARY(counts)*monScale
error = TEMPORARY(error)*monscale

oZ = oItem->GetByName('Counts')
oErr = oItem->GetByName('Error')

PSDDims = size(counts,/dimensions)  ; dimension lenghts
status = oItem->GetMetaData('NPTS',npts)  ; determine nos of datapoints in dataset


showSI = ~Self.sharedSettingsFlag && (nSel ge 2)
if (showSI) then begin
   status = oItem->GetMetaData('DPNTINDEX', dpntIndex)
   status = oItem->GetMetaData('MINDPNT', minDPnt)
   status = oItem->GetMetaData('MAXDPNT', maxDPnt)
endif

status = oItem->GetMetaData('CHDELA4',chDelA4)

case mode of
   0: begin    ; Cts vs Channel
      ;if (~isa(xAxisVar)) then xAxisVar = 0
      if (npts gt 1) then begin
         index = ((dpntIndex-1) > 0) < (PSDDims[1] - 1) ; 0 < xAxisVar < npts-1
         counts = counts[*,index]
         error = error[*,index]
      endif else begin
         ; dataset has a single datapoint, no need to do anything
         index = 0
      endelse
      
      if (Self.calibParamsIsSet) then begin
        ; Determine the A4 accross PSD from norminal A4 and delta A4 (= A4o- DelA4)
        ; and set this to the PSDChannelA4 object before plotting
        oA4 = oItem->GetByName('A4')
        status = oA4->GetData(A4o)
        chA4 = A4o[index] - chDelA4 
        oX = oItem->GetByName('PSDChannelA4',count=cnt)
        status = oX->SetData(chA4,/no_copy,/no_notify)
      endif else begin
        ; just use the PSD Channel index
        oX = oItem->GetByName('PSDChannel',count=cnt)
      endelse
      
      ; If the scan variable was identified from the raw data, then use it in the 
      ; y-axis label in addition to the datapoint index nos
      status = oItem->GetMetaData('INDEP_VAR_TAG',indep_var_tag)
      oIndepVar = oItem->GetByName(indep_var_tag)
      if (obj_valid(oIndepVar)) then begin
        status = oIndepVar->GetData(IV_value)
        status = oIndepVar->GetMetaData('Long_name',IV_name)
        oZ->AddMetaData,'Long_name','Counts, dPnt='+strtrim(string(index),2)+', '+IV_name+'='+strtrim(string(IV_value[index]),2)
      endif else begin 
        oZ->AddMetaData,'Long_name','Counts, dPnt='+strtrim(string(index),2)
      endelse
   end
   
   1: begin    ; Integrated Cts (data points) vs Channel
      if (npts gt 1) then begin
         ;; dataset has moe than 1 datapoint
         if (minDPnt gt maxDPnt) then begin
            Self.maxDPnt = minDPnt
            Self.minDPnt = maxDPnt
            minDPnt = Self.minDPnt
            maxDPnt = Self.maxDPnt
            ;Self->RefreshPropertySheet
         endif
         index =  minDPnt - 1 + indgen(maxDPnt - minDPnt + 1)  ; minDPnt -1 converts pnt nos to index nos
         nIndex = n_elements(index)
         if (nIndex gt 1) then begin
            counts = total(counts[*,index],2,/NAN)   ; integrated along data point from minDPnt to maxDPnt
            error = sqrt(total(error[*,index]^2,2,/NAN))
         endif else begin
            counts = reform(counts[*,index])
            error = reform(error[*,index])      
         endelse
      endif else begin
         ;; dataset has only 1 datapoint
         minDPnt = 1
         maxDPnt = 1
      endelse
      
      oX = oItem->GetByName('PSDChannel',count=cnt)
      strMinDPnt = strtrim(string(minDPnt),2)
      strMaxDPnt = strtrim(string(maxDPnt),2)
      oZ->AddMetaData,'Long_name','$\sum_{dPnt='+strMinDPnt+'}^{'+strMaxDPnt+'}$ Counts'
   end
   
   2: begin    ; Integrated Cts (detector channels) vs ScanVariable
      if (minChan gt maxChan) then begin
         Self.maxChan = minChan
         Self.minChan = maxChan
         minChan = Self.minChan
         maxChan = Self.maxChan
         ;Self->RefreshPropertySheet
      endif
      index =  minChan - 1 + indgen(maxChan - minChan + 1)  ; minChan -1 converts channels to index nos
      nIndex = n_elements(index)
      if (nIndex gt 1) then begin
         counts = total(counts[index,*],1,/NAN)   ; integrated along det channel from minChan to maxChan
         error = sqrt(total(error[index, *]^2,1,/NAN))
      endif else begin
         counts = reform(counts[index,*])
         error = reform(error[index,*])      
      endelse
      
      xvar = varList[xAxisVar]
      oX = oItem->GetByName(xvar,count=cnt)
      if (cnt ne 1) then begin
         oX = oItem->GetByName('Data_Index',count=cnt)
         msg = ['Scan variable "'+xvar+'" not found!','Switching to "Data_Index"']
         Self->ErrorMessage, msg, severity=0
      endif
      strMinChan = strtrim(string(minChan),2)
      strMaxChan = strtrim(string(maxChan),2)
      oZ->AddMetaData,'Long_name','$\sum_{chan='+strMinChan+'}^{'+strMaxChan+'}$ Counts'
   end
   
   else: return

endcase

ctype = partype(counts)

if (ctype ne 'IDLVECTOR') then return  ; can't crate a plot from non-vector data

status = oZ->SetData(counts,/no_copy,/no_notify)
status = oErr->SetData(error,/no_copy,/no_notify)

oZ->SetProperty, type=ctype
oX->SetProperty, type=ctype
oErr->SetProperty, type=ctype

strparms = ['Y','X','Y ERROR','X ERROR','VERTICES','PALETTE','VERTEX_COLORS']
idparms = [oZ->GetFullIdentifier() $
          ,oX->GetFullIdentifier() $
          ,oErr->GetFullIdentifier() $
          ,'','','','']
visID = 'PLOT'
opID = 'Operations/Insert Visualization'
oRequester = (Self->GetByIdentifier(opID))->GetObjectInstance()
oRequester->_setTool, Self
oRequester->GetProperty, show_execution_ui = orig_showUI
oRequester->SetProperty, show_execution_ui = 0

oItem->GetProperty, name=plotName   ; retrieve the dataset name as shown in the data manager and set it as plotname
oRequester->SetProperty, name=plotName, parameter_names=strParms, data_ids=idParms, visualization_id=visID

oWin = Self->GetCurrentWindow()
oView = oWin->GetCurrentView()
activateManip = 0
rangeChange = 0

noOverplot = overplotFlag eq 0
if (Self->HasVisualizations() && noOverplot) then begin
   ;; store the active manipulator
   (*sPtr).manipID = (Self->GetCurrentManipulator())->GetFullIdentifier()
   activateManip = 1

   ;; ensure the existing visualization is selected
   oLayer = oView->GetCurrentLayer()
   oWorld = oLayer->GetWorld()
   oDataSpace = oWorld->GetDataSpaces()
   oVis = oDataSpace->GetVisualizations(count = nVis)
    
   ; retrieve current xrange and yrange to determine if user has zoomed plot
   oDataSpace->GetProperty, xrange=xr, yrange=yr
   xChange = ((*sPtr).xrange[0] ne xr[0]) || ((*sPtr).xrange[1] ne xr[1]) 
   yChange = ((*sPtr).yrange[0] ne yr[0]) || ((*sPtr).yrange[1] ne yr[1]) 
   rangeChange = xChange && yChange
   
   delID = 'Operations/Edit/Delete'
   oDelete = (Self->GetByIdentifier(delID))->GetObjectInstance()
   
   oDataSpace->Select
   void = oDelete->DoAction(Self)

   obj_destroy, [oVis,oDataSpace]

   ; delete any residual annotations that may have been added
   annID = oView->FindIdentifiers('*ANNOTATION LAYER*',count=nAnn)
   if (nAnn gt 2) then begin
     oVoid = []
     for i=2, nAnn-1 do begin
       oTmp = Self->GetByIdentifier(annID[i])
       oTmp->Select
       oVoid = [oVoid,oTmp]
     endfor
     void = oDelete->DoAction(Self) ; delete selected anootations
     obj_destroy, oVoid
   endif
endif

; create the new vis
oCmd = oRequester->DoAction(Self)

;; Get the newly created vis
oLayer = oView->GetCurrentLayer()
oWorld = oLayer->GetWorld()
oDataSpace = oWorld->GetDataSpaces()
oVis = oDataSpace->GetVisualizations(count = nVis)

; TODO - fix vis cuztomization
; Customize it
colors = [[0,0,255],[0255,0,0],[0,255,0],[0,255,255],[255,0,255] $
        ,[128,255,0],[255,128,0],[0,255,128],[128,128,255],[255,128,128] $
        ,[128,255,128],[128,128,128],[255,255,0]]
Self->SetProperty, b_color=colors[*,pos], b_sym_color=colors[*,pos], b_use_default_color=1 $
                 , b_errorbar_color=colors[*,pos], b_sym_fill_color=colors[*,pos]
Self->CustomizeVis, oVis[nVis-1], plotName, 'BKGDPLOT'
;oVis[nVis-1]->SetProperty, /fill $
;                 ,aspect_ratio=aspect_ratio $
;                 ,transparency = 5 $
;                 ,n_levels=15 $
;                 ,pal_color=1  ; => use palette for level colors
;status = oVis[nVis-1]->SetData(oDataRGB, parameter_name='PALETTE',/by_value)


;if ((nVis gt 1) && overplotFlag && lastplotFlag) then begin
;  ; if a multiplot and this is the last entry group all the plots together
;  ; to form a single entity in the property sheet view
;  ; And change the new group's name property to 'Contour'
;  for i=0,nVis-1 do oVis[i]->Select, additive=(i gt 0)
;  grpID = 'Operations/Edit/Grouping/Group'
;  oGrpOI = (Self->GetByIdentifier(grpID))->GetObjectInstance()
;  oGrpCmdSet = oGrpOI->DoAction(Self)
;  oGrpCmd = (oGrpCmdSet->Get(/all,count=nGrpCmd))[0]
;  if (isa(oGrpCmd,'IDLitCommand')) then begin
;    oGrpCmd->GetProperty, target_identifier=targID
;    (Self->GetByIdentifier(targID))->SetProperty, name='Contour'
;  endif
;endif
;oDataSpace->SetProperty, aspect_ratio=aspect_ratio
if (noOverplot) then oDataSpace->Translate, 0.20, 0.0, 0.0 ; shift dataspace a bit to left for first plot only

;; customize axes
;oAxes = oDataSpace->GetAxes(/container)
;oAxes->SetProperty,axis_style=2  $    ; draw box axes
;                  ;,yTitle=ylabel, xTitle=xlabel $
;                  ,zTitle=zlabel
                  

; Update Plot Title text to reflect name of dataset being plotted
; The plot title annotation was automatically created when visualization was first created so
; all we need to do is change the text string
annID = oView->FindIdentifiers('*ANNOTATION LAYER*',count=nAnn)
if (nAnn gt 0) then begin
  oText = Self->GetByIdentifier(annID[1])
  oItem->Getproperty, name=visname
  if (isa(oText,'IDLitvisText')) then oText->SetProperty, string=visname
endif

if (activateManip) then Self->ActivateManipulator, (*sPtr).manipID
Self->RefreshCurrentWindow   ;oWin->Draw   ; force window to refresh




end
;-------------------------------------------------------------------------------


;===============================================================================
;+
; Bt7psdTool::PlotPowder
;
; PURPOSE:
;   Generates a line or contour plot of the specified dataset for the case of a powder sample
;
; PARAMETERS:
;   oItem   - dataset to be plotted
;   sPtr    - heap variable containing useful widget related info
;   overPlotFlag  - is overplotting on (1) or off (0)
;   lastPltFlag   - if overplotting, is this the last plot to be included
;
; KEYWORDS:
;-
pro Bt7psdTool::PlotPowder, oItem, sPtr, overPlotFlag, lastPlotFlag, position=pos
compile_opt idl2

if (~obj_isa(oItem, 'IDLitParameterSet')) then return
if (oItem->GetMetaDataCount() eq 0) then return

; Ensure dataset was processed as 'Powder' sample
status = oItem->GetMetaData('sampTypeFlag',sTypeFlag) 
if (sTypeFlag ne 0) then begin
  stypes = ['Powder','Single Crystal','Raw Data']
  msg = 'Cannot plot dataset; It is of Sample Type: "'+stypes[sTypeFlag]+'" which does not match current setting!'
  Self->StatusMessage, msg
  return
endif

;; Plot the data
Self->GetProperty, xAxisVar=xAxisVar, indepVar2List=varList, calibParamsisSet=calibParamsisSet
xvar = varList[xAxisVar]

axisHasChanged = 0
msg = []

; Default to Data_Index for x-axis if one of the calculated 
; quantities ['2Theta','d-space','|Q|','E'] was selected by the user
; and the calibration parameters are not yet loaded
calculatedVarList = ['2Theta','d-space','|Q|','E']
void = where(calculatedVarList eq xvar, xvarIsCalculated)
if (calibParamsisSet) then begin
  ; if calibration is loaded, use the user selected axes
  oX = oItem->GetByName(xvar,count=count)
  if (count ne 1) then return
endif else begin
   msg = 'PSD Calibration parameters file is not yet loaded!'
   if (xvarIsCalculated) then begin
      oX = oItem->GetByName('Data_Index',count=count)
      if (count ne 1) then return
      xAxisVar = where(varList eq 'Data_Index')
      Self->SetProperty, xAxisVar=xAxisVar
      msg = [msg,xvar+' cannot be evaluated hence switching Scan Variable axis to Data_Index']
      axisHasChanged = 1
      xvar = 'Data_Index'
   endif else $
      oX = oItem->GetByName(xvar,count=count)
endelse

;; If the x data values are constant default to the Data_Index as the axis value
status = oX->GetData(vdata)
vmax = max(vdata,min=vmin)
if (vmax eq vmin) then begin
  oX = oItem->GetByName('Data_Index',count=count)
  if (count ne 1) then return
  xAxisVar = where(varList eq 'Data_Index')
  Self->SetProperty, xAxisVar=xAxisVar
  axisHasChanged = 1
  msg = [msg,xvar+' does not vary hence switching Scan Variable axis to Data_Index']
  xvar = 'Data_Index'
endif          

if (axisHasChanged) then Self->ErrorMessage, msg,severity=0,title='BT7 PSD Reduction - Axes selection' 

; retrieve counts and error which are stored as metadata with the independent variable.
; Store these with the dependent and error variables
status = oX->GetMetaData('Counts',counts)
ctype = partype(counts)

if (ctype ne 'IDLVECTOR') then return  ; can't crate a plot from non-vector data

oZ = oItem->GetByName('Counts')
status = oZ->SetData(counts,/no_copy,/no_notify)
status = oX->GetMetaData('Error',error)
oErr = oItem->GetByName('Error')
status = oErr->SetData(error,/no_copy,/no_notify)


oZ->SetProperty, type=ctype
oX->SetProperty, type=ctype
oErr->SetProperty, type=ctype
 
;; create color Palette using IDL color table colors
;oTempPalette = IDLgrPalette()
;oTempPalette->LoadCT, 13
;oTempPalette->GetProperty, red_values=red, green_values=green, blue_values=blue
;obj_destroy, oTempPalette
;oDataRGB = IDLitDataIDLPalette(transpose([[red],[green],[blue]]),name='RGB Table')

strparms = ['Y','X','Y ERROR','X ERROR','VERTICES','PALETTE','VERTEX_COLORS']
idparms = [oZ->GetFullIdentifier() $
          ,oX->GetFullIdentifier() $
          ,oErr->GetFullIdentifier() $
          ,'','','','']
visID = 'PLOT'
opID = 'Operations/Insert Visualization'
oRequester = (Self->GetByIdentifier(opID))->GetObjectInstance()
oRequester->_setTool, Self
oRequester->GetProperty, show_execution_ui = orig_showUI
oRequester->SetProperty, show_execution_ui = 0

oItem->GetProperty, name=plotName   ; retrieve the dataset name as shown in the data manager and set it as plotname
oRequester->SetProperty, name=plotName, parameter_names=strParms, data_ids=idParms, visualization_id=visID

oWin = Self->GetCurrentWindow()
oView = oWin->GetCurrentView()
activateManip = 0
rangeChange = 0

noOverplot = overplotFlag eq 0
if (Self->HasVisualizations() && noOverplot) then begin
   ;; store the active manipulator
   (*sPtr).manipID = (Self->GetCurrentManipulator())->GetFullIdentifier()
   activateManip = 1

   ;; ensure the existing visualization is selected
   oLayer = oView->GetCurrentLayer()
   oWorld = oLayer->GetWorld()
   oDataSpace = oWorld->GetDataSpaces()
   oVis = oDataSpace->GetVisualizations(count = nVis)
    
   ; retrieve current xrange and yrange to determine if user has zoomed plot
   oDataSpace->GetProperty, xrange=xr, yrange=yr
   xChange = ((*sPtr).xrange[0] ne xr[0]) || ((*sPtr).xrange[1] ne xr[1]) 
   yChange = ((*sPtr).yrange[0] ne yr[0]) || ((*sPtr).yrange[1] ne yr[1]) 
   rangeChange = xChange && yChange
   
   delID = 'Operations/Edit/Delete'
   oDelete = (Self->GetByIdentifier(delID))->GetObjectInstance()
   
   oDataSpace->Select
   void = oDelete->DoAction(Self)

   obj_destroy, [oVis,oDataSpace]

   ; delete any residual annotations that may have been added
   annID = oView->FindIdentifiers('*ANNOTATION LAYER*',count=nAnn)
   if (nAnn gt 2) then begin
     oVoid = []
     for i=2, nAnn-1 do begin
       oTmp = Self->GetByIdentifier(annID[i])
       oTmp->Select
       oVoid = [oVoid,oTmp]
     endfor
     void = oDelete->DoAction(Self) ; delete selected anootations
     obj_destroy, oVoid
   endif
endif

; create the new vis
oCmd = oRequester->DoAction(Self)

;; Get the newly created vis
oLayer = oView->GetCurrentLayer()
oWorld = oLayer->GetWorld()
oDataSpace = oWorld->GetDataSpaces()
oVis = oDataSpace->GetVisualizations(count = nVis)

; TODO - fix vis cuztomization
; Customize it
colors = [[0,0,255],[0255,0,0],[0,255,0],[0,255,255],[255,0,255] $
  ,[128,255,0],[255,128,0],[0,255,128],[128,128,255],[255,128,128] $
  ,[128,255,128],[128,128,128],[255,255,0]]
nColors = N_elements(colors)/3
pos = pos mod nColors
Self->SetProperty, s_color=colors[*,pos], s_sym_color=colors[*,pos], s_use_default_color=1 $
  , s_errorbar_color=colors[*,pos], s_sym_fill_color=colors[*,pos]
Self->CustomizeVis, oVis[nVis-1], plotName, 'SAMPPLOT'
;oVis[nVis-1]->SetProperty, /fill $
;                 ,aspect_ratio=aspect_ratio $
;                 ,transparency = 5 $
;                 ,n_levels=15 $
;                 ,pal_color=1  ; => use palette for level colors
;status = oVis[nVis-1]->SetData(oDataRGB, parameter_name='PALETTE',/by_value)


;if ((nVis gt 1) && overplotFlag && lastplotFlag) then begin
;  ; if a multiplot and this is the last entry group all the plots together
;  ; to form a single entity in the property sheet view
;  ; And change the new group's name property to 'Contour'
;  for i=0,nVis-1 do oVis[i]->Select, additive=(i gt 0)
;  grpID = 'Operations/Edit/Grouping/Group'
;  oGrpOI = (Self->GetByIdentifier(grpID))->GetObjectInstance()
;  oGrpCmdSet = oGrpOI->DoAction(Self)
;  oGrpCmd = (oGrpCmdSet->Get(/all,count=nGrpCmd))[0]
;  if (isa(oGrpCmd,'IDLitCommand')) then begin
;    oGrpCmd->GetProperty, target_identifier=targID
;    (Self->GetByIdentifier(targID))->SetProperty, name='Contour'
;  endif
;endif
;oDataSpace->SetProperty, aspect_ratio=aspect_ratio
;if (noOverplot) then oDataSpace->Translate, 0.20, 0.0, 0.0 ; shift dataspace a bit to left for first plot only

;; customize axes
;oAxes = oDataSpace->GetAxes(/container)
;oAxes->SetProperty,axis_style=2  $    ; draw box axes
;                  ;,yTitle=ylabel, xTitle=xlabel $
;                  ,zTitle=zlabel
                  

; Update Plot Title text to reflect name of dataset being plotted
; The plot title annotation was automatically created when visualization was first created so
; all we need to do is change the text string
annID = oView->FindIdentifiers('*ANNOTATION LAYER*',count=nAnn)
if (nAnn gt 0) then begin
  oText = Self->GetByIdentifier(annID[1])
  oItem->Getproperty, name=visname
  if (isa(oText,'IDLitvisText')) then oText->SetProperty, string=visname
endif

if (activateManip) then Self->ActivateManipulator, (*sPtr).manipID
Self->RefreshCurrentWindow   ;oWin->Draw   ; force window to refresh




end
;-------------------------------------------------------------------------------


;===============================================================================
;+
; Bt7psdTool::PlotSinglecrystal
;
; PURPOSE:
;   Generates a contour plot of the specified dataset for the case of a sngle crystal sample
;
; PARAMETERS:
;   oItem   - dataset to be plotted
;   sPtr    - heap variable containing useful widget related info
;   overPlotFlag  - is overplotting on (1) or off (0)
;   lastPltFlag   - if overplotting, is this the last plot to be included
;
; KEYWORDS:
;-
pro Bt7psdTool::PlotSinglecrystal, oItem, sPtr, overPlotFlag, lastPlotFlag
compile_opt idl2

if (~obj_isa(oItem, 'IDLitParameterSet')) then return
if (oItem->GetMetaDataCount() eq 0) then return

; Ensure dataset was processed as 'Single Crystal' sample
status = oItem->GetMetaData('sampTypeFlag',sTypeFlag) 
if (sTypeFlag ne 1) then begin
  stypes = ['Powder','Single Crystal','Raw Data']
  msg = 'Cannot plot dataset; It is of Sample Type: "'+stypes[sTypeFlag]+'" which does not match current setting!'
  Self->StatusMessage, msg
  return
endif

;; Plot the data
Self->GetProperty, xAxisVar=xAxisVar,yAxisVar=yAxisVar, indepVar1List=varList $
                  ,calibParamsisSet=calibParamsisSet, qUnits=qUnitsFlag
xvar = varList[xAxisVar]
yvar = varList[yAxisVar]

axisHasChanged = 0
msg = []
if (calibParamsisSet) then begin
  ; if calibration is loaded, use the user selected axes
  oX = oItem->GetByName(xvar,count=count)
  if (count ne 1) then return
  oY = oItem->GetByName(yvar,count=count)
  if (count ne 1) then return
endif else begin
  ; else default to PSDChannel for x and 'TEMP' for y axis
  ; if one of the calculated quantities ['QX','QY','|Q|','E'] was selected by the user
  calculatedVarList = ['QX','QY','|Q|','E']
  void = where(calculatedVarList eq xvar, xvarIsCalculated)
  void = where(calculatedVarList eq yvar, yvarIsCalculated)
  msg = 'PSD Calibration parameters file is not yet loaded!'
  
  if (xvarIsCalculated) then begin
     oX = oItem->GetByName('PSDChannel',count=count)
     if (count ne 1) then return
     xAxisVar = where(varList eq 'PSDChannel')
     Self->SetProperty, xAxisVar=xAxisVar
     msg = [msg,xvar+' cannot be evaluated hence switching horizontal axis to PSDChannel']
     axisHasChanged = 1
     xvar = 'PSDChannel'
  endif else $
     oX = oItem->GetByName(xvar,count=count)
  
  if (yvarIsCalculated) then begin
     oY = oItem->GetByName('TEMP',count=count)
     if (count ne 1) then return
     yAxisVar = where(varList eq 'TEMP')
     Self->SetProperty, yaxisVar=yAxisVar
     msg = [msg,yvar+' cannot be evaluated hence switching vertical axis to Temperature']
     axisHasChanged = 1
     yvar = 'TEMP'
  endif else $
     oY = oItem->GetByName(yvar,count=count)
endelse

;; If the x or ydata values are constant default to the PSDChannel as the axis value
status = oX->GetData(vdata)
vmax = max(vdata,min=vmin)
if (vmax eq vmin) then begin
  oX = oItem->GetByName('PSDChannel',count=count)
  if (count ne 1) then return
  xAxisVar = where(varList eq 'PSDChannel')
  Self->SetProperty, xAxisVar=xAxisVar
  axisHasChanged = 1
  msg = [msg,xvar+' does not vary hence switching Horizontal axis to PSDChannel']
  xvar = 'PSDChannel'
endif          
status = oY->GetData(vdata)
vmax = max(vdata,min=vmin)
if (vmax eq vmin) then begin
  oY = oItem->GetByName('PSDChannel',count=count)
  if (count ne 1) then return
  yAxisVar = where(varList eq 'PSDChannel')
  Self->SetProperty, yAxisVar=yAxisVar
  axisHasChanged = 1
  msg = [msg,yvar+' does not vary hence switching Vertical axis to PSDChannel']
  yvar = 'PSDChannel'
endif

if (axisHasChanged) then Self->ErrorMessage, msg,severity=0,title='BT7 PSD Reduction - Axes selection' 

; determine aspect ratio of plot
; It should be set to 1 (isotropic) if both x and y axis variables are
; wavevector ('QX' or 'QY')
xIsQ = (xvar eq varList[0]) || (xvar eq varList[1])
yIsQ = (yvar eq varList[0]) || (yvar eq varList[1])
aspect_ratio = (xIsQ && yIsQ)? 1 : 0
if (xIsQ && yIsQ) then begin
  ; If both x and y are in Q then if the units is in 1/A then the aspect_ratio of the
  ; plot is 1. However when it is r.l.u. then the x and y scales will be different.
  ; In this case, reset the aspect ration to be the same as the ratio of the y and x
  ; relative lattice units so that the aspect ratio will still appear to be 1.
  status = oY->GetMetaData('AA2RLU',yAA2RLU)
  status = oX->GetMetaData('AA2RLU',xAA2RLU)
  aspect_ratio = (qUnitsFlag eq 0)? 1.0 : yAA2RLU/xAA2RLU
  help,'aspect_ratio = ',aspect_ratio
endif

oZ = oItem->GetByName('Counts',count=count)
if (count ne 1) then return
status = oZ->GetData(counts)
CTYPE = partype(counts)
if (ctype ne 'IDLARRAY2D') then return  ; can't crate a plot from non-array (2D) data
;xLabel = ''
;if (oX->GetMetaData('LONG_NAME',label)) then xLabel += label
;if (oX->GetMetaData('UNITS',units)) then $
;  xLabel = (strlen(units) gt 0)? xLabel+' ('+units+')' : xLabel
;yLabel = ''
;if (oY->GetMetaData('LONG_NAME',label)) then yLabel += label
;if (oY->GetMetaData('UNITS',units)) then $
;  yLabel = (strlen(units) gt 0)? yLabel+' ('+units+')' : yLabel
zLabel = ''
if (oZ->GetMetaData('LONG_NAME',label)) then zLabel += label
if (oZ->GetMetaData('UNITS',units)) then $
  zLabel = (strlen(units) gt 0)? zLabel+' ('+units+')' : zLabel
 
; create color Palette using IDL color table colors
oTempPalette = IDLgrPalette()
oTempPalette->LoadCT, 13
oTempPalette->GetProperty, red_values=red, green_values=green, blue_values=blue
obj_destroy, oTempPalette
oDataRGB = IDLitDataIDLPalette(transpose([[red],[green],[blue]]),name='RGB Table')

strparms = ['Z','X','Y','PALETTE','RGB_INDICES']
idparms = [oZ->GetFullIdentifier() $
          ,oX->GetFullIdentifier() $
          ,oY->GetFullIdentifier() $
          ,'' $ ;oDataRGB->GetFullIdentifier() $
          ,'']
visID = 'CONTOUR'
opID = 'Operations/Insert Visualization'
oRequester = (Self->GetByIdentifier(opID))->GetObjectInstance()
oRequester->_setTool, Self
oRequester->GetProperty, show_execution_ui = orig_showUI
oRequester->SetProperty, show_execution_ui = 0

oRequester->SetProperty, parameter_names=strParms, data_ids=idParms, visualization_id=visID
 
;Self->DisableUpdates
oWin = Self->GetCurrentWindow()
oView = oWin->GetCurrentView()
activateManip = 0
rangeChange = 0

noOverplot = overplotFlag eq 0
if (Self->HasVisualizations() && noOverplot) then begin
   print, 'Vis deleted...'
   ;; store the active manipulator
   (*sPtr).manipID = (Self->GetCurrentManipulator())->GetFullIdentifier()
   activateManip = 1

   ;; ensure the existing visualization is selected
   oLayer = oView->GetCurrentLayer()
   oWorld = oLayer->GetWorld()
   oDataSpace = oWorld->GetDataSpaces()
   oVis = oDataSpace->GetVisualizations(count = nVis)
    
   ; retrieve current xrange and yrange to determine if user has zoomed plot
   oDataSpace->GetProperty, xrange=xr, yrange=yr
   xChange = ((*sPtr).xrange[0] ne xr[0]) || ((*sPtr).xrange[1] ne xr[1]) 
   yChange = ((*sPtr).yrange[0] ne yr[0]) || ((*sPtr).yrange[1] ne yr[1]) 
   rangeChange = xChange && yChange
   
   delID = 'Operations/Edit/Delete'
   oDelete = (Self->GetByIdentifier(delID))->GetObjectInstance()
   
    ;for i=0,nVis-1 do begin
    ;  oVis[i]->Select ; make it current selection in Tool window
    ; TODO - save vis props
    ;  ;Self->saveVisProps, oVis  ; save vis properties before deleting it
    ;  void = oDelete->DoAction(Self)   ; delete selected item ie the vis
    ;endfor
   oDataSpace->Select
   void = oDelete->DoAction(Self)

   obj_destroy, [oVis,oDataSpace]

   ; delete any residual annotations that may have been added
   annID = oView->FindIdentifiers('*ANNOTATION LAYER*',count=nAnn)
   if (nAnn gt 2) then begin
     oVoid = []
     for i=2, nAnn-1 do begin
       oTmp = Self->GetByIdentifier(annID[i])
       oTmp->Select
       oVoid = [oVoid,oTmp]
     endfor
     void = oDelete->DoAction(Self) ; delete selected anootations
     obj_destroy, oVoid
   endif
endif

; create the new vis
oCmd = oRequester->DoAction(Self)

;; Get the newly created vis
oLayer = oView->GetCurrentLayer()
oWorld = oLayer->GetWorld()
oDataSpace = oWorld->GetDataSpaces()
oVis = oDataSpace->GetVisualizations(count = nVis)

; TODO - fix vis cuztomization
; Customize it
;Self->CustomizeVis, oVis, plotname
oItem->Getproperty, name=visname
oVis[nVis-1]->SetProperty, /fill $
                 ,aspect_ratio=aspect_ratio $
                 ,transparency = 5 $
                 ,n_levels=15 $
                 ,name = visname $
                 ,description = visname $
                 ,pal_color=1  ; => use palette for level colors

status = oVis[nVis-1]->SetData(oDataRGB, parameter_name='PALETTE',/by_value)

Self->UpdatePlotIntensityRange, oVis[nVis-1]

if ((nVis gt 1) && overplotFlag && lastplotFlag) then begin
  ; if a multiplot and this is the last entry group all the plots together
  ; to form a single entity in the property sheet view
  ; And change the new group's name property to 'Contour'
;  for i=0,nVis-1 do oVis[i]->Select, additive=(i gt 0)
;  grpID = 'Operations/Edit/Grouping/Group'
;  oGrpOI = (Self->GetByIdentifier(grpID))->GetObjectInstance()
;  oGrpCmdSet = oGrpOI->DoAction(Self)
;  oGrpCmd = (oGrpCmdSet->Get(/all,count=nGrpCmd))[0]
;  if (isa(oGrpCmd,'IDLitCommand')) then begin
;    oGrpCmd->GetProperty, target_identifier=targID
;    (Self->GetByIdentifier(targID))->SetProperty, name='Contour';, manipulator_target=1
;  endif
   oVis[0]->Group,oVis[1:nVis-1]
   oDataSpace->GetProperty, transform=tf
   help,tf
endif
oDataSpace->SetProperty, aspect_ratio=aspect_ratio
if (noOverplot) then oDataSpace->Translate, 0.20, 0.0, 0.0 ; shift dataspace a bit to left for first plot only

; customize axes
oAxes = oDataSpace->GetAxes(/container)
oAxes->SetProperty,axis_style=2  $    ; draw box axes
                  ;,yTitle=ylabel, xTitle=xlabel $
                  ,zTitle=zlabel
                  
; Update Plot Title text to reflect name of dataset being plotted
; The plot title annotation was automatically created when visualization was first created so
; all we need to do is change the text string
annID = oView->FindIdentifiers('*ANNOTATION LAYER*',count=nAnn)
if (nAnn gt 0) then begin
  oText = Self->GetByIdentifier(annID[1])
  if (isa(oText,'IDLitvisText')) then oText->SetProperty, string=visname
endif

if (activateManip) then Self->ActivateManipulator, (*sPtr).manipID
Self->RefreshCurrentWindow   ;oWin->Draw   ; force window to refresh

end
;-------------------------------------------------------------------------------


;===============================================================================
; Bt7psdTool::saveAsASCII
;
; PURPOSE:
;   Save output to file using DAVE format
;
; PARAMETERS:
;
; KEYWORDS:
;
function Bt7psdTool::SaveAsASCII
compile_opt idl2

oUI = Self->GetUI()
oUI->GetProperty, group_leader = wTLB


if (~Self->ExportSelection(asciiPtr, nameTag=nametag, /useTexlabels,/ascii)) then return, 0
if (ptr_valid(asciiPtr)) then begin

   Self->GetProperty, working_directory=workDir
   title = "Specify filename to store ASCII format data"
   ext = 'txt'
   filename = dialog_pickfile(title=title,default_extension=ext, dialog_parent=wTLB $
                          ,/overwrite_prompt, file=nameTag+'.txt',path=workDir, get_path=new_workDir)
                          
   if (~strcmp(filename,'')) then begin
      xvals = (*asciiPtr).xvals
      yvals = (*asciiPtr).yvals
      evals = (*asciiPtr).evals
      nd = n_elements(xvals)
      openw, lun, filename, /get_lun
      for i = 0,nd-1 do printf, lun, format='(3(F11.4,2X))',xvals[i],yvals[i],evals[i]
      
      free_lun, lun, /force
      
      if (file_test(new_workDir,/directory)) then Self->SetProperty, working_directory=new_workDir
   
      msg = 'Output stored in: '+filename
      Self->StatusMessage, msg
   endif
   
   ptr_free, asciiPtr
endif

return, 1
end


;===============================================================================
; Bt7psdTool::saveAsDave
;
; PURPOSE:
;   Save output to file using DAVE format
;
; PARAMETERS:
;
; KEYWORDS:
;
function Bt7psdTool::SaveAsDave
compile_opt idl2

oUI = Self->GetUI()
oUI->GetProperty, group_leader = wTLB


if (~Self->ExportSelection(davePtr, nameTag=nametag, /useTexlabels,/dave)) then return, 0
if (ptr_valid(davePtr)) then begin

   Self->GetProperty, working_directory=workDir
   title = "Specify filename to store DAVE format data"
   ext = 'dave'
   filename = dialog_pickfile(title=title,default_extension=ext, dialog_parent=wTLB $
                          ,/overwrite_prompt, file=nameTag+'.dave',path=workDir, get_path=new_workDir)
                          
   if (~strcmp(filename,'')) then begin
      save, davePtr, filename=filename
      if (file_test(new_workDir,/directory)) then Self->SetProperty, working_directory=new_workDir
   
      msg = 'Output stored in: '+filename
      Self->StatusMessage, msg
   endif
   
   heap_free, davePtr
endif

return, 1
end


;===============================================================================
; Bt7psdTool::exportToDaveDM
;
; PURPOSE:
;   export dataset to DAVE Data Manager
;
; PARAMETERS:
;
; KEYWORDS:
;
function Bt7psdTool::exportToDaveDM
compile_opt idl2

; Basic error Handler
if (n_elements(!debug) && (!debug eq 0)) then begin
    catch, catchError
    if (catchError ne 0) then begin
        Self->StatusMessage, !ERROR_STATE.msg
        catch, /cancel
        return, 0
    endif
endif

;oUI = Self->GetUI()
;oUI->GetProperty, group_leader = wTLB
;
;; display dialog to let user select variables to be exported to Mslice 
;Self->GetProperty, selectedIDs=selIDs,nSelectedIDs=nSel, xAxisVar=xAxisVar, indepVar2List=varList
;if (nSel eq 0) then begin
;   Self->StatusMessage, 'No dataset selected!'
;   return, 0  ; exit if no dataset is currently selected
;endif
;xvar = varList[xAxisVar]
;
;for i=0,nSel - 1 do begin
;   oData = Self->GetByIdentifier(selIDs[i])
;   status = oData->GetMetaData('sampTypeFlag',sTypeFlag) ; only export powder data to Mslice
;   if (sTypeFlag ne 0) then continue
;      
;   if (~Self->retrieveDavePtr(oData, xVar, davePtr, /useTexlabels)) then continue
;
;   oData->GetProperty, name=nameTag
;   if ((n_elements(nameTag) le 0) || (strtrim(nameTag,2) eq '')) then nameTag='Untitled'
;   
;   ; Send data to DAVE Data Manager Folder
;   Self.DAVETool->AddDavePtrToDataManager, davePtr, nameTag
;   
;   msg = "New extry '"+nameTag+"' created in DAVE Data Manager"
;   Self->StatusMessage, msg
;   
;   if (ptr_valid(dataPtr)) then  heap_free, davePtr
;endfor

if (~Self->ExportSelection(davePtr, nameTag=nametag, /useTexlabels, /dave)) then return, 0

if (ptr_valid(davePtr)) then begin
   if ((n_elements(nameTag) le 0) || (strtrim(nameTag,2) eq '')) then nameTag='Untitled'
      
   ; Send data to DAVE Data Manager Folder
   Self.DAVETool->AddDavePtrToDataManager, davePtr, nameTag
   
   msg = "New extry '"+nameTag+"' created in DAVE Data Manager"
   Self->StatusMessage, msg

   heap_free, davePtr
endif

return, 1
end




;===============================================================================
; Bt7psdTool::exportToPAN
;
; PURPOSE:
;   export dataset to DAVE Data Manager
;
; PARAMETERS:
;
; KEYWORDS:
;
function Bt7psdTool::ExportSelection, davePtr, nameTag=nameTag, useTexLabels=useTexLabels $
                                    ,dave=dave, ascii=ascii
compile_opt idl2

; Basic error Handler
if (n_elements(!debug) && (!debug eq 0)) then begin
    catch, catchError
    if (catchError ne 0) then begin
        Self->StatusMessage, !ERROR_STATE.msg
        catch, /cancel
        return, 0
    endif
endif

; Check the sample type flag setting
Self->GetProperty, sampTypeFlag=sampTypeFlag
if (sampTypeFlag eq 1) then begin
   Self->StatusMessage, 'Cannot export single crystal data to ASCII or DAVE data Format!'
   return, 0 ; can only export powder sample (type=0) or raw (type=2) data
endif

; determine the independent variable required
if (sampTypeFlag eq 0) then begin
   ; expect powder sample data
   Self->GetProperty, selectedIDs=selIDs,nSelectedIDs=nSel, xAxisVar=xAxisVar, indepVar2List=varList
   xvar = varList[xAxisVar]
endif else begin
   ; raw data
   Self->GetProperty, xAxisVar=mode, yAxisVar=xAxisVar, indepVar3List=varList, nSelectedIDs=nSel, selectedIDs=selIDs
   ;if ((mode eq 0) || (mode eq 1)) then xvar = 'PSDChannel' ; Cts vs Channel || Integrated Cts (data points) vs Channel
   if (mode eq 0) then xvar = 'PSDChannelA4' ; Cts vs Channel
   if (mode eq 1) then xvar = 'PSDChannel' ; Integrated Cts (data points) vs Channel
   if (mode eq 2) then xvar = varList[xAxisVar] ; Integrated Cts (detector channels) vs ScanVariable
endelse

; Are there any datasets selected?
if (nSel eq 0) then begin
   Self->StatusMessage, 'No dataset selected!'
   return, 0  ; exit if no dataset is currently selected
endif

; Retrieve the selected datasets
oData = []
for i=0,nSel - 1 do begin
   oTmp = Self->GetByIdentifier(selIDs[i])
   status = oTmp->GetMetaData('sampTypeFlag',sTypeFlag)
   if (sTypeFlag ne sampTypeFlag) then continue ; dataset's sample type flag must match current that of the app preference
   oData = [oData,oTmp]
endfor
nSel = n_elements(oData)
if (nSel eq 0) then begin
   Self->StatusMessage, 'No valid dataset selected!'
   return, 0  ; exit if no dataset is currently selected
endif

; Convert selected datasets to ASCII or davePtr structure as necessary 
if (~obj_valid(oData[0])) then return, 0
if (~Self->retrieveDavePtr(oData, xVar, davePtr, sampTypeFlag=sampTypeFlag, mode=mode, useTexLabels=useTexLabels, ascii=ascii, dave=dave)) then return, 0

oData[0]->GetProperty, name = nameTag

return, 1

end


;===============================================================================
; Bt7psdTool::exportToPAN
;
; PURPOSE:
;   export dataset to DAVE Data Manager
;
; PARAMETERS:
;
; KEYWORDS:
;
function Bt7psdTool::ExportToPAN
compile_opt idl2

; Basic error Handler
if (n_elements(!debug) && (!debug eq 0)) then begin
    catch, catchError
    if (catchError ne 0) then begin
        Self->StatusMessage, !ERROR_STATE.msg
        catch, /cancel
        return, 0
    endif
endif

oUI = Self->GetUI()
oUI->GetProperty, group_leader = wTLB
notifyIDs = [wTLB,wTLB]
Self->GetProperty, working_directory=workDir, data_directory=dataDir

;status = Self->ExportSelectionToDavePtr(davePtr)
if (~Self->ExportSelection(davePtr, nameTag=nametag, /useTexlabels, /dave)) then return, 0

; Determine in top-level base widget ID for the main DAVE window
; and use this as the group_leader when PAN is called. This way the PAN instance will
; be killed automatically only when DAVE goes down rather than if the BT7 reduction module is killed.
oUIMain = Self.daveTool->GetUI()
oUIMain->GetProperty, group_leader = wTLBMain

if (ptr_valid(davePtr)) then  begin
   oVoid = obj_new('OPAN',notifyID=[wTLBMain,wTLBMain],group_leader=wTLBMain $
                ,davePtr=davePtr,workDir=workDir,daveTool=Self.DAVETool,dataDir=dataDir)

   heap_free, davePtr
endif

return, 1
end


;===============================================================================
; Bt7psdTool::saveProperties
;
; PURPOSE:
;   export dataset to DAVE Data Manager
;
; PARAMETERS:
;
; KEYWORDS:
;
function Bt7psdTool::SaveProperties
compile_opt idl2

oUI = Self->GetUI()
oUI->GetProperty, group_leader = wTLB
wChild = widget_info(wTLB, /child)
widget_control, wChild, get_uvalue=sPtr

; Basic error Handler
if (n_elements(!debug) && (!debug eq 0)) then begin
    catch, catchError
    if (catchError ne 0) then begin
        ;;print, 'Error handled!'
        eTitle = 'Bt7psdTool::saveProperties: Error encountered'
        eMsg = 'An error or unusual condition was encountered!'
        eMsg = [eMsg,'Please, report the following to the DAVE team:']
        eMsg = [eMsg,!error_state.msg]
        void = dialog_message(/error,eMsg,title=eTitle,dialog_parent=wTLB)
        catch, /cancel
        return, 0
    endif
endif

print, 'properties saved...'

return, 1
end


;===============================================================================
; Bt7psdTool::RetrieveDavePtr
;
; PURPOSE:
;   create a davePtr structure from the data in the specified dataset object
;
; PARAMETERS:
;
; KEYWORDS:
;
function Bt7psdTool::RetrieveDavePtr, oData, xVar, davePtr, sampTypeFlag=sampTypeFlag, mode=mode, useTexLabels=useTexLabels $
                                     ,dave=dave, ascii=ascii
compile_opt idl2

; Basic error Handler
if (n_elements(!debug) && (!debug eq 0)) then begin
    catch, catchError
    if (catchError ne 0) then begin
        Self->StatusMessage, !ERROR_STATE.msg
        catch, /cancel
        if (ptr_valid(davePtr)) then heap_free, davePtr
        return, 0
    endif
endif

if (~obj_valid(oData[0])) then return, 0
if (xVar eq '') then return, 0

davePtr = ptr_new()

if (keyword_set(dave) && (sampTypeFlag eq 2) && (mode eq 0)) then begin
  ;; if rawmode and Cts vs Channel handle slightly differenly
  ;; - export 2D data
  ;; - dim 1 is the PSD channels in 2theta or index nos
  ;; - dim 2 is the scan variable or data point nos 

  Self->GetProperty, calibParamsisSet=calibParamsisSet, maskFlag=maskFlag $
                 , MONSCALEVALUE=monScaleVal, NORMALIZETO=normTo, effNormFlag=effFlag,dpntIndex=dpntIndex

  moreThanOne = 0
  for i = 0, n_elements(oData)-1 do begin
    if (~obj_valid(oData[0])) then continue
    
    status = oData[i]->GetMetaData('CHDELA4',chDelA4)
    
    ; Dim 1
    if (calibParamsisSet) then begin
      ; Determine the A4 accross PSD from norminal A4 and delta A4 (= A4o- DelA4)
      ; and set this to the PSDChannelA4 object before plotting
      oA4 = oData[i]->GetByName('A4')
      status = oA4->GetData(A4o)
      index = (dpntIndex > 0) < (n_elements(A4o) - 1) ; 0 < xAxisVar < npts-1    
      xvals = A4o[index] - chDelA4
      oX = oData[i]->GetByName('PSDChannelA4')
    endif else begin
      ; just use the PSD Channel index
      oX = oData[i]->GetByName('PSDChannel')
      status = oX->GetData(xvals)
    endelse
  
    ; Dim 2
    ; If the scan variable was identified from the raw data, then use it as 2nd dim
    ; else use the data point indices
    if (moreThanOne eq 0) then begin
      status = oData[i]->GetMetaData('INDEP_VAR_TAG',indep_var_tag)
      oY = oData[i]->GetByName(indep_var_tag)
      if (~obj_valid(oY)) then begin 
         indep_var_tag = 'Data_Index'
         oY = oData[i]->GetByName(indep_var_tag)
      endif
      if (~obj_valid(oY)) then continue 
      status = oY->GetData(yvals)
    endif else begin
      ; retrieve the same scan var from subsequent datasets
      oY = oData[i]->GetByName(indep_var_tag)
      if (~obj_valid(oY)) then continue 
      status = oY->GetData(yvals)
    endelse
  
    ; Intensity
    status = oData[i]->GetMetaData('OriginalCounts',counts)
    error = counts
    index = where(error le 0.0,cnt)
    if (cnt gt 0) then error[index] = 1.0
    error = sqrt(error)
  
  
    ; efficiency correction
    if (effFlag) then begin
       status = oData[i]->GetMetaData('CHEFF',chEff)
       counts = temporary(counts)/chEff
       error = temporary(error)/sqrt(chEff)
    endif
    
    ;; Mask bad channels
    sBadChans = Self.maskedDets
    maskedChansExist = ~strcmp(sBadChans,'')
    if (maskFlag && maskedChansExist) then begin
       status = compactstring2intvector(sBadChans, badChans)
       badChanIndex = badChans - 1   ; convert channel nos to 0-based channel index
       ; Apply mask by simple replacing the counts in the masked channels by NaNs
       ; Leave error variable untouched
       counts[badChanIndex,*] = !values.F_NAN
       error[badChanIndex,*] = 0.0
    endif
    
    
    ;; Apply monitor normalization
    case normTo of
       0: begin
          status = oData[i]->GetMetaData('MON',mon)
          monScale = monScaleVal/mon
       end
       
       1: begin
          if (~isa(time)) then begin    ; may have retrieve time already
             status = oData[i]->GetMetaData('TIME',time)
          endif
          monScale = monScaleVal/time
       end
       
       else: monScale = monScaleVal
    endcase
    counts = TEMPORARY(counts)*monScale
    error = TEMPORARY(error)*monscale
    
    if (moreThanOne gt 0) then begin
      aggY = [aggY, yvals]
      if (n_elements(yvals) eq 1) then begin
        ; ==> counts and error are vectors of length n so reform to an nx1
        ; array so aggregation statements can work
        counts = reform(counts,n_elements(counts),1)
        error = reform(error,n_elements(error),1)
      endif
      aggCnts = transpose([transpose(aggCnts),transpose(counts)])
      aggErr = transpose([transpose(aggErr),transpose(error)])
    endif else begin
      aggY = yvals
      aggCnts = counts
      aggErr = error   
    endelse
    
    moreThanOne++

    if (i eq 0) then begin
      xLabel = ''
      if (oX->GetMetaData('LONG_NAME',label)) then xLabel += label
      xUnits = ''
      if (oX->GetMetaData('UNITS',units)) then xUnits += units
      yLabel = ''
      if (oY->GetMetaData('LONG_NAME',label)) then yLabel += label
      yUnits = ''
      if (oY->GetMetaData('UNITS',units)) then yUnits += units
      zLabel = 'Counts'
    endif
  endfor

  counts = aggCnts
  error = aggErr
  yvals = aggY
  
  ; sort in increasing yvals
  if (n_elements(yvals) gt 1) then begin
    yindex = sort(yvals)
    yvals = yvals[yindex]
    counts = counts[*,yindex]
    error = error[*,yindex]
    Self->MergeDupGroups, yvals, counts, error, tolerance=0.001
  endif
  
  status = oData[0]->GetMetaData('DATAPTRREF',dataPtr)

  status = create_dave_pointer(davePtr, qty=counts, xvals=xvals, yvals=yvals, err=error)
  if (~status) then begin
     msg = "Unable to allocate memory!"
     Self->StatusMessage, msg
     if (ptr_valid(davePtr)) then heap_free, davePtr
     return, 0
  endif
      
  (*(*davePtr).dataStrPtr).commonStr.instrument = 'BT7'
  (*(*davePtr).dataStrPtr).commonStr.xlabel = xLabel
  (*(*davePtr).dataStrPtr).commonStr.xunits = xunits
  (*(*davePtr).dataStrPtr).commonStr.xtype = 'points'
  (*(*davePtr).dataStrPtr).commonStr.ylabel = yLabel
  (*(*davePtr).dataStrPtr).commonStr.yunits = yunits
  (*(*davePtr).dataStrPtr).commonStr.ytype = 'points'
   
  (*(*davePtr).dataStrPtr).commonStr.histlabel = zLabel
  (*(*davePtr).dataStrPtr).commonStr.histunits = ''
  (*(*(*davePtr).dataStrPtr).commonStr.treatmentPtr) = ''
   
  if (ptr_valid(dataPtr)) then begin
     specificStr = { $
                 filename:(*dataPtr).filename, $
                 monitor:(*dataPtr).monave, $
                 orientation:(*(*dataPtr).orientation_ptr), $
                 latticeParams:(*(*dataPtr).lattice_ptr), $
                 monDSpace:(*dataPtr).dspacem, $
                 anaDSpace:(*dataPtr).dspacea, $
                 pan_filler:!VALUES.F_NAN, $
                 pan_mask:ptr_new(bits2bytes(finite(counts))) $
                }
      
     (*(*davePtr).dataStrPtr).specificPtr = ptr_new(specificStr)
  endif
  
  ; davePtr is fully defined so exit now
  return, 1
endif

n = n_elements(oData)
xvals = []
yvals = []
evals = []
for i = 0,n-1 do begin
   oX = oData[i]->GetByName(xVar)
   status = oX->GetData(x1)
   oZ = oData[i]->GetByName('Counts')
   status = oZ->GetData(y1)
   oErr = oData[i]->GetByName('Error')
   status = oErr->GetData(e1)
   xvals = [xvals,x1]
   yvals = [yvals,y1]
   evals = [evals,e1]
endfor

index = sort(xvals)
xvals = xvals[index]
yvals = yvals[index]
evals = evals[index]

xLabel = ''
if (oX->GetMetaData('LONG_NAME',label)) then xLabel += label
xUnits = ''
if (oX->GetMetaData('UNITS',units)) then xUnits += units
yLabel = 'Counts'

if (~keyword_set(useTexLabels)) then begin
   xLabel = tex2idl(xLabel)
   xUnits = tex2idl(xunits)
   yLabel = tex2idl(yLabel)
endif

; convert to ASCII struct if required
if (keyword_set(ascii)) then begin
   struct = {xvals:xvals,yvals:yvals,evals:evals,xlabel:xlabel,xunits:xunits,ylabel:ylabel}
   davePtr = ptr_new(struct)
endif

; convert to davePtr struct if required
if (keyword_set(dave)) then begin
   status = oData[0]->GetMetaData('DATAPTRREF',dataPtr)

   status = create_dave_pointer(davePtr, qty=yvals, xvals=xvals, err=evals)
   if (~status) then begin
      msg = "Unable to allocate memory!"
      Self->StatusMessage, msg
      if (ptr_valid(davePtr)) then heap_free, davePtr
      return, 0
   endif
      
   (*(*davePtr).dataStrPtr).commonStr.instrument = 'BT7'
   (*(*davePtr).dataStrPtr).commonStr.xlabel = xLabel
   (*(*davePtr).dataStrPtr).commonStr.xunits = xunits
   (*(*davePtr).dataStrPtr).commonStr.xtype = 'points'
   (*(*davePtr).dataStrPtr).commonStr.ylabel = ''
   (*(*davePtr).dataStrPtr).commonStr.yunits = ''
   (*(*davePtr).dataStrPtr).commonStr.ytype = 'points'
   
   (*(*davePtr).dataStrPtr).commonStr.histlabel = yLabel
   (*(*davePtr).dataStrPtr).commonStr.histunits = ''
   (*(*(*davePtr).dataStrPtr).commonStr.treatmentPtr) = ''
   
   if (ptr_valid(dataPtr)) then begin
      specificStr = { $
                  filename:(*dataPtr).filename, $
                  monitor:(*dataPtr).monave, $
                  orientation:(*(*dataPtr).orientation_ptr), $
                  latticeParams:(*(*dataPtr).lattice_ptr), $
                  monDSpace:(*dataPtr).dspacem, $
                  anaDSpace:(*dataPtr).dspacea, $
                  pan_filler:!VALUES.F_NAN, $
                  pan_mask:ptr_new(bits2bytes(finite(yvals))) $
                 }
      
      (*(*davePtr).dataStrPtr).specificPtr = ptr_new(specificStr)
   endif
endif

return, 1
end


;===============================================================================
;+
; Bt7psdTool::WriteProjectAs
;
; PURPOSE:
;   save project (data plus state) to file
;
; PARAMETERS:
;
; KEYWORDS:
;-
function Bt7psdTool::WriteProjectAs
compile_opt idl2

; Basic error Handler
if (n_elements(!debug) && (!debug eq 0)) then begin
    catch, catchError
    if (catchError ne 0) then begin
        ;;print, 'Error handled!'
        eTitle = 'Bt7psdTool::writeProjectAs: Error encountered'
        eMsg = 'An error or unusual condition was encountered!'
        eMsg = [eMsg,'Please, report the following to the DAVE team:']
        eMsg = [eMsg,!error_state.msg]
        void = dialog_message(/error,eMsg,title=eTitle,dialog_parent=wTLB)
        catch, /cancel
        return, 0
    endif
endif

oUI = Self->GetUI()
oUI->GetProperty, group_leader = wTLB
wChild = widget_info(wTLB, /child)
widget_control, wChild, get_uvalue=sPtr

Self->GetProperty, nSamp=_nSamp, working_directory=workDir
if (_nSamp le 0) then begin
   msg = 'No data in current session so cannot proceed with save action!'
   Self->StatusMessage, msg
   Self->ErrorMessage, msg, severity=0, title='Save Unnecessary!'
   return, 0
endif


projName = file_basename(Self.projName,/fold_case,'.tisv')
oSamp = Self.sampContRef->Get(position = 0)
if (strcmp(projName,'Untitled',/fold_case) && obj_valid(oSamp)) then begin
   oSamp->GetProperty, description=desc
   projName = file_basename(desc,/fold_case,'.bt7')
   ; replace spaces in filename with _
   while ((pos = stregex(projName,' ')) ne -1) do strput, projName,'_',pos
endif
title = "Specify filename to store session"
ext = 'tisv'
filter = ['*.tisv']
projName = dialog_pickfile(title=title,filter=filter,default_extension=ext $
                          ,dialog_parent=wTLB, /write $
                          ,/overwrite_prompt, file=projName+'.tisv' $
                          ,path=workDir, get_path=new_workDir)

if (strcmp(projName,'')) then return, 0 

return, Self->writeProject(/noPrompt, new_workDir=file_dirname(projName,/mark_dir), new_projName=file_basename(projName,/fold_case,'.tisv'))

end


;===============================================================================
;+
; Bt7psdTool::WriteProject
;
; PURPOSE:
;   save project (data plus state) to file
;
; PARAMETERS:
;
; KEYWORDS:
;-
function Bt7psdTool::WriteProject, noPrompt=noPrompt, new_workDir=new_workDir,new_projName=new_projName 
compile_opt idl2

oUI = Self->GetUI()
oUI->GetProperty, group_leader = wTLB
wChild = widget_info(wTLB, /child)
widget_control, wChild, get_uvalue=sPtr

; Basic error Handler
if (n_elements(!debug) && (!debug eq 0)) then begin
    catch, catchError
    if (catchError ne 0) then begin
        ;;print, 'Error handled!'
        eTitle = 'Bt7psdTool::writeProject: Error encountered'
        eMsg = 'An error or unusual condition was encountered!'
        eMsg = [eMsg,'Please, report the following to the DAVE team:']
        eMsg = [eMsg,!error_state.msg]
        void = dialog_message(/error,eMsg,title=eTitle,dialog_parent=wTLB)
        catch, /cancel
        return, 0
    endif
endif

; save session preferences
status = Self->SavePreferences(/skip_filesave)

Self->GetProperty, sampTypeFlag=_sampTypeFlag, nSamp=_nSamp, nBkgd=_nBkgd $ ,nFast=_nFast $
                 , minIntensity=_minIntensity, maxIntensity=_maxIntensity $
                 , preferences=_prefStr, working_directory=workDir
noData = (_nSamp le 0) && (_nBkgd le 0) ;&& (_nFast le 0)
if (noData) then begin
   msg = 'No data in current session so cannot proceed with save action!'
   Self->StatusMessage, msg
   Self->ErrorMessage, msg, severity=0, title='Save Unnecessary!'
   return, 0
endif

projName = (keyword_set(new_projName))? new_projName : Self.projName
if (keyword_set(new_workDir)) then workDir = new_workDir

if (keyword_set(noPrompt)) then begin
   projName = workDir+projName+'.tisv'
endif else begin
   projName = Self.projName
   oSamp = Self.sampContRef->Get(position = 0)
   if (strcmp(projName,'Untitled',/fold_case) || strcmp(projName,'')) then begin
      oSamp->GetProperty, description=desc
      projName = file_basename(desc,/fold_case,'.bt7')
      ; replace spaces in filename with _
      while ((pos = stregex(projName,' ')) ne -1) do strput, projName,'_',pos

      title = "Specify filename to store session"
      ext = 'tisv'
      filter = ['*.tisv']
      projName = dialog_pickfile(title=title,filter=filter,default_extension=ext $
                                ,dialog_parent=wTLB, /write $
                                ,/overwrite_prompt, file=projName+'.tisv' $
                                ,path=workDir, get_path=new_workDir)
      ;projName = dialog_pickfile(title=title,default_extension=ext, dialog_parent=wTLB $
      ;                       ,/overwrite_prompt, file=projName+'.tisv',path=workDir, get_path=new_workDir)

   endif else projName = workDir+projName+'.tisv'
endelse
                       
if (~strcmp(projName,'')) then begin
   if (_nSamp gt 0) then _datasets = (Self.sampContRef->Get(/all))[0:_nSamp-1]
   if (_nBkgd gt 0) then _datasets = (n_elements(_datasets) gt 0)? $
                                     [_datasets,(Self.bkgdContRef->Get(/all))[0:_nBkgd-1]] : $
                                     (Self.bkgdContRef->Get(/all))[0:_nBkgd-1]
;   if (_nFast gt 0) then _datasets = (n_elements(_datasets) gt 0)? $
;                                     [_datasets,(Self.fastContRef->Get(/all))[0:_nFast-1]] : $
;                                     (Self.fastContRef->Get(/all))[0:_nFast-1]
   nd = n_elements(_datasets)
   _dataPtrs = ptrarr(nd)
   for i=0,nd-1 do begin
      void = _datasets[i]->GetMetaData('DATAPTRREF',dataPtr)
      _dataPtrs[i] = dataPtr
   endfor
   
   ; Get PSD calibration info
   if (Self.calibParamsIsSet) then begin
      _calibInfo = {calibParamsIsSet:Self.calibParamsIsSet $
                   ,calibParamsFilename:Self.calibParamsFilename $
                   ,calibParamsBaseFilename:Self.calibParamsBaseFilename $
                   ,chDelA4:Self.chDelA4 $
                   ,chDelEf:Self.chDelEf $
                   ,chEff:Self.chEff $
                   }
   endif else _calibInfo = {calibParamsIsSet:Self.calibParamsIsSet}
   
   ; Get masking details
   Self->GetProperty, maskedDets=_maskedDets, maskFlag=_maskFlag

   save, _datasets, _dataPtrs, _nSamp, _prefStr, _calibInfo, _nBkgd $, _nFast
       , _minIntensity, _maxIntensity, _sampTypeFlag, _maskedDets, _maskFlag $
       , description = 'BT7PSD Data Reduction Session' $
       , filename=projName, /compress
   if (isa(new_workDir) && file_test(new_workDir,/directory)) then Self->SetProperty, working_directory=new_workDir

   msg = 'Session stored in: '+projName
   Self->StatusMessage, msg
   
   projName = file_basename(projName,/fold_case,'.tisv')
   Self.projName = projName
   Self->SetProperty, tool_filename=projName
   
   Self.modifiedFlag = 0
endif

return, 1
end


;===============================================================================
; Bt7psdTool::readProject
;
; PURPOSE:
;   export dataset to DAVE Data Manager
;
; PARAMETERS:
;
; KEYWORDS:
;
function Bt7psdTool::ReadProject
compile_opt idl2

oUI = Self->GetUI()
oUI->GetProperty, group_leader = wTLB
wChild = widget_info(wTLB, /child)
widget_control, wChild, get_uvalue=sPtr

; Basic error Handler
if (n_elements(!debug) && (!debug eq 0)) then begin
    catch, catchError
    if (catchError ne 0) then begin
        ;;print, 'Error handled!'
        eTitle = 'Bt7psdTool::ReadProject: Error encountered'
        eMsg = 'An error or unusual condition was encountered!'
        eMsg = [eMsg,'Please, report the following to the DAVE team:']
        eMsg = [eMsg,!error_state.msg]
        void = dialog_message(/error,eMsg,title=eTitle,dialog_parent=wTLB)
        catch, /cancel
        if (obj_valid(oSave)) then obj_destroy, oSave
        Self->EnableUpdates
        return, 0
    endif
endif

filter = '*.tisv'
Self->GetProperty, working_directory=workDir
if (n_elements(title) eq 0) then title='Specify session to load'

filename = dialog_pickfile(title=title,/must_exist, dialog_parent=wTLB $
                       ,filter=filter,path=workDir, get_path=newWorkDir)

if (filename eq '') then return, 0
Self->SetProperty, work_directory=newWorkDir

oSave = obj_new('IDL_Savefile', filename)
contents = oSave->Contents()
obj_destroy, oSave
expectedDesc='BT7PSD Data Reduction Session'
if (~strcmp(contents.description,expectedDesc,/fold_case)) then begin
   msg = 'Not a valid BT7 PSD Reduction session file!'
   Self->StatusMessage, msg
   Self->ErrorMessage, msg, severity=1, title='Invalid file type!'   
   return, 0
endif

restore, filename, restored_objects=oObj, /relaxed_structure_assignment
nObj = n_elements(oObj)
if (nObj le 0) then return, 0

; Check whether or not to save current session before closing it
if (Self.modifiedFlag) then begin
   ; If our current state is dirty, prompt to save first.
   Self->GetProperty, working_directory=workDir
   outfile = workDir+Self.projName+'.tisv'
   status = self->PromptUserYesNo( $
       "Save changes to " + outfile + "?", $
       answer, $
       DEFAULT_NO=0, $
       TITLE='Save', $
       /CANCEL)

   if (status && (answer eq 1)) then begin
      void = Self->writeProject(/noPrompt)
   endif

endif

Self->DisableUpdates

; delete all datasets in current session
void = Self->DeleteAllDatasets(/noPrompt)

projName = file_basename(filename,/fold_case,'.tisv')
Self.projName = projName

; update preferences structure to most current definition
status = BT7PSDReducPreferences(_prefStr,/updateVersion) 

Self->SetProperty, preferences=_prefStr, tool_filename=projName $
                 , maxIntensity=_maxIntensity, minIntensity=_minIntensity, maskedDets=_maskedDets, maskFlag=_maskFlag
Self->SetPropertyAttribute, 'MASKEDDETS', userdef=_maskedDets, sensitive=(_maskFlag eq 1)
; PSD calibration stuff
; Only alter current calib info if one was saved with the project file!
if (_calibInfo.calibParamsIsSet) then begin
   Self.calibParamsIsSet = _calibInfo.calibParamsIsSet
   Self.calibParamsBaseFilename = _calibInfo.calibParamsBaseFilename
   Self.calibParamsFilename = _calibInfo.calibParamsFilename
   Self.chDelA4 = _calibInfo.chDelA4
   Self.chDelEf = _calibInfo.chDelEf
   Self.chEff = _calibInfo.chEff
   Self->SetPropertyAttribute, 'calibParamsBaseFilename', userdef=Self.calibParamsBaseFilename
endif
Self->SetPropertyAttribute, 'effNormFlag', sensitive=_calibInfo.calibParamsIsSet


Self->GetProperty, maskFlag=mF, fastFlag=fF, xAxisVar=xAxisVar $
                 , binWidths=binWidths, defaultIntensityFlag=defIntFlag, sampTypeFlag=sampTypeFlag
Self->SetProperty, minBinWidth = binwidths[xAxisVar]
noHide = (defIntFlag eq 0) && (sampTypeFlag eq 1)  ; don't hide if not using default intensity and single crystal sample
Self->SetPropertyAttribute, 'maxIntensity', hide=~noHide
Self->SetPropertyAttribute, 'minIntensity', hide=~noHide
Self->SetPropertyAttribute, 'MASKEDDETS', sensitive=mF
Self->SetPropertyAttribute, 'FASTVALUE', sensitive=fF
Self->SetPropertyAttribute, 'MONCORFLAG', sensitive=fF

; restore datasets
nd = n_elements(_datasets)
for i=0, nd-1 do begin
   oData = _datasets[i]
   Self->restoreDataVersion, oData
   oData->GetProperty, type=dataType
   case dataType of
      'samp': begin
         Self->AddByIdentifier, 'Sample Data Manager', oData, position=0
         Self.sampContRef->Add, oData
      end
      
      'bkgd': begin
         Self->AddByIdentifier, 'Bkgd Data Manager', oData, position=0
         Self.bkgdContRef->Add, oData
      end
      
      else:
   
   endcase

   ; If samTypeFlag variable was not saved (this would be case for projects save before July, 2012),
   ; then get the value using the sampTypeFlag property of the first dataset
   if (n_elements(_sampTypeFlag) eq 0) then begin
      status = oData->GetMetaData('sampTypeFlag',_sampTypeFlag)
   endif
endfor
Self->SetProperty, sampTypeFlag=_samptypeFlag, typeCheck = 0


Self.nSamp = _nSamp
if (n_elements(_nBkgd) gt 0) then Self.nBkgd = _nBkgd


;Self->SetPropertyAttribute, 'SUMSAMPFLAG', sensitive=(Self.nSamp gt 0)
;
;Self->SetPropertyAttribute, 'SUMBKGDFLAG', sensitive=(Self.nBkgd gt 0)
;Self->SetPropertyAttribute, 'BKGDFLAG', sensitive=(Self.nBkgd gt 0)

Self->UpdatePropertySheet

;; Select a dataset and plot it
if (_nSamp gt 0) then begin
   oItem = Self.sampContRef->Get(position = 0)
   dataType='samp'
endif else if (_nBkgd gt 0) then begin
   oItem = Self.bkgdContRef->Get(position = 0)
   dataType='bkgd'
endif 

if (obj_valid(oItem)) then begin
   (*sPtr).itemID = oItem->GetFullIdentifier()
   cw_bt7psddatamanager_setselect, (*sPtr).wDM, (*sPtr).itemID, /clear, dataType=dataType ; select it

   wd_Bt7psdTool_sendPlotEvent, oUI, oItem  ; plot it
endif

Self.modifiedFlag = 0
Self->UpdatePropertySheet
Self->refreshPropertySheet
Self->EnableUpdates
return, 1
end


;---------------------------------------------------------------------------
; Lifecycle Routines
;---------------------------------------------------------------------------
; Bt7psdTool::Init
;
; Purpose:
; The constructor of the Bt7psdTool object.
;
; Parameters:
; None.
;
function Bt7psdTool::Init, _REF_EXTRA=_EXTRA
compile_opt idl2

;; Call our super class
if (~self->IDLitTool::Init(_EXTRA=_extra)) then $
  return, 0

oSystem = self->_GetSystem()

; Specify current data version number
; Essentially, whevever the data structure used to store 
; datasets changes the version number should be incremented to reflect
; this. When the version number changes, restoreDataVersion() must also
; be modified to handle restoring older version datasets to the
; current version.
Self.dataVersion = 1

; Add a 'DAVEDATA_MANAGER' service
;self->AddService, OBJ_NEW("DAVEsrvDatamanager", name="DAVEDATA_MANAGER")

; Create the data manager folders
oSampFolder = Obj_New("IDLitDataManagerFolder", identifier='Sample Data Manager' $
                      ,name='Sample', description='Sample Data Manager Container',tool=self)
oBkgdFolder = Obj_New("IDLitDataManagerFolder", identifier='Bkgd Data Manager' $
                      ,name='Background', description='Bkgd Data Manager Container',tool=self)
self->Add, [oSampFolder,oBkgdFolder]


self->createfolders,'Operations/       ',NAME='', description='Horizontal Space'
;;---------------------------------------------------------------------
;;*** File Menu

;; create folders
self->createfolders,'Operations/File',NAME=IDLitLangCatQuery('Menu:File')
;   self->createfolders,'Operations/File/New', $
;                       NAME=IDLitLangCatQuery('Menu:File:New')

;     self->RegisterOperation, 'iPlot', 'IDLitOpNewTool', $
;         IDENTIFIER='File/New/Plot'

;     self->RegisterOperation, 'iSurface', 'IDLitOpNewTool', $
;         IDENTIFIER='File/New/Surface'

;     self->RegisterOperation, 'iContour', 'IDLitOpNewTool', $
;         IDENTIFIER='File/New/Contour'

;     self->RegisterOperation, 'iImage', 'IDLitOpNewTool', $
;         IDENTIFIER='File/New/Image'

;     self->RegisterOperation, 'iVolume', 'IDLitOpNewTool', $
;         IDENTIFIER='File/New/Volume'

;     self->RegisterOperation, 'iMap', 'IDLitOpNewTool', $
;         IDENTIFIER='File/New/Map'

;self->RegisterOperation, IDLitLangCatQuery('Menu:File:Open'), $
;  'IDLitopFileOpen', $
;  ACCELERATOR='Ctrl+O', $
;  DESCRIPTION='Open an existing data or image file', $
;  IDENTIFIER='File/Open', ICON='open'


                                ;-----------------
;self->RegisterOperation, IDLitLangCatQuery('Menu:File:Import'), $
;  'IDLitopImportData', $
;  IDENTIFIER='File/Import', $
;  /SEPARATOR
;self->RegisterOperation, IDLitLangCatQuery('Menu:File:Export'), $
;  'IDLitopExportData',icon='camera', $
;  IDENTIFIER='File/Export',/separator



self->RegisterOperation, 'Open Session', $
  'DAVEopMenuItem',icon='open', IDENTIFIER='File/readProject'

self->RegisterOperation, 'Save Session', $
  'DAVEopMenuItem',icon='save', IDENTIFIER='File/writeProject'
self->RegisterOperation, 'Save Session As', $
  'DAVEopMenuItem',icon='Save_24', IDENTIFIER='File/writeProjectAs'

                                ;-----------------
;self->RegisterOperation, IDLitLangCatQuery('Menu:File:Save'), $
;  'IDLitopFileSave', $
;  ACCELERATOR='Ctrl+S', $
;  IDENTIFIER='File/Save', ICON='save', /SEPARATOR

;self->RegisterOperation, IDLitLangCatQuery('Menu:File:SaveAs'), $
;  'IDLitopFileSaveAs', $
;  IDENTIFIER='File/SaveAs', ICON='save'

                                ;-----------------
Self->RegisterOperation,'Export Graphics...' $
  ,'DAVEopExportGraphics' $
  ,identifier='File/Export' $
  ,ACCELERATOR='Ctrl+E' $
  ,writertypes=['IDLDEST','IDLIMAGE'] $
  ,icon='export',/separator

self->RegisterOperation, IDLitLangCatQuery('Menu:File:PrintPreview'), $
  'IDLitopPrintPreview', $
  DESCRIPTION='Print the contents of the active window', $
  IDENTIFIER='File/PrintPreview', ICON='print'

self->RegisterOperation, IDLitLangCatQuery('Menu:File:Print'), $
  'IDLitopFilePrint', $
  ACCELERATOR='Ctrl+P', $
  DESCRIPTION='Print the contents of the active window', $
  IDENTIFIER='File/Print', ICON='print_24'


self->RegisterOperation, 'Save Preferences', $
  'DAVEopMenuItem',icon='save', IDENTIFIER='File/savePreferences',/separator

self->RegisterOperation, 'Reset Preferences (Default Values)', $
  'DAVEopMenuItem',icon='save', IDENTIFIER='File/resetPreferences'

;self->RegisterOperation, IDLitLangCatQuery('Menu:File:Preferences'), $
;  'IDLitopBrowserPrefs', $
;  IDENTIFIER='File/Preferences', /SEPARATOR

                                ;-----------------
self->RegisterOperation, 'Exit', 'DAVEopMenuItem', ACCELERATOR='Ctrl+Q', $
  IDENTIFIER='File/Exit', /SEPARATOR

self->RegisterOperation, 'ExitOp', 'DAVEopFileExit', IDENTIFIER='ExitOp'


;-----------
; Data Input Menu
self->createfolders,'Operations/Input',NAME='Data Input'

self->RegisterOperation, 'Load PSD Calibration Parameters file', $
  'DAVEopMenuItem',icon='open', IDENTIFIER='Input/loadCalibParams'
self->RegisterOperation, 'View Loaded PSD Calibration Parameters', $
  'DAVEopMenuItem',icon='open', IDENTIFIER='Input/ViewCalibParams'
;self->RegisterOperation, 'Channel Efficiency', $
;  'DAVEopMenuItem',icon='open', IDENTIFIER='Input/PSD Calibration/loadChEfficiency'
;self->RegisterOperation, 'Channel A4 Spacing', $
;  'DAVEopMenuItem',icon='open', IDENTIFIER='Input/PSD Calibration/loadChA4'
;self->RegisterOperation, 'Channel Ef Spacing', $
;  'DAVEopMenuItem',icon='open', IDENTIFIER='Input/PSD Calibration/loadChEf'

self->RegisterOperation, 'Load Sample Data', $
  'DAVEopMenuItem',icon='open', IDENTIFIER='Input/loadSamp', /separator

self->RegisterOperation, 'Load Sample Data (NCNR ftp server)', $
  'DAVEopMenuItem',icon='open', IDENTIFIER='Input/loadSampFtp'

self->RegisterOperation, 'Load Background Data', $
  'DAVEopMenuItem',icon='open', IDENTIFIER='Input/loadBkgd', /separator

self->RegisterOperation, 'Load Background Data (NCNR ftp server)', $
  'DAVEopMenuItem',icon='open', IDENTIFIER='Input/loadBkgdFtp'

self->RegisterOperation, 'Delete All Loaded Data', $
  'DAVEopMenuItem',icon='open', IDENTIFIER='Input/deleteAllDatasets',/separator



;-----------
; Data Output Menu
self->createfolders,'Operations/Output',NAME='Data Output'
;
;self->RegisterOperation, 'Save output as DAVE file', $
;  'DAVEopMenuItem',icon='save', IDENTIFIER='Output/saveAsDave', sensitive=0
;
;self->RegisterOperation, 'Save output as ASCII file', $
;  'DAVEopMenuItem',icon='save', IDENTIFIER='Output/saveAsAscii', sensitive=0
;
;self->RegisterOperation, 'Export output to DAVE Data Maneger', $
;  'DAVEopMenuItem',icon='save', IDENTIFIER='Output/exportToDaveDM', sensitive=0
;
;self->RegisterOperation, 'Fit output in PAN', $
;  'DAVEopMenuItem',icon='save', IDENTIFIER='Output/exportToPAN',/separator, sensitive=0
  
self->RegisterOperation, 'Export Selection to Mslice (Single Crystal, powder)', $
  'DAVEopMenuItem',icon='save', IDENTIFIER='Output/Mslice'
self->RegisterOperation, 'Export Selection to Data Manager (Powder, Raw)', $
  'DAVEopMenuItem',icon='save', IDENTIFIER='Output/ExportToDaveDM',/separator
self->RegisterOperation, 'Save Selection as DAVE file (Powder, Raw)', $
  'DAVEopMenuItem',icon='save', IDENTIFIER='Output/SaveAsDave'
self->RegisterOperation, 'Save Selection as ASCII file (Powder, Raw)', $
  'DAVEopMenuItem',icon='save', IDENTIFIER='Output/SaveAsASCII'
self->RegisterOperation, 'Fit Selection in PAN (Powder, Raw)', $
  'DAVEopMenuItem',icon='save', IDENTIFIER='Output/exportToPAN',/separator




;;---------------------------------------------------------------------
;; Create our File toolbar container.
;;
;self->Register, IDLitLangCatQuery('Menu:File:New'), 'IDLitOpNewTool', $
;                IDENTIFIER='Toolbar/File/NewTool', ICON='new'
;self->Register, IDLitLangCatQuery('Menu:File:Open'), $
;                PROXY='Operations/File/Open', $
;                IDENTIFIER='Toolbar/File/Open'
;self->Register, IDLitLangCatQuery('Menu:File:Save'), $
;                PROXY='Operations/File/Save', $
;                IDENTIFIER='Toolbar/File/Save'
self->Register, "Open Session", $
                PROXY='Operations/File/ReadProject', $
                IDENTIFIER='Toolbar/File/ReadProject'
self->Register, "Save Session", $
                PROXY='Operations/File/WriteProject', $
                IDENTIFIER='Toolbar/File/WriteProject'
self->Register, "Save Session As", $
                PROXY='Operations/File/WriteProjectAs', $
                IDENTIFIER='Toolbar/File/WriteProjectAs'
self->Register, IDLitLangCatQuery('Menu:File:Print'), $
                PROXY='Operations/File/Print', $
                IDENTIFIER='Toolbar/File/Print'
self->Register, IDLitLangCatQuery('Menu:File:PrintPreview'), $
                PROXY='Operations/File/PrintPreview', $
                IDENTIFIER='Toolbar/File/PrintPreview'
self->Register, IDLitLangCatQuery('Menu:File:Export'), $
                PROXY='Operations/File/Export', $
                IDENTIFIER='Toolbar/File/Export'


;;---------------------------------------------------------------------
;;*** Edit Menu

self->createfolders,'Operations/Edit',NAME=IDLitLangCatQuery('Menu:Edit')

self->RegisterOperation, IDLitLangCatQuery('Menu:Edit:Undo'), $
  'IDLitopUndo', $
  ACCELERATOR='Ctrl+Z', $
  IDENTIFIER='Edit/Undo', ICON='undo', /disable, $
  /IGNORE_AVAILABILITY

self->RegisterOperation, IDLitLangCatQuery('Menu:Edit:Redo'), $
  'IDLitopRedo', $
  ACCELERATOR='Ctrl+Y', $
  IDENTIFIER='Edit/Redo', ICON='redo',/disable, $
  /IGNORE_AVAILABILITY

;; Clipboard...note these are proxied from the system registry!

self->RegisterOperation, IDLitLangCatQuery('Menu:Edit:Cut'), $
  PROXY="/REGISTRY/OPERATIONS/CUT",$
  IDENTIFIER='Edit/Cut'

self->RegisterOperation, IDLitLangCatQuery('Menu:Edit:Copy') , $
  PROXY='/REGISTRY/OPERATIONS/COPY', $
  IDENTIFIER='Edit/Copy'

self->RegisterOperation, IDLitLangCatQuery('Menu:Edit:Paste') , $
  PROXY='/REGISTRY/OPERATIONS/Paste', $
  IDENTIFIER='Edit/Paste'

self->RegisterOperation, IDLitLangCatQuery('Menu:Edit:PasteSpecial') , $
  PROXY='/REGISTRY/OPERATIONS/PASTESPECIAL', $
  IDENTIFIER='Edit/PasteSpecial'

                                ;-----------------
self->RegisterOperation, IDLitLangCatQuery('Menu:Edit:Delete') , $
  PROXY='/REGISTRY/OPERATIONS/Delete', $
  IDENTIFIER='Edit/Delete'

self->RegisterOperation, IDLitLangCatQuery('Menu:Edit:SelectAll') , $
  'IDLitopSelectAll', $
  DESCRIPTION="Select all visualizations in the current view.", $
  IDENTIFIER='Edit/SelectAll', $
  ICON='select'


                                ;-----------------
self->createfolders,'Operations/Edit/Grouping', $
                    NAME=IDLitLangCatQuery('Menu:Edit:Grouping')

self->RegisterOperation, IDLitLangCatQuery('Menu:Edit:Group'), $
  'IDLitopGroup', $
  IDENTIFIER='Edit/Grouping/Group', $
  ICON='group', $
  /SEPARATOR

self->RegisterOperation, IDLitLangCatQuery('Menu:Edit:Ungroup'), $
  'IDLitopUngroup', $
  IDENTIFIER='Edit/Grouping/Ungroup', $
  ICON='ungroup'

;self->createfolders,'Operations/Edit/Order', $
;                    NAME=IDLitLangCatQuery('Menu:Edit:Order')
;
;self->RegisterOperation, IDLitLangCatQuery('Menu:Edit:BringToFront'), $
;  'IDLitopBringToFront', $
;  IDENTIFIER='Edit/Order/BringToFront', $
;  ICON='front'
;
;self->RegisterOperation, IDLitLangCatQuery('Menu:Edit:SendToBack'), $
;  'IDLitopSendToBack', $
;  IDENTIFIER='Edit/Order/SendToBack', $
;  ICON='back'
;
;self->RegisterOperation, IDLitLangCatQuery('Menu:Edit:BringForward'), $
;  'IDLitopBringForward', $
;  IDENTIFIER='Edit/Order/BringForward', $
;  ICON='front'
;
;self->RegisterOperation, IDLitLangCatQuery('Menu:Edit:SendBackward'), $
;  'IDLitopSendBackward', $
;  IDENTIFIER='Edit/Order/SendBackward', $
;  ICON='back'


;;---------------------------------------------------------------------
;;*** Format Menu

self->createfolders,'Operations/Edit/Style', $
                    NAME=IDLitLangCatQuery('Menu:Style')

self->RegisterOperation, IDLitLangCatQuery('Menu:Style:ApplyStyle') , $
  PROXY='/REGISTRY/OPERATIONS/Apply Style', $
  IDENTIFIER='Edit/Style/Apply Style'

self->RegisterOperation, $
  IDLitLangCatQuery('Menu:Style:CreateStylefromSelection'), $
  'IDLitopStyleCreate', $
  IDENTIFIER='Edit/Style/Create Style', $
  ICON='style'

self->RegisterOperation, IDLitLangCatQuery('Menu:Style:StyleEditor'), $
  'IDLitopStyleEditor', $
  IDENTIFIER='Edit/Style/StyleEditor', $
  ICON='style'


;;-----------------
;; Register here but make private. We only expose this operation
;; in the context menu, but we want it to be (de)sensitized along with
;; the rest of the Edit menu.
;self->RegisterOperation, 'Export to IDL...', 'IDLitopclExport', $
;  /PRIVATE, $
;  DESCRIPTION='Export Visualization Parameters to IDL Variables. ', $
;  IDENTIFIER='Edit/CLExport', $
;  /separator


;;-----------------
;self->RegisterOperation, IDLitLangCatQuery('Menu:Edit:Parameters'), $
;  'IDLitopEditParameters', $
;  IDENTIFIER='Edit/EditParameters', $
;  ICON='binary', $
;  /separator
self->RegisterOperation, 'Edit Parameters','IDLitopEditParameters'

Self->RegisterOperation, 'SaveProperties','DAVEopMenuItem',identifier='Edit/SaveProperties'

self->RegisterOperation, IDLitLangCatQuery('Menu:Edit:Properties'), $
  'IDLitopPropertySheet', $
  DESCRIPTION='Display the Property Sheet for the item', $
  IDENTIFIER='Edit/Properties', $
  ICON='propsheet'


;;---------------------------------------------------------------------
;; Create our Edit toolbar container.
;;
self->Register, IDLitLangCatQuery('Menu:Edit:Undo'), $
                PROXY='Operations/Edit/Undo', $
                IDENTIFIER='Toolbar/Edit/Undo'
self->Register, IDLitLangCatQuery('Menu:Edit:Redo'), $
                PROXY='Operations/Edit/Redo', $
                IDENTIFIER='Toolbar/Edit/Redo'
self->Register, IDLitLangCatQuery('Menu:Edit:Cut'), $
                PROXY="/REGISTRY/OPERATIONS/CUT",$
                IDENTIFIER='Toolbar/Edit/Cut'
self->Register, IDLitLangCatQuery('Menu:Edit:Copy'), $
                PROXY='/REGISTRY/OPERATIONS/COPY', $
                IDENTIFIER='Toolbar/Edit/Copy'
self->Register, IDLitLangCatQuery('Menu:Edit:Paste'), $
                PROXY='/REGISTRY/OPERATIONS/Paste', $
                IDENTIFIER='Toolbar/Edit/Paste'


;;---------------------------------------------------------------------
;;*** Insert menu

self->createfolders,'Operations/Insert', $
                    NAME=IDLitLangCatQuery('Menu:Insert')

;self->RegisterOperation, IDLitLangCatQuery('Menu:Insert:Visualization'), 'IDLitopInsertVis', $
;;  IDENTIFIER='Insert/Visualization', ICON='view'

self->RegisterOperation, 'Insert Visualization','IDLitopInsertVis';, $
;  IDENTIFIER='Insert/Visualization', ICON='view'


self->RegisterOperation, IDLitLangCatQuery('Menu:Insert:View'), $
  'IDLitopInsertView', $
  IDENTIFIER='Insert/View', ICON='view'

self->RegisterOperation, IDLitLangCatQuery('Menu:Insert:DataSpace'), $
  'IDLitopInsertDataSpace';, $
  ;IDENTIFIER='Insert/Data Space', ICON='mcr'

;self->createfolders,'Operations/Insert/Axis', NAME=IDLitLangCatQuery('Menu:Insert:Axis')

self->RegisterOperation, IDLitLangCatQuery('Menu:Insert:XAxis'), $
  'IDLitOpInsertAxisX';, IDENTIFIER='Insert/Axis/X Axis', ICON='mcr'
self->RegisterOperation, IDLitLangCatQuery('Menu:Insert:YAxis'), $
  'IDLitOpInsertAxisY';, IDENTIFIER='Insert/Axis/Y Axis', ICON='mcr'
self->RegisterOperation, IDLitLangCatQuery('Menu:Insert:ZAxis'), $
  'IDLitOpInsertAxisZ';, IDENTIFIER='Insert/Axis/Z Axis', ICON='mcr'


;;---------------------------------------------------------------------
;;*** Operations Menu
self->createfolders,'Operations/Operations', $
                    NAME=IDLitLangCatQuery('Menu:Operations')

;self->RegisterOperation, $
;  IDLitLangCatQuery('Menu:Operations:OperationsBrowser'), $
;  'IDLitopBrowserOperation', $
;  IDENTIFIER='Operations/Operations Browser' ; ,ICON='mcr'

self->createfolders,'Operations/Operations/Macros', $
                    NAME=IDLitLangCatQuery('Menu:Operations:Macros')

self->RegisterOperation, $
  IDLitLangCatQuery('Menu:Operations:RunMacro'), $
  PROXY='/Registry/MacroTools/Run Macro', $
  IDENTIFIER='Operations/Macros/Run Macro'
self->RegisterOperation, $
  IDLitLangCatQuery('Menu:Operations:StartRecording'), $
  PROXY='/Registry/MacroTools/Start Recording', $
  IDENTIFIER='Operations/Macros/Start Recording'
self->RegisterOperation, $
  IDLitLangCatQuery('Menu:Operations:StopRecording'), $
  PROXY='/Registry/MacroTools/Stop Recording', $
  IDENTIFIER='Operations/Macros/Stop Recording'
self->RegisterOperation, $
  IDLitLangCatQuery('Menu:Operations:MacroEditor'), $
  PROXY='/Registry/MacroTools/Macro Editor', $
  IDENTIFIER='Operations/Macros/Macro Editor'

;self->RegisterOperation, $
;  IDLitLangCatQuery('Menu:Operations:Statistics'), $
;  'IDLitopStatistics', $
;  DESCRIPTION='Display statistics for the selected item', $
;  IDENTIFIER='Operations/Statistics', ICON='sum', /SEPARATOR
;
;;self->RegisterOperation, IDLitLangCatQuery('Menu:Operations:Histogram'), $
;;  'IDLitopHistogram', $
;;  DESCRIPTION='Perform the histogram operation on the selected item', $
;;  IDENTIFIER='Operations/Histogram', ICON='hist'
;
self->createfolders,'Operations/Operations/Filter', $
                    NAME=IDLitLangCatQuery('Menu:Operations:Filter')

;self->RegisterOperation, $
;  IDLitLangCatQuery('Menu:Operations:Convolution'), $
;  'IDLitopConvolution', $
;  DESCRIPTION='Perform the convolution operation on the selected item', $
;  IDENTIFIER='Operations/Filter/Convolution', ICON='sum'
;
self->RegisterOperation, $
  IDLitLangCatQuery('Menu:Operations:Median'), $
  'IDLitopMedianFilter', $
  DESCRIPTION='Perform the median filter operation on the selected item', $
  IDENTIFIER='Operations/Filter/Median', ICON='sum'

self->RegisterOperation, IDLitLangCatQuery('Menu:Operations:Smooth'), $
  'IDLitopSmooth', $
  DESCRIPTION='Perform the smooth operation on the selected item', $
  IDENTIFIER='Operations/Filter/Smooth', ICON='sum'

self->createfolders,'Operations/Operations/Rotate', $
                    NAME=IDLitLangCatQuery('Menu:Operations:Rotate')

self->RegisterOperation, $
  IDLitLangCatQuery('Menu:Operations:RotateLeft'), $
  'IDLitopRotateLeft', $
  DESCRIPTION='Rotate left by 90 degrees', $
  IDENTIFIER='Operations/Rotate/RotateLeft', $
  ICON='rotate'

self->RegisterOperation, $
  IDLitLangCatQuery('Menu:Operations:RotateRight'), $
  'IDLitopRotateRight', $
  DESCRIPTION='Rotate right by 90 degrees', $
  IDENTIFIER='Operations/Rotate/RotateRight', $
  ICON='rotate'

self->RegisterOperation, $
  IDLitLangCatQuery('Menu:Operations:RotateByAngle'), $
  'IDLitopRotateByAngle', $
  DESCRIPTION='Rotate by a specified angle', $
  IDENTIFIER='Operations/Rotate/RotateByAngle', $
  ICON='rotate'

;self->createfolders,'Operations/Operations/Transform', $
;                    NAME=IDLitLangCatQuery('Menu:Operations:Transform')
;
;self->RegisterOperation, IDLitLangCatQuery('Menu:Operations:Resample'), $
;  'IDLitopResample', $
;  IDENTIFIER='Operations/Transform/Resample', ICON='sum'
;
;self->RegisterOperation, $
;  IDLitLangCatQuery('Menu:Operations:RotateData'), $
;  'IDLitopRotateData', $
;  DESCRIPTION='Rotate the data by a specified angle', $
;  IDENTIFIER='Operations/Transform/RotateData', $
;  ICON='sum'
;
;self->RegisterOperation, IDLitLangCatQuery('Menu:Operations:ScaleData'), $
;  'IDLitopScaleFactor', $
;  DESCRIPTION='Scale the data by a given factor', $
;  IDENTIFIER='Operations/Transform/ScaleFactor', ICON='sum'
;

;self->RegisterOperation, IDLitLangCatQuery('Menu:Operations:MapProjection'), $
;  'IDLitopMapProjection', $
;  IDENTIFIER='Operations/Map Projection', ICON='surface'


;;---------------------------------------------------------------------
;;*** Window Menu

self->createfolders,'Operations/Window', $
                    NAME=IDLitLangCatQuery('Menu:Window')

;;  Note: temporarily disabling ICON settings for browsers.
;self->RegisterOperation, IDLitLangCatQuery('Menu:Window:DataManager'), $
;  'IDLitopDataManager', $
;  IDENTIFIER='Window/Data Manager', $
;  ICON='prop'                   ; , ICON='mcr'

;     self->RegisterOperation,'Manipulator Browser','IDLitopBrowserManip', $
;         IDENTIFIER='Window/Manipulator Browser'; , ICON='mcr'

;     self->RegisterOperation, 'Tool Browser', 'IDLitopBrowserTool', $
;         IDENTIFIER='Window/Tool Browser'; , ICON='mcr'

;self->RegisterOperation, $
;  IDLitLangCatQuery('Menu:Window:VisualizationBrowser'), $
;  'IDLitopBrowserVis', $
;  IDENTIFIER='Window/Visualization Browser' ; , ICON='mcr'

;;-----------------
Self->RegisterOperation, 'Reset to Default Data Range', 'IDLitopRangeReset' $
   ,description='Reset to default data range', identifier='Window/Resetrange' $
   ,icon='range_reset'

;;--
self->createfolders,'Operations/Toolbar/Dataspace',name='Dataspace'
self->Register, 'Reset to Default Data Range', $
                PROXY='Operations/Window/Resetrange', $
                IDENTIFIER='Toolbar/Dataspace/Resetrange'

self->createfolders,'Operations/Window/Canvas Zoom', $
                    NAME=IDLitLangCatQuery('Menu:Window:CanvasZoom')

self->RegisterOperation, '800%', 'IDLitopCanvasZoom', $
  IDENTIFIER='Window/Canvas Zoom/800%', $
  ICON='zoom', /CHECKED;, /SEPARATOR
self->RegisterOperation, '400%', 'IDLitopCanvasZoom', $
  IDENTIFIER='Window/Canvas Zoom/400%', $
  ICON='zoom', /CHECKED
self->RegisterOperation, '200%', 'IDLitopCanvasZoom', $
  IDENTIFIER='Window/Canvas Zoom/200%', $
  ICON='zoom', /CHECKED
self->RegisterOperation, '100%', 'IDLitopCanvasZoom', $
  IDENTIFIER='Window/Canvas Zoom/100%', $
  ICON='zoom', /CHECKED
self->RegisterOperation, '75%', 'IDLitopCanvasZoom', $
  IDENTIFIER='Window/Canvas Zoom/75%', $
  ICON='zoom', /CHECKED
self->RegisterOperation, '50%', 'IDLitopCanvasZoom', $
  IDENTIFIER='Window/Canvas Zoom/50%', $
  ICON='zoom', /CHECKED
self->RegisterOperation, '25%', 'IDLitopCanvasZoom', $
  IDENTIFIER='Window/Canvas Zoom/25%', $
  ICON='zoom', /CHECKED

self->RegisterOperation, IDLitLangCatQuery('Menu:Window:ZoomonResize'), $
  'IDLitopZoomResize', $
  ICON='zoom', $
  IDENTIFIER='Window/ZoomResize', /CHECKED

self->RegisterOperation, IDLitLangCatQuery('Menu:Window:Layout'), $
  'IDLitopWindowLayout', $
  IDENTIFIER='Window/Layout'

                                ;-----------------

self->RegisterOperation, IDLitLangCatQuery('Menu:Window:FittoView'), $
  'IDLitopFitToView', $
  IDENTIFIER='Window/FitToView', $
  ICON='fitwindow', /SEPARATOR

;;---------------------------------------------------------------------
;;*** Help Menu
self->createfolders,'Operations/Help', $
                    NAME=IDLitLangCatQuery('Menu:Help')


self->RegisterOperation, "User's Manual", 'DAVEopHelp', $
  IDENTIFIER='Help/BT7PSDReductionManual',helptopic='BT7PSD_REDUCTION'
;-----------------

self->RegisterOperation, 'About BT7PSD Data Reduction', 'BT7PSDopHelpAbout', $
  IDENTIFIER='Help/About BT7PSD',/separator

self->RegisterOperation, 'About DAVE', 'DAVEopHelpAbout', $
  IDENTIFIER='Help/About DAVE',/separator




; self->RegisterOperation, IDLitLangCatQuery('Menu:Help:HelponiTools'), $
;   'IDLitopHelpiTools', $
;   ACCELERATOR='F1', $
;   IDENTIFIER='Help/HelpiTools'
;
; self->RegisterOperation, $
;   IDLitLangCatQuery('Menu:Help:HelpontheiToolsDataManager'), $
;   'IDLitopHelpDataManager', IDENTIFIER='Help/HelpDataManager'
;
; self->RegisterOperation, $
;   IDLitLangCatQuery('Menu:Help:HelpontheiToolsParameterEditor'), $
;   'IDLitopHelpParamEditor', IDENTIFIER='Help/HelpParamEditor'
;
; self->RegisterOperation, $
;   IDLitLangCatQuery('Menu:Help:HelponSelectedItem'), $
;   'IDLitopHelpSelection', $
;   IDENTIFIER='Help/HelpSelection', /SEPARATOR
;
; self->RegisterOperation, IDLitLangCatQuery('Menu:Help:HelponthisiTool'), $
;   'IDLitopHelpTool', $
;   IDENTIFIER='Help/HelpTool'
;
;self->RegisterOperation, IDLitLangCatQuery('Menu:Help:AboutiTools'), $
;  'IDLitopHelpAbout', $
;  IDENTIFIER='Help/About iTools'


;;---------------------------------------------------------------------
;;*** Manipulators
self->RegisterManipulator, 'Arrow', 'IDLitManipArrow', $
  ICON='arrow', /DEFAULT, IDENTIFIER="ARROW", $
  DESCRIPTION=IDLitLangCatQuery('Status:Framework:Select')

self->RegisterManipulator, 'Rotate', 'IDLitManipRotate', $
  ICON='rotate', IDENTIFIER="ROTATE", $
  DESCRIPTION=IDLitLangCatQuery('Status:Framework:Rotate')

self->RegisterManipulator, 'View Pan', 'IDLitManipViewPan', $
  ICON='hand', IDENTIFIER="VIEWPAN", $
  DESCRIPTION=IDLitLangCatQuery('Status:Framework:Pan')


;;---------------------------------------------------------------------
;;*** View Zoom Manipulator
self->RegisterManipulator, 'View Zoom', 'IDLitManipViewZoom', $
  IDENTIFIER='View/ViewZoom', ICON='zoom', $
  DESCRIPTION=IDLitLangCatQuery('Status:Framework:ViewZoom')

;;---------------------------------------------------------------------
;; *** View Zoom Combobox

                                ; Combobox is not available on True64 (OSF Alpha)
useCombobox = ~(!VERSION.os eq 'OSF')
self->Register, 'View Zoom', 'IDLitopViewZoom', $
                IDENTIFIER='Toolbar/View/ViewZoom', $
                DROPLIST_EDIT=useCombobox, $
                DROPLIST_ITEMS=['800%', $
                                '400%', $
                                '200%', $
                                '150%', $
                                '130%', $
                                '100%', $
                                '75%',  $
                                '50%',  $
                                '25%'], $
                DROPLIST_INDEX=4, $
                /SINGLETON

;;---------------------------------------------------------------------
;;*** Annotation Manipulators
self->RegisterManipulator, 'Text', 'IDLitAnnotateText', $
  ICON='text', IDENTIFIER="ANNOTATION/TEXT", $
  DESCRIPTION=IDLitLangCatQuery('Status:Framework:AnnotateText')

self->RegisterManipulator, 'Line', 'IDLitAnnotateLine', $
  ICON='line', IDENTIFIER="ANNOTATION/LINE", $
  DESCRIPTION=IDLitLangCatQuery('Status:Framework:AnnotateLine')

self->RegisterManipulator, 'Rectangle', 'IDLitAnnotateRectangle', $
  ICON='rectangl', IDENTIFIER="ANNOTATION/RECTANGLE", $
  DESCRIPTION=IDLitLangCatQuery('Status:Framework:AnnotateRectangle')

self->RegisterManipulator, 'Oval', 'IDLitAnnotateOval', $
  ICON='ellipse', IDENTIFIER="ANNOTATION/OVAL", $
  DESCRIPTION=IDLitLangCatQuery('Status:Framework:AnnotateOval')

self->RegisterManipulator, 'Polygon', 'IDLitAnnotatePolygon', $
  ICON='segpoly', IDENTIFIER="ANNOTATION/POLYGON", $
  DESCRIPTION=IDLitLangCatQuery('Status:Framework:AnnotatePolygon')

self->RegisterManipulator, 'Freehand', 'IDLitAnnotateFreehand', $
  ICON='freehand', IDENTIFIER="ANNOTATION/FREEHAND", $
  DESCRIPTION='Click & drag to draw'



;;---------------------------------------------------------------------
;;*** DrawContext
;self->Register, 'Cut', $
;                PROXY="/REGISTRY/OPERATIONS/CUT",$
;                IDENTIFIER='ContextMenu/DrawContext/Cut'
;
;self->Register, 'Copy', $
;                PROXY='/REGISTRY/OPERATIONS/COPY', $
;                IDENTIFIER='ContextMenu/DrawContext/Copy'
;
;self->Register, 'Paste', $
;                PROXY='/REGISTRY/OPERATIONS/Paste', $
;                IDENTIFIER='ContextMenu/DrawContext/Paste'
;
;                                ;-----------------
self->Register, 'Delete', $
                PROXY='/REGISTRY/OPERATIONS/Delete', $
                IDENTIFIER='ContextMenu/DrawContext/Delete'

                                ;-----------------
self->Register, 'Group', $
                PROXY='Operations/Edit/Grouping/Group', $
                IDENTIFIER='ContextMenu/DrawContext/Grouping/Group'

self->Register, 'Ungroup', $
                PROXY='Operations/Edit/Grouping/Ungroup', $
                IDENTIFIER='ContextMenu/DrawContext/Grouping/Ungroup'
;
;self->Register, 'Bring To Front', $
;                PROXY='Operations/Edit/Order/BringToFront', $
;                IDENTIFIER='ContextMenu/DrawContext/Order/BringToFront'
;
;self->Register, 'Send To Back', $
;                PROXY='Operations/Edit/Order/SendToBack', $
;                IDENTIFIER='ContextMenu/DrawContext/Order/SendToBack'
;
;self->Register, 'Bring Forward', $
;                PROXY='Operations/Edit/Order/BringForward', $
;                IDENTIFIER='ContextMenu/DrawContext/Order/BringForward'
;
;self->Register, 'Send Backward', $
;                PROXY='Operations/Edit/Order/SendBackward', $
;                IDENTIFIER='ContextMenu/DrawContext/Order/SendBackward'
;
;                                ;-----------------
;self->Register, 'Export to IDL...', $
;                PROXY='Operations/Edit/CLExport', $
;                IDENTIFIER='ContextMenu/DrawContext/CLExport'

                                ;-----------------
;self->Register, 'Parameters...', $
;                PROXY='Operations/Edit/EditParameters', $
;                IDENTIFIER='ContextMenu/DrawContext/EditParameters'

;self->Register, 'Save Graph Properties...', $
;                PROXY='Operations/Edit/SaveProperties', $
;                IDENTIFIER='ContextMenu/DrawContext/SaveProperties'

self->Register, 'Modify Properties...', $
                PROXY='Operations/Edit/Properties', $
                IDENTIFIER='ContextMenu/DrawContext/Properties'

;;---------------------------------------------------------------------
;;*** CropContext
;self->Register, 'Crop...', 'IDLitopCropImage', $
;                DESCRIPTION='Crop the selected image', $
;                IDENTIFIER='ContextMenu/CropDrawContext/Crop', $
;                ICON='crop', $
;                /SINGLETON

Self.sampContRef = obj_new('IDL_Container') ;obj_new('IDLitContainer',name='SampContainer')
Self.bkgdContRef = obj_new('IDL_Container') ;obj_new('IDLitContainer',name='BkgdContainer')

;; Initialise data and Register Properties

Self.sampTypeFlag = 1
sampTyps = ['Powder','Single Crystal','Raw Data']
Self.sampTyps = ptr_new(sampTyps)
Self->RegisterProperty, 'sampTypeFlag', enumlist=sampTyps, name='Sample Type' $
                      ,description='Sample Type'
hideIfPowder = (Self.sampTypeFlag eq 0)? 1 : 0
;hideIfNotPowder = (Self.sampTypeFlag eq 0)? 1 : 0

Self.calibParamsBaseFilename = ''
Self->RegisterProperty, 'calibParamsBaseFilename', userdef='', name='PSD Calib Parameter file' $
                      ,description='Channel effieciency file'
;Self.chEffbaseFilename = ''
;Self->RegisterProperty, 'chEfFbaseFilename', userdef='', name='Calib: PSD Eff. file' $
;                      ,description='Channel effieciency file'
;
;Self.chDelA4baseFilename = ''
;Self->RegisterProperty, 'chDelA4baseFilename', userdef='', name='Calib: PSD A4 file' $
;                      ,description='Channel A4 Spacing file'
;
;Self.chDelEfbaseFilename = ''
;Self->RegisterProperty, 'chDelEfbaseFilename', userdef='', name='Calib: PSD Ef file' $
;                      ,description='Channel final energy file'

Self.effNormFlag = 1
Self->RegisterProperty, 'effNormFlag', enumlist=['No','Yes'], name='Apply Channel Eff Corr?' $
                      ,description='Should channel effieciency correction be applied?', sensitive=0


Self.nSamp = 0
Self.nBkgd = 0

Self.sumSampFlag = 0
Self->RegisterProperty, 'sumSampFlag', enumlist=['No','Yes'], name='Sum Sample Data?' $
                      ,description='Sum all loaded Sample data?', sensitive=Self.nSamp, hide=1

Self.sumBkgdFlag = 0
Self->RegisterProperty, 'sumBkgdFlag', enumlist=['No','Yes'], name='Sum Bkgd Data?' $
                      ,description='Sum all loaded Bkgd data?', sensitive=Self.nBkgd, hide=1

Self.indepVars1Ptr = ptr_new(['QX','QY','E','|Q|','TEMP','MAGFIELD','PSDChannel'])
Self.exportIndex1Ptr = ptr_new([1,1,0,0,0,0,0])  ; used when exporting to Mslice
Self.indepVars2Ptr = ptr_new(['2Theta','d-space','|Q|','E','TEMP','MAGFIELD','Data_Index'])
Self.exportIndex2Ptr = ptr_new([1,0,0,0,1,0,0])  ; used when exporting to Mslice
Self.indepVars3Ptr = ptr_new(['Data_Index','A1','A2','A3','A4','A5','A6','H','K','L','E','TEMP','MAGFIELD'])
Self.exportIndex3Ptr = ptr_new([0,0,0,1,1,0,0,0,0,0,0,0,0])  ; used when exporting to Mslice
Self.rawModesPtr = ptr_new(['Counts vs Channel','Integrated Cts vs Channel','Integrated Cts vs ScanVariable'])

if (Self.sampTypeFlag eq 1) then begin
   ; sxtal
   xindVars = (*Self.indepVars1Ptr)
   yindVars = xindVars
   xname = 'Horizontal Axis Variable'
   yname = 'Vertical Axis Variable'
   xAxisVar = 0
   yAxisVar = 1
endif else if (Self.sampTypeFlag eq 0) then begin
   ; powder
   xindVars = (*Self.indepVars2Ptr)
   yindVars = xindVars
   xname = 'Scan Variable'
   yname = 'Vertical axis variable'
   xAxisVar = 0
   yAxisVar = 0
endif else begin
   ; raw 
   xindVars = (*Self.rawModesPtr)
   yindVars = (*Self.indepVars3Ptr)
   xname = 'Plot Quantity'
   yname = 'Scan Variable'
   xAxisVar = 0
   yAxisVar = 0
endelse

Self.xAxisVar = xAxisVar
Self->RegisterProperty, 'xAxisVar', enumlist=xindVars, name=xname, description='X-Axis Variable'
Self.yAxisVar = yAxisVar
Self->RegisterProperty, 'yAxisVar', enumlist=xindVars, name=yname, description='Y-Axis Variable' $
                      ,hide=hideifpowder


Self.sharedSettingsFlag = 1
Self->RegisterProperty, 'sharedSettingsFlag', enumlist=['No','Yes'], name='Datasets share same settings?'
Self.settingIndex = 0
Self->RegisterProperty, 'settingIndex', enumlist=[''], name='Dataset name'

Self.dpntIndex=1
Self->RegisterProperty, 'dpntIndex', /integer, name='Data Point to view';, valid_range[1,1,1]
Self.minChan=1
Self->RegisterProperty, 'minChan', /integer, name='Min Channel (to sum)';, valid_range[1,48,1]
Self.maxChan=48
Self->RegisterProperty, 'maxChan', /integer, name='Max Channel (to sum)';, valid_range[1,48,1]
Self.minDPnt=1
Self->RegisterProperty, 'minDPnt', /integer, name='Min Data Point (to sum)';, valid_range[1,1,1]
Self.maxDPnt=1
Self->RegisterProperty, 'maxDPnt', /integer, name='Max Data Point (to sum)';, valid_range[1,1,1]

Self.minBinWidth=0.01
Self->RegisterProperty, 'minBinWidth', /float, name='Min Bin Width (scan var)' $
                      ,description='Min Bin Width (scan var)' 
Self.constBinFlag = 0
Self->RegisterProperty, 'constBinFlag', enumlist=['No','Yes'], name='Enforce constant bin widths?'
Self.useTempAverageFlag = 0
Self->RegisterProperty, 'useTempAverageFlag', enumlist=['No','Yes'], name='Use the average scan temp?'


Self.qUnits = 1
Self->RegisterProperty, 'qUnits', enumlist=['Inverse Angstroms','Relative lattice units'], name='Units of QX, QY' $
                      ,description='Units to use for QX, QY',hide=hideifpowder

Self.logFlag = 0
Self->RegisterProperty, 'logFlag', enumlist=['Linear','Logbase10'], name='Intensity Scale' $
                      ,description='Intensity Scale' ,hide=hideifpowder

Self.defaultIntensityFlag = 1
Self->RegisterProperty, 'defaultIntensityFlag', enumlist=['No','Yes'], name='Use Default Intensity Range?' $
                      ,description='Use Default Intensity Range?' ,hide=hideifpowder
Self.maxIntensity = 0.0
Self->RegisterProperty, 'maxIntensity', /float, name='Max Intensity Value' $
                      ,description='Max Intensity Value' ,hide=hideifpowder
Self.minIntensity = 0.0
Self->RegisterProperty, 'minIntensity', /float, name='Min Intensity Value' $
                      ,description='Min Intensity Value' ,hide=hideifpowder

Self.latParamFlag = 0
Self->RegisterProperty, 'latParamFlag', enumlist=['From Data','User Specified'], name='Latt Param source' $
                      ,description='Lattice Parameter flag' ,hide=hideifpowder
Self.latticeParameters = ''
Self->RegisterProperty, 'latticeParameters', /string,name='Latt Param: a b c al be ga' $
                      ,description='Lattice Parameters' $
                      ,sensitive=Self.latParamFlag,hide=hideifpowder

Self.phiOffset = 0.0
Self->RegisterProperty, 'phiOffset', /float, name='A3 Correction (Phi Offset)' $
                      ,description='A3 Correction (Phi Offset)' ,hide=hideifpowder

Self.maskFlag = 0
Self->RegisterProperty, 'maskFlag', enumlist=['No','Yes'], name='Remove Masked Channels?' $
                      ,description='Remove Masked Channels' 
Self.maskedDets = ''
Self->RegisterProperty, 'maskedDets', userdef='',name='Channels to Mask' $
                      ,description='Channels to be masked' ,sensitive=Self.maskFlag

;Self.mergeChannelsFlag = 0
;Self->RegisterProperty, 'mergeChannelsFlag', enumlist=['No','Yes'], name='Merge PSD Channels?' $
;                      ,description='Should PSD channels be combined?' $
;                      ,hide=~hideIfPowder

Self.normalizeTo = 0
Self->RegisterProperty, 'normalizeTo', enumlist=['Monitor','Counting Time','No Normalization'], name='Normalize To' $
                      ,description='Normalize Counts To'

Self.monScaleValue = 1.0
Self->RegisterProperty, 'monScaleValue', /float, name='Overall Scale Factor' $
                      ,description='Overall Scale Factor'

Self.bkgdFlag = 0
Self->RegisterProperty, 'bkgdFlag', enumlist=['No','Yes'], name='Subtract Bkgd data?' $
                      ,description='Should bkgd data be subtracted' $
                      ,sensitive=(Self.nBkgd gt 0)? 1 : 0
Self.bkgdSubVars = ['Not Applicable','A1','A2','A3','A4','A5','A6','H','K','L','HKL','E','Ei','Ef','Temp','Magfield']
Self->RegisterProperty, 'bkgdSubVarFlag', enumlist=Self.bkgdSubVars, name='Bkgd Sub: Matching Var' $
                      ,description='Matching variable for paring signal and bkgd datasets' $
                      ,sensitive=(Self.bkgdFlag gt 0)? 1 : 0
Self.bkgdSubVarTol = 0.005
Self->RegisterProperty, 'bkgdSubVarTol', /float, name='Bkgd Sub: Matching Var Tol' $
                      ,description='Tolerance threshold for bkgd subtraction ketword variable' $
                      ,sensitive=(Self.bkgdFlag gt 0)? 1 : 0

Self.fastFlag = 0
Self->RegisterProperty, 'fastFlag', enumlist=['No','Yes'], name='Fast Bkgd Corr.?' $
                      ,description='Should fast bkgd correction be applied?'
Self.fastValue = 0.0
Self->RegisterProperty, 'fastValue', /float, name='Fast Bkgd (cts/min)' $
                      ,description='Fast Background Level', sensitive=Self.fastFlag

Self.monCorFlag = 0
Self->RegisterProperty, 'monCorFlag', enumlist=['No','Yes'], name='Monitor Correction?' $
                      ,description='Should monitor correction be applied?' $
                      ,sensitive=Self.fastFlag

Self.resCorFlag = 0
Self->RegisterProperty, 'resCorFlag', enumlist=['No','Yes'], name='Resolution Correction?' $
                      ,description='Should resolution volume correction be applied?' $
                      ,sensitive=0

Self.detailedBalanceFlag = 0
Self->RegisterProperty, 'detailedBalanceFlag', enumlist=['No','Yes'], name='Det. balance Correction?' $
                      ,description='Should detailed balance correction be applied?' $
                      ,sensitive=Self.fastFlag,hide=1
                      
Self.multipleMsliceWinFlag = 0
Self->RegisterProperty, 'multipleMsliceWinFlag', enumlist=['No','Yes'], name='Allow multiple Mslice?' $
                      ,description='Should multiple Mslice be allowed?'

Self->SetPropertyAttribute, 'NAME', /hide
Self->SetPropertyAttribute, 'DESCRIPTION', /hide
Self.projName = 'Untitled'

;; Finally, set the prompt property to 1 meaning display a warning dialog 
;; before deleting this tool
Self.prompt = 1
Self.promptDesc = 'BT7PSD Reduction Window'
Self.promptTitle = 'Delete Reduction Window?'

Self.ftpObject = DAVEftpURL()
Self.ftpObject->SetProperty, currentDir = '/pub/ncnrdata/bt7/'

Self.historyPtr = ptr_new('')

sxtalOnlyProps = ['latticeParameters','latParamFlag','qUnits','yAxisVar']
sxtalOnlyProps = [sxtalOnlyProps,'defaultIntensityFlag','maxIntensity','minIntensity','phiOffset','logFlag']
Self.sxtalOnlyPropsPtr = ptr_new(sxtalOnlyProps)
Self.powderOnlyPropsPtr = ptr_new(['minBinWidth','useTempAverageFlag','constBinFlag'])
Self.sxtalPowderPropsPtr = ptr_new(['fastFlag','fastValue','monCorFlag','resCorFlag','multipleMsliceWinFlag']) ;'maskFlag','maskedDets',
Self.rawOnlyPropsPtr = ptr_new(['minChan','maxChan','minDPnt','maxDPnt','dpntIndex','sharedSettingsFlag','settingIndex'])

; PSD detector calibration init
Self.chDelA4 = 0.0  ; all channel a4 delta to 0
Self.chDelEf = 0.0  ; all channel Ef delta to 0
Self.chEff = 1.0    ; all channel efficiency to 1.0

Self.ekFac = 2.072194

return, 1

end


;-------------------------------------------------------------------------
; Bt7psdTool::SaveVisProps
;+
; Purpose:
;    Retrieve the curent vis properties and save them
;
; Parameters:
;    oVis     - The visualization whose properties are to be saved
;    
;-
pro Bt7psdTool::SaveVisProps, oVis
compile_opt idl2

if (~obj_valid(oVis)) then return
if (~obj_isa(oVis,'IDLitVisualization')) then return

oUI = Self->GetUI()
oUI->GetProperty, group_leader=wTLB

; Basic error Handler
if (n_elements(!debug) && (!debug eq 0)) then begin
    catch, catchError
    if (catchError ne 0) then begin
        ;;print, 'Error handled!'
        eTitle = 'Bt7psdTool::SaveVisProps: Error encountered'
        eMsg = 'An error or unusual condition was encountered!'
        eMsg = [eMsg,'Please, report the following to the DAVE team:']
        eMsg = [eMsg,!error_state.msg]
        void = dialog_message(/error,eMsg,title=eTitle,dialog_parent=wTLB)
        catch, /cancel
        return
    endif
endif

oVis->GetProperty, description=plotDesc
case strupcase(plotDesc) of
   'SAMPPLOT': prefix = 's_'
   'BKGDPLOT': prefix = 'b_'
;   'FASTPLOT': prefix = 'f_'
;   'SAMPBKGDPLOT': prefix = 'sb_'
;   'FASTFITPLOT': prefix = 'ff_'
   else: return
endcase

switch strupcase(plotDesc) of
   'SAMPPLOT':
   'BKGDPLOT':
   'FASTPLOT': begin
      ;; store the vis props
      vTags = ['use_default_color','color','errorbar_color','sym_color','errorbar_capsize' $
              ,'sym_increment','sym_index','sym_size','sym_thick','y_errorbars','antialias' $
              ,'linestyle','nsum','thick','sym_filled','sym_fill_color','transparency']
;      vTags = ['errorbar_capsize','transparency','antialias','color','use_default_color' $
;              ,'sym_increment','sym_index','sym_size','sym_color','sym_thick','y_errorbars' $
;              ,'linestyle','nsum','thick','sym_filled','sym_fill_color']
      tTags = prefix + vTags

      nTags = n_elements(vTags)
      etc = {dummy:''}
      for i=0, nTags-1 do begin
         status = oVis->GetPropertyByIdentifier(vTags[i], value)
         if (status) then etc = create_struct(etc,tTags[i],value)
      endfor
      Self->SetProperty, _EXTRA=etc

      ;; store the axes props
      vTags = ['font_name','font_size','text_color']
      tTags = prefix + vTags
      nTags = n_elements(vTags)
      oDataSpace = oVis->GetDataSpace()
      oAxes = oDataSpace->GetAxes(count=naxes)
      etc = {dummy:''}
      for i=0, nTags-1 do begin
         status = oAxes[0]->GetPropertyByIdentifier(vTags[i], value)
         if (status) then etc = create_struct(etc,tTags[i],value)
      endfor
      Self->SetProperty, _EXTRA=etc   
   
      break
   end
   
;   'SAMPBKGDPLOT':
;   'FASTFITPLOT': begin
;      ;; Set the vis props
;      vTags = ['transparency','antialias','color','use_default_color','sym_index' $
;              ,'linestyle','thick']
;      tTags = prefix + vTags
;
;      nTags = n_elements(vTags)
;      etc = {dummy:''}
;      for i=0, nTags-1 do begin
;         status = oVis->GetPropertyByIdentifier(vTags[i], value)
;         if (status) then etc = create_struct(etc,tTags[i],value)
;      endfor
;      Self->SetProperty, _EXTRA=etc
;   
;      break
;   end
   
   else:
endswitch

end




;-------------------------------------------------------------------------
; Bt7psdTool::customizeVis
;+
; Purpose:
;    Customize a newly created visualization using attributes obtained from the tool
;
; Parameters:
;    oVis     - The visualization to be customized
;    
;    plotname - a string name to be used to identify the visualization
;-
pro Bt7psdTool::CustomizeVis, oVis, plotName, plotDesc
compile_opt idl2

if (~obj_valid(oVis)) then return
if (~obj_isa(oVis,'IDLitVisualization')) then return

oUI = Self->GetUI()
oUI->GetProperty, group_leader=wTLB

; Basic error Handler
if (n_elements(!debug) && (!debug eq 0)) then begin
    catch, catchError
    if (catchError ne 0) then begin
        ;;print, 'Error handled!'
        eTitle = 'wd_Bt7psdTool_customizeVis: Error encountered'
        eMsg = 'An error or unusual condition was encountered!'
        eMsg = [eMsg,'Please, report the following to the DAVE team:']
        eMsg = [eMsg,!error_state.msg]
        void = dialog_message(/error,eMsg,title=eTitle,dialog_parent=wTLB)
        catch, /cancel
        return
    endif
endif

case strupcase(plotDesc) of
   'SAMPPLOT': prefix = 's_'
   'BKGDPLOT': prefix = 'b_'
;   'FASTPLOT': prefix = 'f_'
;   'SAMPBKGDPLOT': prefix = 'sb_'
;   'FASTFITPLOT': prefix = 'ff_'
   else: return
endcase

switch strupcase(plotDesc) of
   'SAMPPLOT':
   'BKGDPLOT':
   'FASTPLOT': begin
      ;; Set the vis props
      vTags = ['use_default_color','color','errorbar_color','sym_color','errorbar_capsize' $
              ,'sym_increment','sym_index','sym_size','sym_thick','y_errorbars','antialias' $
              ,'linestyle','nsum','thick','sym_filled','sym_fill_color','transparency']
      tTags = prefix + vTags

      nTags = n_elements(vTags)
      etc = {dummy:''}
      for i=0, nTags-1 do begin
         status = Self->GetPropertyByIdentifier(tTags[i], value)
         if (status) then etc = create_struct(etc,vTags[i],value)
      endfor

      if (float((strtok(!version.release,/extract))[0]) lt 8.0) then begin
         ; workaround for bug in IDL 7.1 or lower; use_default_color is not honoured for errorbar_color
         if (etc.use_default_color eq 1) then etc.errorbar_color = etc.color
      endif

      oVis->SetProperty, name=strupcase(plotName)
      oVis->SetProperty, name=strupcase(plotName), description=plotDesc, _EXTRA=etc
      oVis->SetPropertyAttribute, 'NSUM', sensitive=0   ; disable point averaging/smoothing b/c errors are not taken into account

      ;; Set the axes props
      vTags = ['font_name','font_size','text_color']
      tTags = prefix + vTags
      nTags = n_elements(vTags)
      oDataSpace = oVis->GetDataSpace()
      oAxes = oDataSpace->GetAxes(count=naxes)
      etc = {dummy:''}
      for i=0, nTags-1 do begin
         status = Self->GetPropertyByIdentifier(tTags[i], value)
         if (status) then etc = create_struct(etc,vTags[i],value)
      endfor
      for i=0,naxes-1 do oAxes[i]->SetProperty, _EXTRA=etc   
   
      break
   end
   
;   'SAMPBKGDPLOT':
;   'FASTFITPLOT': begin
;      ;; Set the vis props
;      vTags = ['transparency','antialias','color','use_default_color','sym_index' $
;              ,'linestyle','thick']
;      tTags = prefix + vTags
;
;      nTags = n_elements(vTags)
;      etc = {dummy:''}
;      for i=0, nTags-1 do begin
;         status = Self->GetPropertyByIdentifier(tTags[i], value)
;         if (status) then etc = create_struct(etc,vTags[i],value)
;      endfor
;      oVis->SetProperty, name=strupcase(plotname), _EXTRA=etc
;      oVis->SetPropertyAttribute, 'NSUM', sensitive=0   ; disable point averaging/smoothing b/c errors are not taken into account
;   
;      break
;   end
   
   else:
endswitch



end



;---------------------------------------------------------------------------
; Bt7psdTool::InitPreferences
;
; Purpose:
;   This method
;
pro Bt7psdTool::InitPreferences
compile_opt idl2

; Init working and data directory from the DAVETool values
;daveTool = Self.daveTool
;if (obj_valid(daveTool) and obj_isa(daveTool,'DAVETool')) then begin
;   daveTool->GetProperty, data_directory=datDir, working_directory=workDir
;   Self->SetProperty, data_directory=datDir, working_directory=workDir
;endif


; Retrieve user settings on disk and initialise current session
status = BT7PSDReducPreferences(preferences)
if (status) then Self->SetProperty, preferences=preferences

Self->GetProperty, maskFlag=mF, fastFlag=fF, xAxisVar=xAxisVar $
                 , binWidths=binWidths, defaultIntensityFlag=defIntFlag, sampTypeFlag=sampTypeFlag
Self->SetProperty, minBinWidth = binwidths[xAxisVar]
noHide = (defIntFlag eq 0) && (sampTypeFlag eq 1)  ; don't hide if not using default intensity and single crystal sample
Self->SetPropertyAttribute, 'maxIntensity', hide=~noHide
Self->SetPropertyAttribute, 'minIntensity', hide=~noHide
Self->SetPropertyAttribute, 'MASKEDDETS', sensitive=mF
Self->SetPropertyAttribute, 'FASTVALUE', sensitive=fF
Self->SetPropertyAttribute, 'MONCORFLAG', sensitive=fF
;Self->SetPropertyAttribute, 'RESCORFLAG', sensitive=mF
;Self->SetPropertyAttribute, 'DETAILEDBALANCEFLAG', sensitive=mF

; For safety, always initialise phiOffset to 0.0 b/c a user may not be fully aware
; about this setting. A value of 0.0 is harmless
Self->SetProperty, phiOffset = 0.0

Self->refreshPropertySheet

; Update title to reflect session name
; setting tool_filename generates a callback on the registered UI callback
Self->SetProperty, tool_filename=Self.projName

; Retrieve the graphics window
; and set auto-resize and zoom-on-resize to 1 so the graphics window behaves properly when its size is changed
oWin = Self->GetCurrentWindow()
oWin->SetProperty, auto_resize=1, zoom_on_resize=1

;if (float((strtok(!version.release,/extract))[0]) ge 8.0) then begin
;   ; For IDL 8.0 and newer, a title text is added to the graphics by default
;   ; Hide it!
;
;   idText = iGetID('*title*',tool=Self)
;   oText = Self->GetByIdentifier(idText[0])
;   if (isa(oText,'IDLitvisText')) then oText->setProperty, hide=1
;   Self->RefreshCurrentWindow
;endif

end


;---------------------------------------------------------------------------
; Bt7psdTool::resetPreferences
;
; Purpose:
;   This method
;
function Bt7psdTool::ResetPreferences
compile_opt idl2


; Retrieve user settings on disk and initialise current session
status = BT7PSDReducPreferences(preferences,/reset)
if (status) then Self->SetProperty, preferences=preferences

Self->GetProperty, maskFlag=mF, fastFlag=fF

Self->SetPropertyAttribute, 'MASKEDDETS', sensitive=mF
Self->SetPropertyAttribute, 'FASTVALUE', sensitive=fF
Self->SetPropertyAttribute, 'MONCORFLAG', sensitive=fF
;Self->SetPropertyAttribute, 'RESCORFLAG', sensitive=mF
;Self->SetPropertyAttribute, 'DETAILEDBALANCEFLAG', sensitive=mF

Self->refreshPropertySheet

end


;---------------------------------------------------------------------------
; Bt7psdTool::EditUserDefProperty
;
; Purpose:
;   This method
;
function Bt7psdTool::EditUserDefProperty, oTool, PropID
compile_opt idl2

;; The API requires the oTool arg. It is redundant here since it is Self!

oUI = Self->GetUI()
oUI->GetProperty, group_leader=wTLB
wChild = widget_info(wTLB, /child)
widget_control, wChild, get_uvalue=sPtr

case PropID of
  'CALIBPARAMSBASEFILENAME': begin
    status = Self->loadCalibParams()
  end

;  'CHDELEFBASEFILENAME': begin
;    status = Self->loadChEf()
;    break
;  end
;
;  'CHEFFBASEFILENAME': begin
;    status = Self->loadChEfficiency()
;    break
;  end

  'MASKEDDETS': begin
     ; The currently selected dataset is registered as (*sPtr).itemID
     noSelection = strcmp(strtrim((*sPtr).itemID),'')
     if (noselection) then begin
       ; retrieve the first loaded dataset, if any
       Self->GetProperty, sampContRef=oCont
       oItem = oCont->Get(position=0,count=nItem)
     endif else begin
       oItem = Self->GetByIdentifier((*sPtr).itemID)
       nItem = 1
     endelse
     if (nItem ne 1) then return, 0
     
     wd_BT7PSDMaskChannels, oUI, oItem, Self.calibParamsIsSet, Self.chEff
  end
   
  else:
endcase

Self->refreshPropertySheet

return, 1
end


;---------------------------------------------------------------------------
; Bt7psdTool__Define
;
; Purpose:
;   This method defines the Bt7psdTool class.
;
pro Bt7psdTool__Define
compile_opt idl2

bkgdSubVars = ['Not Applicable','A1','A2','A3','A4','A5','A6','H','K','L','HKL','E','Ei','Ef','Temp','Magfield']
void = { Bt7psdTool,                 $
         inherits IDLitTool        $   ; Provides iTool interface
         ,dataDir:''               $   ; data directory (Note: working_directory is defined by the base class
         ,nSamp:0                  $   ; nos of valid Sample datasets
         ,nBkgd:0                  $   ; nos of valid bkgd datasets
;         ,nFast:0                  $   ; nof of valid fast bkgd datasets
         ,sampContRef:obj_new()    $   ; container for Sample datasets
         ,bkgdContRef:obj_new()    $   ; container for bkgd datasets
;         ,fastContRef:obj_new()    $   ; container for fast bkgd datasets
         ,sampSelected:obj_new()   $   ; most recent selection from samp datasets
         ,bkgdSelected:obj_new()   $   ; most recent selection from bkgd datasets
;         ,fastSelected:obj_new()   $   ; most recent selection from fast datasets
         ,oPlotBkgdFlag:0          $
         ,DAVETool:obj_new()       $   ; DAVETool object
         ,projName:''              $   ; name of current project
         ,nameTag:''               $   ; Tag to be used for identification purposes
         ,prompt:1                 $
         ,promptTitle:''           $
         ,promptDesc:''            $
;         ,mev2wnos:8.065541        $   ; meV to wavenumber conversion factor
;         ,nActiveDetectors:50      $   ; nos of active detectors
;         ,energyUnitFlag:0         $   ; energy unit flag 0=meV, 1=cm-1
         ,sumSampFlag:0            $   ; sum multiple sample files? 0=no, 1=yes
         ,sumBkgdFlag:0            $   ; sum multiple bkgd files? 0=no, 1=yes
         ,maskFlag:0               $
         ,maskedDets:''            $   ; masked detectors (stored as a string)
;         ,fastFuncs:strarr(4)      $   ; available functions to fit fast bkgd
;         ,fastFunc:0               $   ; function to be used to fit fast bkgd 0=arcsin, 1=linear, 2=quadratic, 3=cubic
         ,latParamFlag:0           $   ; Use lattice parameter from: 0:raw data file 1:user specified
         ,qUnits:0                 $   ; Units for QX, QY variables: 0->1/A, 1->r.l.u
         ,xAxisVar:0               $   ; Variable to plotted as the x-axis
         ,yAxisVar:0               $   ; variable to be plotted as the y-axis
         ,indepVars1Ptr:ptr_new()  $   ; list of valid sxtal independent variables ('QX','QY','E','|Q|','TEMP','MAGFIELD')
         ,indepVars2Ptr:ptr_new()  $   ; list of valid powder x-axis independent variables ('2THETA','Q','DSPACE','TEMP','MAGFIELD','DATAPOINT')
         ,indepVars3Ptr:ptr_new()  $   ; list of valid powder y-axis independent variables ('2THETA','Q','DSPACE')
         ,exportIndex1Ptr:ptr_new() $   ; index specifying items in indepVars1Ptr to be exported
         ,exportIndex2Ptr:ptr_new() $   ; index specifying items in indepVars2Ptr to be exported
         ,exportIndex3Ptr:ptr_new() $   ; index specifying items in indepVars3Ptr to be exported
         ,rawModesPtr:ptr_new()    $   ; list of possible viewing modes when Samp type is set to 'Raw Data'
         ,latticeParameters:''     $   ; lattice parameters
         ,monScaleValue:1.0        $   ; user specified mon scale factor
         ,detailedBalanceFlag:0    $   ; apply detailed balance correction 1=yes, 0=no
         ,bkgdFlag:0               $   ; subtract bkgd data  1=yes, 0=no
         ,bkgdSubVarFlag:0        $   ; a key variable used to determine if a signal and bkgd dataset pair match for bkgd subtraction
         ,bkgdSubVars:bkgdSubVars   $   ; unum lists of vars that can be used for pairing signal ad bkgd datasets - used by bkgdSubVarsFlag
         ,bkgdSubVarTol:0.0       $   ; a tolerance threshold used to determine if two bkgdSubVar values match
         ,fastFlag:0               $   ; apply fast bkgd correction? 1=yes, 0=no
         ,fastValue:0.0            $   ; fast bkgd level in cnts/min
         ,resCorFlag:0             $   ; apply resolution volume correction ? 1=yes, 0=no
         ,monCorFlag:0             $   ; apply monitor correction (due to energy dependent change to incident flux)? 1=yes, 0=no
         ,normalizeTo:0            $   ; normalize counts to: 0: Monitor, 1:Counting Time, 2: Nothing
         ,sampTypeFlag:0           $   ; sample type: 1:single crystal, 0:powder
         ,sampTyps:ptr_new()      $   ; vector of sample types: [single crystal,powder,]
         ,prefsPtr:ptr_new()       $   ; points to preference structure
         ,historyPtr:ptr_new()     $
         ,modifiedFlag:0B          $   ; determines modification state of tool - unsufficient control over _bDirty property!
         ,dataVersion:0            $   ; current version number for dataset structure
         ,ftpObject:obj_new()      $   ; DAVEftpURL() object used to access data on ftp://ftp.ncnr.nist.gov/pub/ncnrdata/
         ,calibParamsFilename:''   $
         ,calibParamsBaseFilename:'' $
         ,calibParamsIsSet:0       $
         ,chDelA4:fltarr(48)       $   ; Channel A4 deltas
         ,chDelEf:fltarr(48)       $   ; Channel Ef deltas (or maybe just Channel Ef)
         ,chEff:fltarr(48)         $   ; Channel efficiencies
         ,effNormFlag:0            $
         ,minBinWidth:0.01         $   ; user specified current bin width
         ,binWidths:fltarr(7)      $   ; initial bin widths for various powder scan variables
         ,defaultIntensityFlag:0   $   ; use default intensity range 1 = yes, 0 = no
         ,maxIntensity:0.0         $
         ,minIntensity:0.0         $
         ,phiOffset:0.0            $   ; A3 Correction or Phi Offset
         ,logFlag:0                $   ; use linear (0) or log (1) intensity scale
;         ,lolim:0                  $   ; low limit of range to sum (raw mode)
;         ,hilim:0                  $   ; high limit of range to sum (raw mode)
         ,ConstBinFlag:0           $   ; Enforce constant bin width for powder data when rebinning
         ,useTempAverageFlag:0     $   ; when set, take the everage temperature of the scan as the temperature at each point in the scan
         ,minChan:0                $
         ,maxChan:0                $
         ,minDPnt:0                $
         ,maxDPnt:0                $
         ,dpntIndex:0              $
         ,pMinDPnt:ptr_new()       $
         ,pMaxDPnt:ptr_new()       $
         ,pDpntIndex:ptr_new()     $      
         ,sharedSettingsFlag:0     $
         ,settingIndex:0           $
;         ,rawPlotType:0            $
;         ,chDelA4isSet:0           $
;         ,chDelA4baseFilename:''   $
;         ,chDelA4fullFilename:''   $
;         ,chDelEfisSet:0           $
;         ,chDelEfbaseFilename:''   $
;         ,chDelEffullFilename:''   $
;         ,chEffisSet:0             $
;         ,chEffbaseFilename:''     $
;         ,chEfffullFilename:''     $
         ,ekFac:2.072194           $   ; Energy = ekFac * k^2
         ,objMslice:obj_new()      $   ; place holder for Mslice object
         ,multipleMsliceWinFlag:0  $   ; should multiple mslice windows be created?
         ,mergeChannelsFlag:0      $   ; (powder) whether (1) or not (0) to combine PSD channels
         ,sxtalOnlyPropsPtr:ptr_new() $   ;
         ,powderOnlyPropsPtr:ptr_new() $  ;
         ,sxtalPowderPropsPtr:ptr_new() $
         ,rawOnlyPropsPtr:ptr_new()    $
         ,s_errorbar_capsize:    0.1 $
         ,s_transparency:        50 $
         ,s_antialias:           3 $
         ,s_color:               [0,0,255] $
         ,s_errorbar_color:      [0,0,255] $
         ,s_use_default_color:   1   $
         ,s_sym_increment:       1   $
         ,s_sym_index:           4   $
         ,s_sym_size:            1.0   $          ; 
         ,s_sym_color:           [0,0,255] $   ;
         ,s_sym_thick:           1.0 $            ;
         ,s_y_errorbars:         1 $              ; show y errors
         ,s_linestyle:           0 $              ; 
         ,s_nsum:                1 $              ; no point everaging
         ,s_thick:               2.0 $            ; line thickness
         ,s_sym_filled:          1   $            ; use filled symbols
         ,s_sym_fill_color:      [0,0,255] $   ; symbol fill color
         ,s_font_name:           'Helvetica'  $   ;
         ,s_font_size:           13 $             ;
         ,s_text_color:          [0,0,0]  $    ;
;
         ,b_errorbar_capsize:    0.1 $
         ,b_transparency:        50 $
         ,b_antialias:           3 $
         ,b_color:               [0,255,255] $
         ,b_errorbar_color:      [0,255,255] $
         ,b_use_default_color:   1   $
         ,b_sym_increment:       1   $
         ,b_sym_index:           4   $
         ,b_sym_size:            1.0   $          ; 
         ,b_sym_color:           [0,255,255] $   ;
         ,b_sym_thick:           1.0 $            ;
         ,b_y_errorbars:         1 $              ; show y errors
         ,b_linestyle:           6 $              ; no line
         ,b_nsum:                1 $              ; no point everaging
         ,b_thick:               1.0 $            ; line thickness
         ,b_sym_filled:          1   $            ; use filled symbols
         ,b_sym_fill_color:      [0,255,255] $   ; symbol fill color
         ,b_font_name:           'Helvetica'  $   ;
         ,b_font_size:           13 $             ;
         ,b_text_color:          [0,0,0]  $    ;
;
;         ,f_errorbar_capsize:    0.1 $
;         ,f_transparency:        50 $
;         ,f_antialias:           3 $
;         ,f_color:               [0,0,255] $
;         ,f_errorbar_color:      [0,0,255] $
;         ,f_use_default_color:   1   $
;         ,f_sym_increment:       1   $
;         ,f_sym_index:           4   $
;         ,f_sym_size:            1.0   $          ; 
;         ,f_sym_color:           [0,0,255] $   ;
;         ,f_sym_thick:           1.0 $            ;
;         ,f_y_errorbars:         1 $              ; show y errors
;         ,f_linestyle:           6 $              ; no line
;         ,f_nsum:                1 $              ; no point everaging
;         ,f_thick:               1.0 $            ; line thickness
;         ,f_sym_filled:          1   $            ; use filled symbols
;         ,f_sym_fill_color:      [0,0,255] $   ; symbol fill color
;         ,f_font_name:           'Helvetica'  $   ;
;         ,f_font_size:           13 $             ;
;         ,f_text_color:          [0,0,0]  $    ;
;
;         ,sb_transparency:        60 $
;         ,sb_antialias:           3 $
;         ,sb_color:               [0,128,0] $
;         ,sb_use_default_color:   1   $
;         ,sb_sym_index:           0   $            ; no symbol
;         ,sb_linestyle:           0 $              ;
;         ,sb_thick:               5.0 $            ; line thickness
;
;         ,ff_transparency:        60 $
;         ,ff_antialias:           3 $
;         ,ff_color:               [0,128,0] $
;         ,ff_use_default_color:   1   $
;         ,ff_sym_index:           0   $            ; no symbol
;         ,ff_linestyle:           0 $              ;
;         ,ff_thick:               5.0 $            ; line thickness


       }
end
